@@ -19,12 +19,13 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: filebrowser.c 2444 2008-04-03 16:06:41Z ntrel $
+ * $Id: filebrowser.c 3085 2008-10-14 17:03:44Z ntrel $
  */
 
 /* Sidebar file browser plugin. */
 
 #include "geany.h"
+#include <string.h>
 
 #include <gdk/gdkkeysyms.h>
 
@@ -34,18 +35,19 @@
 #include "utils.h"
 #include "keybindings.h"
 #include "project.h"
+#include "ui_utils.h"
 
 #include "plugindata.h"
 #include "pluginmacros.h"
 
 
-PluginFields	*plugin_fields;
 GeanyData		*geany_data;
+GeanyFunctions	*geany_functions;
 
 
-PLUGIN_VERSION_CHECK(26)
+PLUGIN_VERSION_CHECK(69)
 
-PLUGIN_INFO(_("File Browser"), _("Adds a file browser tab to the sidebar."), VERSION,
+PLUGIN_SET_INFO(_("File Browser"), _("Adds a file browser tab to the sidebar."), VERSION,
 	_("The Geany developer team"))
 
 
@@ -60,13 +62,6 @@ enum
 PLUGIN_KEY_GROUP(file_browser, KB_COUNT)
 
 
-/* number of characters to skip the root of an absolute path("c:" or "d:" on Windows) */
-#ifdef G_OS_WIN32
-# define ROOT_OFFSET 2
-#else
-# define ROOT_OFFSET 0
-#endif
-
 enum
 {
 	FILEVIEW_COLUMN_ICON = 0,
@@ -83,10 +78,12 @@ static GtkListStore			*file_store;
 static GtkTreeIter			*last_dir_iter = NULL;
 static GtkEntryCompletion	*entry_completion = NULL;
 
+static GtkWidget	*filter_entry;
 static GtkWidget	*path_entry;
 static gchar		*current_dir = NULL;	/* in locale-encoding */
 static gchar		*open_cmd;				/* in locale-encoding */
 static gchar		*config_file;
+static gchar 		*filter = NULL;
 
 static struct
 {
@@ -128,6 +125,21 @@ static gboolean check_hidden(const gchar *base_name)
 }
 
 
+/* Returns: whether name has been removed by filter. */
+static gboolean check_filtered(const gchar *base_name)
+{
+	if (filter == NULL)
+		return FALSE;
+
+	if (! p_utils->str_equal(base_name, "*") && ! g_pattern_match_simple(filter, base_name))
+	{
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+
 /* name is in locale encoding */
 static void add_item(const gchar *name)
 {
@@ -138,6 +150,9 @@ static void add_item(const gchar *name)
 	if (! show_hidden_files && check_hidden(name))
 		return;
 
+	if (check_filtered(name))
+		return;
+
 	fname = g_strconcat(current_dir, G_DIR_SEPARATOR_S, name, NULL);
 	dir = g_file_test(fname, G_FILE_TEST_IS_DIR);
 	g_free(fname);
@@ -165,21 +180,13 @@ static void add_item(const gchar *name)
 }
 
 
-static gboolean is_top_level_directory(const gchar *dir)
-{
-	g_return_val_if_fail(dir && strlen(dir) > ROOT_OFFSET, FALSE);
-
-	return (p_utils->str_equal(dir + ROOT_OFFSET, G_DIR_SEPARATOR_S));
-}
-
-
 /* adds ".." to the start of the file list */
 static void add_top_level_entry(void)
 {
 	GtkTreeIter iter;
 
-	if (is_top_level_directory(current_dir))
-		return;
+	if (! NZV(g_path_skip_root(current_dir)))
+		return;	/* ignore 'C:\' or '/' */
 
 	gtk_list_store_prepend(file_store, &iter);
 	last_dir_iter = gtk_tree_iter_copy(&iter);
@@ -216,10 +223,11 @@ static void refresh(void)
 	gtk_entry_set_text(GTK_ENTRY(path_entry), utf8_dir);
 	g_free(utf8_dir);
 
+	add_top_level_entry();	/* ".." item */
+
 	list = p_utils->get_file_list(current_dir, NULL, NULL);
 	if (list != NULL)
 	{
-		add_top_level_entry();
 		g_slist_foreach(list, (GFunc) add_item, NULL);
 		g_slist_foreach(list, (GFunc) g_free, NULL);
 		g_slist_free(list);
@@ -238,6 +246,7 @@ static void on_go_home(void)
 static gchar *get_default_dir(void)
 {
 	const gchar *dir = NULL;
+	GeanyProject *project = geany->app->project;
 
 	if (project)
 		dir = project->base_path;
@@ -252,16 +261,15 @@ static void on_current_path(void)
 {
 	gchar *fname;
 	gchar *dir;
-	gint idx = p_document->get_cur_idx();
+	GeanyDocument *doc = p_document->get_current();
 
-	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_name == NULL ||
-		! g_path_is_absolute(doc_list[idx].file_name))
+	if (doc == NULL || doc->file_name == NULL || ! g_path_is_absolute(doc->file_name))
 	{
 		setptr(current_dir, get_default_dir());
 		refresh();
 		return;
 	}
-	fname = doc_list[idx].file_name;
+	fname = doc->file_name;
 	fname = p_utils->get_locale_from_utf8(fname);
 	dir = g_path_get_dirname(fname);
 	g_free(fname);
@@ -521,8 +529,7 @@ static GtkWidget *create_popup_menu(void)
 	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_OPEN, NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer) item, "activate",
-		G_CALLBACK(on_open_clicked), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_open_clicked), NULL);
 	popup_items.open = item;
 
 	image = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
@@ -531,8 +538,7 @@ static GtkWidget *create_popup_menu(void)
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer) item, "activate",
-		G_CALLBACK(on_external_open), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_external_open), NULL);
 	popup_items.open_external = item;
 
 	image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
@@ -541,7 +547,7 @@ static GtkWidget *create_popup_menu(void)
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_find_in_files), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_find_in_files), NULL);
 	popup_items.find_in_files = item;
 
 	item = gtk_separator_menu_item_new();
@@ -551,7 +557,7 @@ static GtkWidget *create_popup_menu(void)
 	item = gtk_check_menu_item_new_with_mnemonic(_("Show _Hidden Files"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_hidden_files_clicked), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_hidden_files_clicked), NULL);
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -562,7 +568,7 @@ static GtkWidget *create_popup_menu(void)
 		gtk_image_new_from_stock("gtk-close", GTK_ICON_SIZE_MENU));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_hide_sidebar), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_hide_sidebar), NULL);
 
 	return menu;
 }
@@ -572,6 +578,8 @@ static gboolean on_button_press(GtkWidget *widget, GdkEventButton *event, gpoint
 {
 	if (event->button == 1 && event->type == GDK_2BUTTON_PRESS)
 		on_open_clicked(NULL, NULL);
+	else if (event->button == 3)
+		return TRUE;
 	return FALSE;
 }
 
@@ -623,6 +631,16 @@ static gboolean on_key_press(GtkWidget *widget, GdkEventKey *event, gpointer dat
 }
 
 
+static void on_clear_filter(GtkEntry *entry, gpointer user_data)
+{
+	setptr(filter, NULL);
+
+	gtk_entry_set_text(GTK_ENTRY(filter_entry), "");
+
+	refresh();
+}
+
+
 static void on_path_entry_activate(GtkEntry *entry, gpointer user_data)
 {
 	gchar *new_dir = (gchar*) gtk_entry_get_text(entry);
@@ -640,6 +658,20 @@ static void on_path_entry_activate(GtkEntry *entry, gpointer user_data)
 		new_dir = g_strdup(g_get_home_dir());
 
 	setptr(current_dir, new_dir);
+
+	on_clear_filter(NULL, NULL);
+}
+
+
+static void on_filter_activate(GtkEntry *entry, gpointer user_data)
+{
+	setptr(filter, g_strdup(gtk_entry_get_text(entry)));
+
+	if (! NZV(filter))
+	{
+		setptr(filter, g_strdup("*"));
+	}
+
 	refresh();
 }
 
@@ -654,6 +686,7 @@ static void prepare_file_view(void)
 	file_store = gtk_list_store_new(FILEVIEW_N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
 
 	gtk_tree_view_set_model(GTK_TREE_VIEW(file_view), GTK_TREE_MODEL(file_store));
+	g_object_unref(file_store);
 
 	icon_renderer = gtk_cell_renderer_pixbuf_new();
 	text_renderer = gtk_cell_renderer_text_new();
@@ -668,7 +701,7 @@ static void prepare_file_view(void)
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(file_view), TRUE);
 	gtk_tree_view_set_search_column(GTK_TREE_VIEW(file_view), FILEVIEW_COLUMN_NAME);
 
-	pfd = pango_font_description_from_string(prefs->tagbar_font);
+	pfd = pango_font_description_from_string(geany->interface_prefs->tagbar_font);
 	gtk_widget_modify_font(file_view, pfd);
 	pango_font_description_free(pfd);
 
@@ -677,12 +710,9 @@ static void prepare_file_view(void)
 	gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
 
 	g_signal_connect(G_OBJECT(file_view), "realize", G_CALLBACK(on_current_path), NULL);
-	g_signal_connect(G_OBJECT(file_view), "button-press-event",
-		G_CALLBACK(on_button_press), NULL);
-	g_signal_connect(G_OBJECT(file_view), "button-release-event",
-		G_CALLBACK(on_button_release), NULL);
-	g_signal_connect(G_OBJECT(file_view), "key-press-event",
-		G_CALLBACK(on_key_press), NULL);
+	g_signal_connect(file_view, "button-press-event", G_CALLBACK(on_button_press), NULL);
+	g_signal_connect(file_view, "button-release-event", G_CALLBACK(on_button_release), NULL);
+	g_signal_connect(file_view, "key-press-event", G_CALLBACK(on_key_press), NULL);
 }
 
 
@@ -690,7 +720,7 @@ static GtkWidget *make_toolbar(void)
 {
 	GtkWidget *wid, *toolbar;
 	GtkTooltips *tooltips = GTK_TOOLTIPS(p_support->lookup_widget(
-		app->window, "tooltips"));
+		geany->main_widgets->window, "tooltips"));
 
 	toolbar = gtk_toolbar_new();
 	gtk_toolbar_set_icon_size(GTK_TOOLBAR(toolbar), GTK_ICON_SIZE_MENU);
@@ -699,31 +729,57 @@ static GtkWidget *make_toolbar(void)
 	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_GO_UP);
 	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(wid), tooltips,
 		_("Up"), NULL);
-	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(on_go_up), NULL);
+	g_signal_connect(wid, "clicked", G_CALLBACK(on_go_up), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
 	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_REFRESH);
 	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(wid), tooltips,
 		_("Refresh"), NULL);
-	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(refresh), NULL);
+	g_signal_connect(wid, "clicked", G_CALLBACK(refresh), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
 	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_HOME);
 	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(wid), tooltips,
 		_("Home"), NULL);
-	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(on_go_home), NULL);
+	g_signal_connect(wid, "clicked", G_CALLBACK(on_go_home), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
 	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_JUMP_TO);
 	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(wid), tooltips,
 		_("Set path from document"), NULL);
-	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(on_current_path), NULL);
+	g_signal_connect(wid, "clicked", G_CALLBACK(on_current_path), NULL);
+	gtk_container_add(GTK_CONTAINER(toolbar), wid);
+
+	wid = (GtkWidget *) gtk_separator_tool_item_new();
+	gtk_container_add(GTK_CONTAINER(toolbar), wid);
+
+	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_CLEAR);
+	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(wid), tooltips, _("Clear the filter"), NULL);
+	g_signal_connect(wid, "clicked", G_CALLBACK(on_clear_filter), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
 	return toolbar;
 }
 
 
+static GtkWidget *make_filterbar(void)
+{
+	GtkWidget *label, *filterbar;
+
+	filterbar = gtk_hbox_new(FALSE, 1);
+
+	label = gtk_label_new(_("Filter:"));
+
+	filter_entry = gtk_entry_new();
+	g_signal_connect(filter_entry, "activate", G_CALLBACK(on_filter_activate), NULL);
+
+	gtk_box_pack_start(GTK_BOX(filterbar), label, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(filterbar), filter_entry, TRUE, TRUE, 0);
+
+	return filterbar;
+}
+
+
 static gboolean completion_match_func(GtkEntryCompletion *completion, const gchar *key,
 									  GtkTreeIter *iter, gpointer user_data)
 {
@@ -802,7 +858,7 @@ static void load_settings(void)
 	GError *error = NULL;
 	gboolean tmp;
 
-	config_file = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "plugins", G_DIR_SEPARATOR_S,
+	config_file = g_strconcat(geany->app->configdir, G_DIR_SEPARATOR_S, "plugins", G_DIR_SEPARATOR_S,
 		"filebrowser", G_DIR_SEPARATOR_S, "filebrowser.conf", NULL);
 	g_key_file_load_from_file(config, config_file, G_KEY_FILE_NONE, NULL);
 	open_cmd = g_key_file_get_string(config, "filebrowser", "open_command", &error);
@@ -835,17 +891,22 @@ static void kb_activate(guint key_id)
 }
 
 
-void init(GeanyData *data)
+void plugin_init(GeanyData *data)
 {
-	GtkWidget *scrollwin, *toolbar;
+	GtkWidget *scrollwin, *toolbar, *filterbar;
+
+	filter = NULL;
 
 	file_view_vbox = gtk_vbox_new(FALSE, 0);
 	toolbar = make_toolbar();
 	gtk_box_pack_start(GTK_BOX(file_view_vbox), toolbar, FALSE, FALSE, 0);
 
+	filterbar = make_filterbar();
+	gtk_box_pack_start(GTK_BOX(file_view_vbox), filterbar, FALSE, FALSE, 0);
+
 	path_entry = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(file_view_vbox), path_entry, FALSE, FALSE, 2);
-	g_signal_connect(G_OBJECT(path_entry), "activate", G_CALLBACK(on_path_entry_activate), NULL);
+	g_signal_connect(path_entry, "activate", G_CALLBACK(on_path_entry_activate), NULL);
 
 	file_view = gtk_tree_view_new();
 	prepare_file_view();
@@ -859,7 +920,7 @@ void init(GeanyData *data)
 	gtk_container_add(GTK_CONTAINER(file_view_vbox), scrollwin);
 
 	gtk_widget_show_all(file_view_vbox);
-	gtk_notebook_append_page(GTK_NOTEBOOK(app->treeview_notebook), file_view_vbox,
+	gtk_notebook_append_page(GTK_NOTEBOOK(geany->main_widgets->sidebar_notebook), file_view_vbox,
 		gtk_label_new(_("Files")));
 
 	load_settings();
@@ -872,61 +933,27 @@ void init(GeanyData *data)
 }
 
 
-void configure(GtkWidget *parent)
+static struct
 {
-	GtkWidget *dialog, *label, *entry, *checkbox_of, *checkbox_hf, *vbox;
-	GtkTooltips *tooltips = gtk_tooltips_new();
+	GtkWidget *open_cmd_entry;
+	GtkWidget *show_hidden_checkbox;
+	GtkWidget *hide_objects_checkbox;
+}
+pref_widgets;
 
-	dialog = gtk_dialog_new_with_buttons(_("File Browser"),
-		GTK_WINDOW(parent), GTK_DIALOG_DESTROY_WITH_PARENT,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-	vbox = p_ui->dialog_vbox_new(GTK_DIALOG(dialog));
-	gtk_widget_set_name(dialog, "GeanyDialog");
-	gtk_box_set_spacing(GTK_BOX(vbox), 6);
-
-	label = gtk_label_new(_("External open command:"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_container_add(GTK_CONTAINER(vbox), label);
-
-	entry = gtk_entry_new();
-	gtk_widget_show(entry);
-	if (open_cmd != NULL)
-		gtk_entry_set_text(GTK_ENTRY(entry), open_cmd);
-	gtk_tooltips_set_tip(tooltips, entry,
-		_("The command to execute when using \"Open with\". You can use %f and %d wildcards.\n"
-		  "%f will be replaced with the filename including full path\n"
-		  "%d will be replaced with the path name of the selected file without the filename"),
-		  NULL);
-	gtk_container_add(GTK_CONTAINER(vbox), entry);
-
-	checkbox_hf = gtk_check_button_new_with_label(_("Show hidden files"));
-	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_hf), FALSE);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_hf), show_hidden_files);
-	gtk_box_pack_start(GTK_BOX(vbox), checkbox_hf, FALSE, FALSE, 5);
-
-	checkbox_of = gtk_check_button_new_with_label(_("Hide object files"));
-	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_of), FALSE);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_of), hide_object_files);
-	gtk_tooltips_set_tip(tooltips, checkbox_of,
-		_("Don't show generated object files in the file browser, this includes "
-		  "*.o, *.obj. *.so, *.dll, *.a, *.lib"),
-		  NULL);
-	gtk_box_pack_start(GTK_BOX(vbox), checkbox_of, FALSE, FALSE, 5);
-
-
-	gtk_widget_show_all(vbox);
-
-	/* run the dialog and check for the response code */
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
+static void
+on_configure_response(GtkDialog *dialog, gint response, gpointer user_data)
+{
+	if (response == GTK_RESPONSE_OK || response == GTK_RESPONSE_APPLY)
 	{
 		GKeyFile *config = g_key_file_new();
 		gchar *data;
 		gchar *config_dir = g_path_get_dirname(config_file);
 
 		g_free(open_cmd);
-		open_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
-		show_hidden_files = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbox_hf));
-		hide_object_files = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbox_of));
+		open_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(pref_widgets.open_cmd_entry)));
+		show_hidden_files = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pref_widgets.show_hidden_checkbox));
+		hide_object_files = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pref_widgets.hide_objects_checkbox));
 
 		g_key_file_load_from_file(config, config_file, G_KEY_FILE_NONE, NULL);
 
@@ -953,14 +980,60 @@ void configure(GtkWidget *parent)
 		g_free(config_dir);
 		g_key_file_free(config);
 	}
-	gtk_widget_destroy(dialog);
 }
 
 
-void cleanup(void)
+GtkWidget *plugin_configure(GtkDialog *dialog)
+{
+	GtkWidget *label, *entry, *checkbox_of, *checkbox_hf, *vbox;
+	GtkTooltips *tooltips = gtk_tooltips_new();
+
+	vbox = gtk_vbox_new(FALSE, 6);
+
+	label = gtk_label_new(_("External open command:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	gtk_container_add(GTK_CONTAINER(vbox), label);
+
+	entry = gtk_entry_new();
+	gtk_widget_show(entry);
+	if (open_cmd != NULL)
+		gtk_entry_set_text(GTK_ENTRY(entry), open_cmd);
+	gtk_tooltips_set_tip(tooltips, entry,
+		_("The command to execute when using \"Open with\". You can use %f and %d wildcards.\n"
+		  "%f will be replaced with the filename including full path\n"
+		  "%d will be replaced with the path name of the selected file without the filename"),
+		  NULL);
+	gtk_container_add(GTK_CONTAINER(vbox), entry);
+	pref_widgets.open_cmd_entry = entry;
+
+	checkbox_hf = gtk_check_button_new_with_label(_("Show hidden files"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_hf), FALSE);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_hf), show_hidden_files);
+	gtk_box_pack_start(GTK_BOX(vbox), checkbox_hf, FALSE, FALSE, 5);
+	pref_widgets.show_hidden_checkbox = checkbox_hf;
+
+	checkbox_of = gtk_check_button_new_with_label(_("Hide object files"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_of), FALSE);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_of), hide_object_files);
+	gtk_tooltips_set_tip(tooltips, checkbox_of,
+		_("Don't show generated object files in the file browser, this includes "
+		  "*.o, *.obj. *.so, *.dll, *.a, *.lib"),
+		  NULL);
+	gtk_box_pack_start(GTK_BOX(vbox), checkbox_of, FALSE, FALSE, 5);
+	pref_widgets.hide_objects_checkbox = checkbox_of;
+
+	gtk_widget_show_all(vbox);
+
+	g_signal_connect(dialog, "response", G_CALLBACK(on_configure_response), NULL);
+	return vbox;
+}
+
+
+void plugin_cleanup(void)
 {
 	g_free(config_file);
 	g_free(open_cmd);
+	g_free(filter);
 	gtk_widget_destroy(file_view_vbox);
 	g_object_unref(G_OBJECT(entry_completion));
 }
