@@ -28,25 +28,34 @@
  * e.g. ".svn". */
 
 #include "geany.h"
+#include <string.h>
+
 #include "support.h"
 #include "plugindata.h"
 #include "document.h"
+#include "editor.h"
 #include "filetypes.h"
 #include "utils.h"
 #include "project.h"
+#include "ui_utils.h"
 #include "pluginmacros.h"
 
+#define project	geany->app->project
+
 
-PluginFields	*plugin_fields;
 GeanyData		*geany_data;
+GeanyFunctions	*geany_functions;
 
 
-PLUGIN_VERSION_CHECK(27)
+PLUGIN_VERSION_CHECK(GEANY_API_VERSION)
 
-PLUGIN_INFO(_("Version Diff"), _("Creates a patch of a file against version control."), VERSION,
+PLUGIN_SET_INFO(_("Version Diff"), _("Creates a patch of a file against version control."), VERSION,
 	_("The Geany developer team"))
 
 
+static GtkWidget *main_menu_item = NULL;
+
+
 enum
 {
 	VC_COMMAND_DIFF_FILE,
@@ -74,9 +83,9 @@ static const gchar* CVS_CMD_DIFF_PROJECT[] = {"cvs", "diff", "-u", NULL};
 static void* CVS_COMMANDS[] = { CVS_CMD_DIFF_FILE, CVS_CMD_DIFF_DIR, CVS_CMD_DIFF_PROJECT };
 
 
-static const gchar* GIT_CMD_DIFF_FILE[] = {"git", "diff", BASE_FILENAME,  NULL};
-static const gchar* GIT_CMD_DIFF_DIR[]  = {"git", "diff", NULL};
-static const gchar* GIT_CMD_DIFF_PROJECT[] = {"git", "diff", NULL};
+static const gchar* GIT_CMD_DIFF_FILE[] = {"git", "diff", "HEAD", "--", BASE_FILENAME,  NULL};
+static const gchar* GIT_CMD_DIFF_DIR[]  = {"git", "diff", "HEAD", NULL};
+static const gchar* GIT_CMD_DIFF_PROJECT[] = {"git", "diff", "HEAD", NULL};
 
 static const gchar* GIT_ENV_DIFF_FILE[] = {"PAGER=cat", NULL};
 static const gchar* GIT_ENV_DIFF_DIR[]  = {"PAGER=cat", NULL};
@@ -114,20 +123,20 @@ static gboolean find_subdir(const gchar* filename, const gchar *subdir)
 	gchar *base_prev = g_strdup(":");
 
 	if (g_file_test(filename, G_FILE_TEST_IS_DIR))
-	    base = g_strdup(filename);
+		base = g_strdup(filename);
 	else
-	    base = g_path_get_dirname(filename);
+		base = g_path_get_dirname(filename);
 
-	while(strcmp(base, base_prev) != 0)
+	while (strcmp(base, base_prev) != 0)
 	{
-	    gitdir = g_build_path("/", base, subdir, NULL);
-	    ret = g_file_test(gitdir, G_FILE_TEST_IS_DIR);
-	    g_free(gitdir);
-	    if (ret)
+		gitdir = g_build_path("/", base, subdir, NULL);
+		ret = g_file_test(gitdir, G_FILE_TEST_IS_DIR);
+		g_free(gitdir);
+		if (ret)
 			break;
-	    g_free(base_prev);
-	    base_prev = base;
-	    base = g_path_get_dirname(base);
+		g_free(base_prev);
+		base_prev = base;
+		base = g_path_get_dirname(base);
 	}
 
 	g_free(base);
@@ -155,9 +164,9 @@ static gboolean check_filename(const gchar* filename, VC_RECORD *vc)
 		return find_subdir(filename, vc->subdir);
 
 	if (g_file_test(filename, G_FILE_TEST_IS_DIR))
-	    base = g_strdup(filename);
+		base = g_strdup(filename);
 	else
-	    base = g_path_get_dirname(filename);
+		base = g_path_get_dirname(filename);
 	dir = g_build_path("/", base, vc->subdir, NULL);
 
 	ret = g_file_test(dir, G_FILE_TEST_IS_DIR);
@@ -244,67 +253,53 @@ static void* get_cmd_env(gint cmd_type, gboolean cmd, const gchar* filename, int
 }
 
 
-static int find_by_filename(const gchar* filename)
-{
-	guint i;
-
-	for (i = 0; i < doc_array->len; i++)
-	{
-		if (doc_list[i].is_valid && doc_list[i].file_name &&
-			strcmp(doc_list[i].file_name, filename) == 0)
-			return i;
-	}
-	return -1;
-}
-
-
-/* name_prefix should be in UTF-8, and can have a path. */
-static void show_output(const gchar *std_output, const gchar *name_prefix,
+/* utf8_name_prefix can have a path. */
+static void show_output(const gchar *std_output, const gchar *utf8_name_prefix,
 		const gchar *force_encoding)
 {
 	gchar	*text, *detect_enc = NULL;
-	gint 	idx, page;
+	gint 	page;
+	GeanyDocument *doc;
 	GtkNotebook *book;
 	gchar	*filename;
 
-	filename = g_path_get_basename(name_prefix);
+	filename = g_path_get_basename(utf8_name_prefix);
 	setptr(filename, g_strconcat(filename, ".vc.diff", NULL));
 
 	/* need to convert input text from the encoding of the original file into
 	 * UTF-8 because internally Geany always needs UTF-8 */
 	if (force_encoding)
 	{
-		text = p_encoding->convert_to_utf8_from_charset(
+		text = p_encodings->convert_to_utf8_from_charset(
 			std_output, (gsize)-1, force_encoding, TRUE);
 	}
 	else
 	{
-		text = p_encoding->convert_to_utf8(std_output, (gsize)-1, &detect_enc);
+		text = p_encodings->convert_to_utf8(std_output, (gsize)-1, &detect_enc);
 	}
 	if (text)
 	{
-		idx = find_by_filename(filename);
-		if ( idx == -1)
+		doc = p_document->find_by_filename(filename);
+		if (doc == NULL)
 		{
-			idx = p_document->new_file(filename,
-				geany_data->filetypes[GEANY_FILETYPES_DIFF], text);
+			GeanyFiletype *ft = p_filetypes->lookup_by_name("Diff");
+			doc = p_document->new_file(filename, ft, text);
 		}
 		else
 		{
-			p_sci->set_text(doc_list[idx].sci, text);
-			book = GTK_NOTEBOOK(app->notebook);
-			page = gtk_notebook_page_num(book, GTK_WIDGET(doc_list[idx].sci));
+			p_sci->set_text(doc->editor->sci, text);
+			book = GTK_NOTEBOOK(geany->main_widgets->notebook);
+			page = gtk_notebook_page_num(book, GTK_WIDGET(doc->editor->sci));
 			gtk_notebook_set_current_page(book, page);
-			doc_list[idx].changed = FALSE;
-			p_document->set_text_changed(idx);
+			p_document->set_text_changed(doc, FALSE);
 		}
 
-		p_document->set_encoding(idx,
+		p_document->set_encoding(doc,
 			force_encoding ? force_encoding : detect_enc);
 	}
 	else
 	{
-		p_ui->set_statusbar(FALSE, _("Could not parse the output of the diff"));
+		p_ui->set_statusbar(FALSE, _("Input conversion of the diff output failed."));
 	}
 	g_free(text);
 	g_free(detect_enc);
@@ -346,7 +341,7 @@ static gchar *make_diff(const gchar *filename, gint cmd)
 		/* CVS dump stuff to stderr when diff nested dirs */
 		if (strcmp(argv[0], "cvs") != 0 && NZV(std_error))
 		{
-		    p_dialogs->show_msgbox(1,
+			p_dialogs->show_msgbox(1,
 				_("%s exited with an error: \n%s."), argv[0], g_strstrip(std_error));
 		}
 		else if (NZV(std_output))
@@ -392,26 +387,27 @@ static gchar *make_diff(const gchar *filename, gint cmd)
 /* Make a diff from the current directory */
 static void vcdirectory_activated(GtkMenuItem *menuitem, gpointer gdata)
 {
-	gint	idx;
+	GeanyDocument *doc;
 	gchar	*base_name = NULL;
 	gchar	*locale_filename = NULL;
 	gchar	*text;
 
-	idx = p_document->get_cur_idx();
+	doc = p_document->get_current();
 
-	g_return_if_fail(DOC_IDX_VALID(idx) && doc_list[idx].file_name != NULL);
+	g_return_if_fail(doc != NULL && doc->file_name != NULL);
 
-	if (doc_list[idx].changed)
+	if (doc->changed)
 	{
-		p_document->save_file(idx, FALSE);
+		p_document->save_file(doc, FALSE);
 	}
 
-	locale_filename = p_utils->get_locale_from_utf8(doc_list[idx].file_name);
+	locale_filename = p_utils->get_locale_from_utf8(doc->file_name);
 	base_name = g_path_get_dirname(locale_filename);
 
 	text = make_diff(base_name, VC_COMMAND_DIFF_DIR);
 	if (text)
 	{
+		setptr(base_name, p_utils->get_utf8_from_locale(base_name));
 		show_output(text, base_name, NULL);
 		g_free(text);
 	}
@@ -424,17 +420,17 @@ static void vcdirectory_activated(GtkMenuItem *menuitem, gpointer gdata)
 /* Callback if menu item for the current project was activated */
 static void vcproject_activated(GtkMenuItem *menuitem, gpointer gdata)
 {
-	gint	idx;
+	GeanyDocument *doc;
 	gchar	*locale_filename = NULL;
 	gchar	*text;
 
-	idx = p_document->get_cur_idx();
+	doc = p_document->get_current();
 
 	g_return_if_fail(project != NULL && NZV(project->base_path));
 
-	if (DOC_IDX_VALID(idx) && doc_list[idx].changed && doc_list[idx].file_name != NULL)
+	if (doc != NULL && doc->changed && doc->file_name != NULL)
 	{
-		p_document->save_file(idx, FALSE);
+		p_document->save_file(doc, FALSE);
 	}
 
 	locale_filename = p_utils->get_locale_from_utf8(project->base_path);
@@ -451,24 +447,24 @@ static void vcproject_activated(GtkMenuItem *menuitem, gpointer gdata)
 /* Callback if menu item for a single file was activated */
 static void vcfile_activated(GtkMenuItem *menuitem, gpointer gdata)
 {
-	gint	idx;
-	gchar	*locale_filename, *text;
+	GeanyDocument *doc;
+	gchar *locale_filename, *text;
 
-	idx = p_document->get_cur_idx();
+	doc = p_document->get_current();
 
-	g_return_if_fail(DOC_IDX_VALID(idx) && doc_list[idx].file_name != NULL);
+	g_return_if_fail(doc != NULL && doc->file_name != NULL);
 
-	if (doc_list[idx].changed)
+	if (doc->changed)
 	{
-		p_document->save_file(idx, FALSE);
+		p_document->save_file(doc, FALSE);
 	}
 
-	locale_filename = p_utils->get_locale_from_utf8(doc_list[idx].file_name);
+	locale_filename = p_utils->get_locale_from_utf8(doc->file_name);
 
 	text = make_diff(locale_filename, VC_COMMAND_DIFF_FILE);
 	if (text)
 	{
-		show_output(text, doc_list[idx].file_name, doc_list[idx].encoding);
+		show_output(text, doc->file_name, doc->encoding);
 		g_free(text);
 	}
 	g_free(locale_filename);
@@ -481,7 +477,7 @@ static GtkWidget *menu_vcdiff_project = NULL;
 
 static void update_menu_items(void)
 {
-	document	*doc;
+	GeanyDocument *doc;
 	gboolean	have_file;
 	gboolean    have_vc = FALSE;
 
@@ -498,7 +494,7 @@ static void update_menu_items(void)
 
 
 /* Called by Geany to initialize the plugin */
-void init(GeanyData *data)
+void plugin_init(GeanyData *data)
 {
 	GtkWidget	*menu_vcdiff = NULL;
 	GtkWidget	*menu_vcdiff_menu = NULL;
@@ -507,10 +503,9 @@ void init(GeanyData *data)
 	tooltips = gtk_tooltips_new();
 
 	menu_vcdiff = gtk_image_menu_item_new_with_mnemonic(_("_Version Diff"));
-	gtk_container_add(GTK_CONTAINER(data->tools_menu), menu_vcdiff);
+	gtk_container_add(GTK_CONTAINER(geany->main_widgets->tools_menu), menu_vcdiff);
 
-	g_signal_connect((gpointer) menu_vcdiff, "activate",
-		G_CALLBACK(update_menu_items), NULL);
+	g_signal_connect(menu_vcdiff, "activate", G_CALLBACK(update_menu_items), NULL);
 
 	menu_vcdiff_menu = gtk_menu_new ();
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_vcdiff), menu_vcdiff_menu);
@@ -521,8 +516,7 @@ void init(GeanyData *data)
 	gtk_tooltips_set_tip (tooltips, menu_vcdiff_file,
 		_("Make a diff from the current active file"), NULL);
 
-	g_signal_connect((gpointer) menu_vcdiff_file, "activate",
-		G_CALLBACK(vcfile_activated), NULL);
+	g_signal_connect(menu_vcdiff_file, "activate", G_CALLBACK(vcfile_activated), NULL);
 
 	/* Directory */
 	menu_vcdiff_dir = gtk_menu_item_new_with_mnemonic(_("From Current _Directory"));
@@ -530,8 +524,7 @@ void init(GeanyData *data)
 	gtk_tooltips_set_tip (tooltips, menu_vcdiff_dir,
 		_("Make a diff from the directory of the current active file"), NULL);
 
-	g_signal_connect((gpointer) menu_vcdiff_dir, "activate",
-		G_CALLBACK(vcdirectory_activated), NULL);
+	g_signal_connect(menu_vcdiff_dir, "activate", G_CALLBACK(vcdirectory_activated), NULL);
 
 	/* Project */
 	menu_vcdiff_project = gtk_menu_item_new_with_mnemonic(_("From Current _Project"));
@@ -539,19 +532,18 @@ void init(GeanyData *data)
 	gtk_tooltips_set_tip (tooltips, menu_vcdiff_project,
 		_("Make a diff from the current project's base path"), NULL);
 
-	g_signal_connect((gpointer) menu_vcdiff_project, "activate",
-		G_CALLBACK(vcproject_activated), NULL);
+	g_signal_connect(menu_vcdiff_project, "activate", G_CALLBACK(vcproject_activated), NULL);
 
 	gtk_widget_show_all(menu_vcdiff);
 
-	plugin_fields->menu_item = menu_vcdiff;
-	plugin_fields->flags = PLUGIN_IS_DOCUMENT_SENSITIVE;
+	p_ui->add_document_sensitive(menu_vcdiff);
+	main_menu_item = menu_vcdiff;
 }
 
 
 /* Called by Geany before unloading the plugin. */
-void cleanup(void)
+void plugin_cleanup(void)
 {
-	/* remove the menu item added in init() */
-	gtk_widget_destroy(plugin_fields->menu_item);
+	/* remove the menu item added in plugin_init() */
+	gtk_widget_destroy(main_menu_item);
 }
