@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: about.c 2498 2008-04-18 11:37:37Z frlan $
+ * $Id: about.c 3091 2008-10-14 17:38:48Z frlan $
  */
 
 /*
@@ -30,12 +30,14 @@
 #include "utils.h"
 #include "ui_utils.h"
 #include "support.h"
+#include "geanywraplabel.h"
+#include "main.h"
 
 static GtkWidget *gb_window = NULL;
 #include "gb.c"
 
 
-#define HEADER "<span size=\"larger\" weight=\"bold\">Geany " VERSION "%s</span>"
+#define HEADER "<span size=\"larger\" weight=\"bold\">Geany %s</span>"
 #define INFO "<span size=\"larger\" weight=\"bold\">%s</span>"
 #define CODENAME "<span weight=\"bold\">\"" GEANY_CODENAME "\"</span>"
 #define BUILDDATE "<span size=\"smaller\">%s</span>"
@@ -45,7 +47,7 @@ const gchar *translators[][2] = {
 	{ "be_BY", "Yura Semashko &lt;yurand2@gmail.com&gt;" },
 	{ "bg", "Dilyan Rusev &lt;dilyanrusev@gmail.com&gt;" },
 	{ "ca_ES", "Toni Garcia-Navarro &lt;topi@elpiset.net&gt;" },
-	{ "cs_CZ", "Petr Messner &lt;messa@messa.cz&gt;" },
+	{ "cs_CZ", "Petr Messner &lt;messa@messa.cz&gt;\nAnna Talianova &lt;anickat1@gmail.com&gt;" },
 	{ "de_DE", "Enrico Tröger &lt;enrico.troeger@uvena.de&gt;\nFrank Lanitz &lt;frank@frank.uvena.de&gt;" },
 	{ "el", "Stavros Temertzidis &lt;bullgr@gmail.com&gt;" },
 	{ "en_GB", "Jeff Bailes &lt;thepizzaking@gmail.com&gt;" },
@@ -55,15 +57,17 @@ const gchar *translators[][2] = {
 	{ "hu", "Gabor Kmetyko &lt;kg_kilo@freemail.hu&gt;" },
 	{ "it", "Max Baldinelli &lt;m.baldinelli@agora.it&gt;,\nDario Santomarco &lt;dariello@yahoo.it&gt;" },
 	{ "ja", "Tarot Osuji &lt;tarot@sdf.lonestar.org&gt;\nChikahiro Masami &lt;cmasa.z321@gmail.com&gt;" },
+	{ "ko", "Park Jang-heon &lt;dotkabi@gmail.com&gt;" },
 	{ "nl", "Kurt De Bree &lt;kdebree@telenet.be&gt;" },
 	{ "pl_PL", "Jarosław Foksa &lt;jfoksa@gmail.com&gt;" },
 	{ "pt_BR", "Alexandra Moreire &lt;alexandream@gmail.com&gt;\nAdrovane Marques Kade &lt;adrovane@gmail.com&gt;" },
 	{ "ro", "Alex Eftimie &lt;alex@rosedu.org&gt;" },
 	{ "ru_RU", "brahmann_ &lt;brahmann@pisem.net&gt;,\nNikita E. Shalaev &lt;nshalaev@eu.spb.ru&gt;" },
 	{ "sv", "Tony Mattsson &lt;superxorn@gmail.com&gt;" },
+	{ "tr", "Gürkan Gür &lt;seqizz@gmail.com&gt;"},
 	{ "uk", "Boris Dibrov &lt;dibrov.bor@gmail.com&gt;" },
 	{ "vi_VN", "Clytie Siddall &lt;clytie@riverland.net.au&gt;" },
-	{ "zh_CN", "Dormouse Young &lt;dormouse.young@gmail.com&gt;" },
+	{ "zh_CN", "Dormouse Young &lt;mouselinux@163.com&gt;" },
 	{ "zh_TW", "KoViCH &lt;kovich.ian@gmail.com&gt;" }
 };
 static const gint translators_len = G_N_ELEMENTS(translators);
@@ -73,6 +77,14 @@ const gchar *prev_translators[][2] = {
 };
 static const gint prev_translators_len = G_N_ELEMENTS(prev_translators);
 
+static const gchar *contributors =
+"Alexander Rodin, Andrew Rowland, Anh Phạm, blackdog, Bo Lorentsen, Bob Doan, Bronisław Białek, Catalin Marinas, "
+"Christoph Berg, Daniel Richard G., Dave Moore, Dirk Weber, Felipe Pena, François Cami, "
+"Giuseppe Torelli, Guillaume Hoffmann, Jason Oster, Jean-François Wauthy, Jeff Pohlmeyer, "
+"John Gabriele, Josef Whiter, Kevin Ellwood, Kristoffer A. Tjernås, Marko Peric, Matti Mårds, "
+"Peter Strand, Pierre Joye, Rob van der Linde, Robert McGinley, S Jagannathan, Saleem Abdulrasool, "
+"Sebastian Kraft, Shiv, Slava Semushin, Stefan Oltmanns, Tamim, Tomás Vírseda, Yura Siamashka";
+
 
 static void header_eventbox_style_set(GtkWidget *widget);
 static void header_label_style_set(GtkWidget *widget);
@@ -118,14 +130,14 @@ static GtkWidget *create_dialog(void)
 	dialog = gtk_dialog_new();
 
 	/* configure dialog */
-	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(app->window));
+	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(main_widgets.window));
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_title(GTK_WINDOW(dialog), _("About Geany"));
 	gtk_widget_set_name(dialog, "GeanyDialog");
 	button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
-	g_signal_connect(G_OBJECT(dialog), "key-press-event", G_CALLBACK(gb_on_key_pressed), NULL);
+	g_signal_connect(dialog, "key-press-event", G_CALLBACK(gb_on_key_pressed), NULL);
 
 	/* create header */
 	header_eventbox = gtk_event_box_new();
@@ -139,15 +151,14 @@ static GtkWidget *create_dialog(void)
 	header_label = gtk_label_new(NULL);
 	gtk_label_set_use_markup(GTK_LABEL(header_label), TRUE);
 	/* print the subversion revision generated by ./configure if it is available */
-	g_snprintf(buffer, sizeof(buffer), HEADER,
-		(utils_str_equal(REVISION, "-1")) ? "" : "svn >= " REVISION);
+	g_snprintf(buffer, sizeof(buffer), HEADER, main_get_version_string());
 	gtk_label_set_markup(GTK_LABEL(header_label), buffer);
 	gtk_widget_show(header_label);
 	gtk_box_pack_start(GTK_BOX(header_hbox), header_label, FALSE,FALSE,0);
 	header_eventbox_style_set(header_eventbox);
 	header_label_style_set(header_label);
-	g_signal_connect_after(G_OBJECT(header_eventbox), "style_set", G_CALLBACK(header_eventbox_style_set), NULL);
-	g_signal_connect_after(G_OBJECT(header_label), "style_set", G_CALLBACK(header_label_style_set), NULL);
+	g_signal_connect_after(header_eventbox, "style-set", G_CALLBACK(header_eventbox_style_set), NULL);
+	g_signal_connect_after(header_label, "style-set", G_CALLBACK(header_label_style_set), NULL);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), header_eventbox, FALSE, FALSE, 0);
 
 	/* set image */
@@ -202,7 +213,7 @@ static GtkWidget *create_dialog(void)
 	box = gtk_hbutton_box_new();
 	url_button = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(url_button), GTK_RELIEF_NONE);
-	g_signal_connect(G_OBJECT(url_button), "clicked", G_CALLBACK(homepage_clicked), GEANY_HOMEPAGE);
+	g_signal_connect(url_button, "clicked", G_CALLBACK(homepage_clicked), GEANY_HOMEPAGE);
 	label = gtk_label_new(NULL);
 	gtk_label_set_text(GTK_LABEL(label), GEANY_HOMEPAGE);
 	gtk_widget_show(label);
@@ -234,7 +245,7 @@ static GtkWidget *create_dialog(void)
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(credits_scrollwin),
 		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
-	table = gtk_table_new(14 + translators_len + prev_translators_len, 3, FALSE);
+	table = gtk_table_new(18 + translators_len + prev_translators_len, 3, FALSE);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
 
 	row = 0;
@@ -309,6 +320,44 @@ static GtkWidget *create_dialog(void)
 		row++;
 	}
 
+
+	ROW("", row, 0, 0, 0, 0);
+	row++;
+
+	g_snprintf(buffer, sizeof(buffer),
+		"<span size=\"larger\" weight=\"bold\">%s</span>", _("Contributors"));
+	label = gtk_label_new(buffer);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row + 1,
+					(GtkAttachOptions) (GTK_FILL),
+					(GtkAttachOptions) (0), 0, 5);
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	row++;
+
+	g_snprintf(buffer, sizeof(buffer),
+		_("Some of the many contributors (for a more detailed list, see the file %s):"),
+#ifdef G_OS_WIN32
+			"Thanks.txt"
+#else
+			"THANKS"
+#endif
+		);
+	label = gtk_label_new(buffer);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row + 1,
+					(GtkAttachOptions) (GTK_FILL),
+					(GtkAttachOptions) (0), 0, 5);
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	row++;
+
+	label = geany_wrap_label_new(contributors);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row + 1,
+					(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
+					(GtkAttachOptions) (0), 0, 5);
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	row++;
+
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(credits_scrollwin), table);
 	gtk_viewport_set_shadow_type(GTK_VIEWPORT(gtk_widget_get_parent(table)), GTK_SHADOW_NONE);
 	gtk_widget_show_all(table);
