@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: build.c 2430 2008-03-31 13:09:56Z ntrel $
+ * $Id: build.c 3026 2008-09-30 20:30:38Z eht16 $
  */
 
 /*
@@ -45,11 +45,11 @@
 
 #include "prefs.h"
 #include "support.h"
+#include "document.h"
 #include "utils.h"
 #include "ui_utils.h"
 #include "dialogs.h"
 #include "msgwindow.h"
-#include "document.h"
 #include "filetypes.h"
 #include "keybindings.h"
 #include "vte.h"
@@ -58,7 +58,7 @@
 #include "win32.h"
 
 
-BuildInfo build_info = {GBO_COMPILE, 0, NULL, GEANY_FILETYPES_ALL, NULL};
+GeanyBuildInfo build_info = {GBO_COMPILE, 0, NULL, GEANY_FILETYPES_NONE, NULL};
 
 static gchar *current_dir_entered = NULL;
 
@@ -66,7 +66,7 @@ static struct
 {
 	GPid pid;
 	gint file_type_id;
-} run_info = {0, GEANY_FILETYPES_ALL};
+} run_info = {0, GEANY_FILETYPES_NONE};
 
 #ifdef G_OS_WIN32
 static const gchar RUN_SCRIPT_CMD[] = "geany_run_script.bat";
@@ -83,9 +83,9 @@ enum
 };
 
 static BuildMenuItems default_menu_items =
-	{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+	{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 static BuildMenuItems latex_menu_items =
-	{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+	{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 
 
 static struct
@@ -98,7 +98,7 @@ widgets;
 
 static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data);
 static gboolean build_create_shellscript(const gchar *fname, const gchar *cmd, gboolean autoclose);
-static GPid build_spawn_cmd(gint idx, const gchar *cmd, const gchar *dir);
+static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *dir);
 static void set_stop_button(gboolean stop);
 static void build_exit_cb(GPid child_pid, gint status, gpointer user_data);
 static void run_exit_cb(GPid child_pid, gint status, gpointer user_data);
@@ -110,6 +110,7 @@ static void on_build_build_activate(GtkMenuItem *menuitem, gpointer user_data);
 static void on_build_make_activate(GtkMenuItem *menuitem, gpointer user_data);
 static void on_build_execute_activate(GtkMenuItem *menuitem, gpointer user_data);
 static void on_build_next_error(GtkMenuItem *menuitem, gpointer user_data);
+static void on_build_previous_error(GtkMenuItem *menuitem, gpointer user_data);
 static void kill_process(GPid *pid);
 
 
@@ -125,28 +126,29 @@ void build_finalize()
 }
 
 
-static GPid build_compile_tex_file(gint idx, gint mode)
+static GPid build_compile_tex_file(GeanyDocument *doc, gint mode)
 {
 	const gchar *cmd = NULL;
 
-	if (idx < 0 || doc_list[idx].file_name == NULL) return (GPid) 1;
+	if (doc == NULL || doc->file_name == NULL)
+		return (GPid) 1;
 
 	if (mode == LATEX_CMD_TO_DVI)
 	{
-		cmd = doc_list[idx].file_type->programs->compiler;
+		cmd = doc->file_type->programs->compiler;
 		build_info.type = GBO_COMPILE;
 	}
 	else
 	{
-		cmd = doc_list[idx].file_type->programs->linker;
+		cmd = doc->file_type->programs->linker;
 		build_info.type = GBO_BUILD;
 	}
 
-	return build_spawn_cmd(idx, cmd, NULL);
+	return build_spawn_cmd(doc, cmd, NULL);
 }
 
 
-static GPid build_view_tex_file(gint idx, gint mode)
+static GPid build_view_tex_file(GeanyDocument *doc, gint mode)
 {
 	gchar **argv, **term_argv;
 	gchar  *executable = NULL;
@@ -161,12 +163,12 @@ static GPid build_view_tex_file(gint idx, gint mode)
 	GError *error = NULL;
 	struct stat st;
 
-	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_name == NULL)
+	if (doc == NULL || doc->file_name == NULL)
 		return (GPid) 1;
 
 	run_info.file_type_id = GEANY_FILETYPES_LATEX;
 
-	executable = utils_remove_ext_from_filename(doc_list[idx].file_name);
+	executable = utils_remove_ext_from_filename(doc->file_name);
 	view_file = g_strconcat(executable, (mode == LATEX_CMD_VIEW_DVI) ? ".dvi" : ".pdf", NULL);
 
 	/* try convert in locale for stat() */
@@ -183,8 +185,8 @@ static GPid build_view_tex_file(gint idx, gint mode)
 
 	/* replace %f and %e in the run_cmd string */
 	cmd_string = g_strdup((mode == LATEX_CMD_VIEW_DVI) ?
-										g_strdup(doc_list[idx].file_type->programs->run_cmd) :
-										g_strdup(doc_list[idx].file_type->programs->run_cmd2));
+										g_strdup(doc->file_type->programs->run_cmd) :
+										g_strdup(doc->file_type->programs->run_cmd2));
 	cmd_string = utils_str_replace(cmd_string, "%f", view_file);
 	cmd_string = utils_str_replace(cmd_string, "%e", executable);
 
@@ -192,7 +194,7 @@ static GPid build_view_tex_file(gint idx, gint mode)
 	locale_cmd_string = utils_get_locale_from_utf8(cmd_string);
 
 	/* get the terminal path */
-	locale_term_cmd = utils_get_locale_from_utf8(prefs.tools_term_cmd);
+	locale_term_cmd = utils_get_locale_from_utf8(tool_prefs.term_cmd);
 	/* split the term_cmd, so arguments will work too */
 	term_argv = g_strsplit(locale_term_cmd, " ", -1);
 	term_argv_len = g_strv_length(term_argv);
@@ -209,7 +211,7 @@ static GPid build_view_tex_file(gint idx, gint mode)
 	{
 		ui_set_statusbar(TRUE,
 			_("Could not find terminal \"%s\" "
-				"(check path for Terminal tool setting in Preferences)"), prefs.tools_term_cmd);
+				"(check path for Terminal tool setting in Preferences)"), tool_prefs.term_cmd);
 
 		utils_free_pointers(executable, view_file, locale_filename, cmd_string, locale_cmd_string,
 										locale_term_cmd, NULL);
@@ -275,7 +277,7 @@ static GPid build_view_tex_file(gint idx, gint mode)
 	{
 		/*setpgid(0, getppid());*/
 		g_child_watch_add(run_info.pid, (GChildWatchFunc) run_exit_cb, NULL);
-		build_menu_update(idx);
+		build_menu_update(doc);
 	}
 
 	utils_free_pointers(executable, view_file, locale_filename, cmd_string, locale_cmd_string,
@@ -288,13 +290,13 @@ static GPid build_view_tex_file(gint idx, gint mode)
 
 
 /* get curfile.o in locale encoding from document::file_name */
-static gchar *get_object_filename(gint idx)
+static gchar *get_object_filename(GeanyDocument *doc)
 {
 	gchar *locale_filename, *short_file, *noext, *object_file;
 
-	if (doc_list[idx].file_name == NULL) return NULL;
+	if (doc->file_name == NULL) return NULL;
 
-	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
+	locale_filename = utils_get_locale_from_utf8(doc->file_name);
 
 	short_file = g_path_get_basename(locale_filename);
 	g_free(locale_filename);
@@ -309,15 +311,16 @@ static gchar *get_object_filename(gint idx)
 }
 
 
-static GPid build_make_file(gint idx, gint build_opts)
+static GPid build_make_file(GeanyDocument *doc, gint build_opts)
 {
 	GString *cmdstr;
 	gchar *dir = NULL;
 	GPid pid;
 
-	if (idx < 0 || doc_list[idx].file_name == NULL) return (GPid) 1;
+	if (doc == NULL || doc->file_name == NULL)
+		return (GPid) 1;
 
-	cmdstr = g_string_new(prefs.tools_make_cmd);
+	cmdstr = g_string_new(tool_prefs.make_cmd);
 	g_string_append_c(cmdstr, ' ');
 
 	if (build_opts == GBO_MAKE_OBJECT)
@@ -325,7 +328,7 @@ static GPid build_make_file(gint idx, gint build_opts)
 		gchar *tmp;
 
 		build_info.type = build_opts;
-		tmp = get_object_filename(idx);
+		tmp = get_object_filename(doc);
 		g_string_append(cmdstr, tmp);
 		g_free(tmp);
 	}
@@ -342,42 +345,43 @@ static GPid build_make_file(gint idx, gint build_opts)
 		dir = project_get_make_dir();
 	}
 
-	pid = build_spawn_cmd(idx, cmdstr->str, dir);	/* if dir is NULL, idx filename is used */
+	pid = build_spawn_cmd(doc, cmdstr->str, dir); /* if dir is NULL, idx filename is used */
 	g_free(dir);
 	g_string_free(cmdstr, TRUE);
 	return pid;
 }
 
 
-static GPid build_compile_file(gint idx)
+static GPid build_compile_file(GeanyDocument *doc)
 {
-	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_name == NULL)
+	if (doc == NULL || doc->file_name == NULL)
 		return (GPid) 1;
 
 	build_info.type = GBO_COMPILE;
-	return build_spawn_cmd(idx, doc_list[idx].file_type->programs->compiler, NULL);
+	return build_spawn_cmd(doc, doc->file_type->programs->compiler, NULL);
 }
 
 
-static GPid build_link_file(gint idx)
+static GPid build_link_file(GeanyDocument *doc)
 {
-	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_name == NULL)
+	if (doc == NULL || doc->file_name == NULL)
 		return (GPid) 1;
 
 	build_info.type = GBO_BUILD;
-	return build_spawn_cmd(idx, doc_list[idx].file_type->programs->linker, NULL);
+	return build_spawn_cmd(doc, doc->file_type->programs->linker, NULL);
 }
 
 
 /* If linking, clear all error indicators in all documents.
  * Otherwise, just clear error indicators in document idx. */
-static void clear_errors(gint idx)
+static void clear_errors(GeanyDocument *doc)
 {
 	switch (build_info.type)
 	{
 		case GBO_COMPILE:
 		case GBO_MAKE_OBJECT:
-			document_clear_indicators(idx);
+			g_return_if_fail(doc);
+			editor_clear_indicators(doc->editor);
 			break;
 
 		case GBO_BUILD:
@@ -386,10 +390,10 @@ static void clear_errors(gint idx)
 		{
 			guint i;
 
-			for (i = 0; i < doc_array->len; i++)
+			for (i = 0; i < documents_array->len; i++)
 			{
-				if (doc_list[i].is_valid)
-					document_clear_indicators(i);
+				if (documents[i]->is_valid)
+					editor_clear_indicators(documents[i]->editor);
 			}
 			break;
 		}
@@ -422,7 +426,7 @@ static gchar *quote_executable(const gchar *cmd)
 
 /* dir is the UTF-8 working directory to run cmd in. It can be NULL to use the
  * idx document directory */
-static GPid build_spawn_cmd(gint idx, const gchar *cmd, const gchar *dir)
+static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *dir)
 {
 	GError  *error = NULL;
 	gchar  **argv;
@@ -436,12 +440,12 @@ static GPid build_spawn_cmd(gint idx, const gchar *cmd, const gchar *dir)
 	gint     stdout_fd;
 	gint     stderr_fd;
 
-	g_return_val_if_fail(DOC_IDX_VALID(idx), (GPid) 1);
+	g_return_val_if_fail(doc != NULL, (GPid) 1);
 
-	clear_errors(idx);
+	clear_errors(doc);
 	setptr(current_dir_entered, NULL);
 
-	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
+	locale_filename = utils_get_locale_from_utf8(doc->file_name);
 	executable = utils_remove_ext_from_filename(locale_filename);
 
 	cmd_string = g_strdup(cmd);
@@ -472,7 +476,7 @@ static GPid build_spawn_cmd(gint idx, const gchar *cmd, const gchar *dir)
 
 	utf8_cmd_string = utils_get_utf8_from_locale(cmd_string);
 	utf8_working_dir = (dir != NULL) ? g_strdup(dir) :
-		g_path_get_dirname(doc_list[idx].file_name);
+		g_path_get_dirname(doc->file_name);
 	working_dir = utils_get_locale_from_utf8(utf8_working_dir);
 
 	gtk_list_store_clear(msgwindow.store_compiler);
@@ -484,7 +488,7 @@ static GPid build_spawn_cmd(gint idx, const gchar *cmd, const gchar *dir)
 	/* set the build info for the message window */
 	g_free(build_info.dir);
 	build_info.dir = g_strdup(working_dir);
-	build_info.file_type_id = FILETYPE_ID(doc_list[idx].file_type);
+	build_info.file_type_id = FILETYPE_ID(doc->file_type);
 
 	if (! g_spawn_async_with_pipes(working_dir, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
 						NULL, NULL, &(build_info.pid), NULL, &stdout_fd, &stderr_fd, &error))
@@ -502,7 +506,7 @@ static GPid build_spawn_cmd(gint idx, const gchar *cmd, const gchar *dir)
 	if (build_info.pid > 0)
 	{
 		g_child_watch_add(build_info.pid, (GChildWatchFunc) build_exit_cb, NULL);
-		build_menu_update(idx);
+		build_menu_update(doc);
 	}
 
 	/* use GIOChannels to monitor stdout and stderr */
@@ -586,12 +590,12 @@ static gchar *get_build_executable(const gchar *locale_filename, gboolean check_
 /* Returns: NULL if there was an error, or the working directory the script was created in.
  * vte_cmd_nonscript is the location of a string which is filled with the command to be used
  * when vc->skip_run_script is set, otherwise it will be set to NULL */
-static gchar *prepare_run_script(gint idx, gchar **vte_cmd_nonscript)
+static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript)
 {
 	gchar	*locale_filename = NULL;
 	gboolean have_project;
 	GeanyProject *project = app->project;
-	filetype *ft = doc_list[idx].file_type;
+	GeanyFiletype *ft = doc->file_type;
 	gboolean check_exists;
 	gchar	*cmd = NULL;
 	gchar	*executable = NULL;
@@ -603,7 +607,7 @@ static gchar *prepare_run_script(gint idx, gchar **vte_cmd_nonscript)
 	if (vte_cmd_nonscript != NULL)
 		*vte_cmd_nonscript = NULL;
 
-	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
+	locale_filename = utils_get_locale_from_utf8(doc->file_name);
 
 	have_project = (project != NULL && NZV(project->run_cmd));
 	cmd = (have_project) ?
@@ -688,22 +692,20 @@ static gchar *prepare_run_script(gint idx, gchar **vte_cmd_nonscript)
 }
 
 
-static GPid build_run_cmd(gint idx)
+static GPid build_run_cmd(GeanyDocument *doc)
 {
 	gchar	*working_dir;
 	gchar	*vte_cmd_nonscript = NULL;
 	GError	*error = NULL;
 
-	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_name == NULL)
+	if (doc == NULL || doc->file_name == NULL)
 		return (GPid) 0;
 
-	working_dir = prepare_run_script(idx, &vte_cmd_nonscript);
+	working_dir = prepare_run_script(doc, &vte_cmd_nonscript);
 	if (working_dir == NULL)
-	{
 		return (GPid) 0;
-	}
 
-	run_info.file_type_id = FILETYPE_ID(doc_list[idx].file_type);
+	run_info.file_type_id = FILETYPE_ID(doc->file_type);
 
 #ifdef HAVE_VTE
 	if (vte_info.load_vte && vc != NULL && vc->run_in_vte)
@@ -718,7 +720,7 @@ static GPid build_run_cmd(gint idx)
 			g_free(vte_cmd_nonscript);
 		}
 		else
-			vte_cmd = g_strconcat(RUN_SCRIPT_CMD, "\n", NULL);
+			vte_cmd = g_strconcat("\n/bin/sh ", RUN_SCRIPT_CMD, "\n", NULL);
 
 		/* change into current directory if it is not done by default or we have a project and
 		 * project run command(working_dir is already set accordingly) */
@@ -751,7 +753,7 @@ static GPid build_run_cmd(gint idx)
 		gchar  **argv = NULL;
 
 		/* get the terminal path */
-		locale_term_cmd = utils_get_locale_from_utf8(prefs.tools_term_cmd);
+		locale_term_cmd = utils_get_locale_from_utf8(tool_prefs.term_cmd);
 		/* split the term_cmd, so arguments will work too */
 		term_argv = g_strsplit(locale_term_cmd, " ", -1);
 		term_argv_len = g_strv_length(term_argv);
@@ -768,7 +770,7 @@ static GPid build_run_cmd(gint idx)
 		{
 			ui_set_statusbar(TRUE,
 				_("Could not find terminal \"%s\" "
-					"(check path for Terminal tool setting in Preferences)"), prefs.tools_term_cmd);
+					"(check path for Terminal tool setting in Preferences)"), tool_prefs.term_cmd);
 			run_info.pid = (GPid) 1;
 			goto free_strings;
 		}
@@ -792,7 +794,7 @@ static GPid build_run_cmd(gint idx)
 		}
 #else
 		argv[term_argv_len   ]  = g_strdup("-e");
-		argv[term_argv_len + 1] = g_strdup(RUN_SCRIPT_CMD);
+		argv[term_argv_len + 1] = g_strconcat("/bin/sh ", RUN_SCRIPT_CMD, NULL);
 #endif
 		argv[term_argv_len + 2] = NULL;
 
@@ -810,7 +812,7 @@ static GPid build_run_cmd(gint idx)
 		if (run_info.pid > 0)
 		{
 			g_child_watch_add(run_info.pid, (GChildWatchFunc) run_exit_cb, NULL);
-			build_menu_update(idx);
+			build_menu_update(doc);
 		}
 		free_strings:
 		g_strfreev(argv);
@@ -853,9 +855,10 @@ static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
 					&filename, &line);
 				if (line != -1 && filename != NULL)
 				{
-					gint idx = document_find_by_filename(filename, FALSE);
+					GeanyDocument *doc = document_find_by_filename(filename);
 
-					document_set_indicator(idx, line - 1);	/* will check valid idx */
+					if (doc)
+						editor_set_indicator_on_line(doc->editor, line - 1);
 					color = COLOR_RED;	/* error message parsed on the line */
 				}
 				g_free(filename);
@@ -965,7 +968,7 @@ static void build_exit_cb(GPid child_pid, gint status, gpointer user_data)
 
 	build_info.pid = 0;
 	/* enable build items again */
-	build_menu_update(-1);
+	build_menu_update(NULL);
 }
 
 
@@ -975,7 +978,7 @@ static void run_exit_cb(GPid child_pid, gint status, gpointer user_data)
 
 	run_info.pid = 0;
 	/* reset the stop button and menu item to the original meaning */
-	build_menu_update(-1);
+	build_menu_update(NULL);
 }
 
 
@@ -1000,13 +1003,6 @@ static gboolean build_create_shellscript(const gchar *fname, const gchar *cmd, g
 	fputs(str, fp);
 	g_free(str);
 
-#ifndef G_OS_WIN32
-	if (chmod(fname, 0700) != 0)
-	{
-		g_unlink(fname);
-		return FALSE;
-	}
-#endif
 	fclose(fp);
 
 	return TRUE;
@@ -1016,10 +1012,10 @@ static gboolean build_create_shellscript(const gchar *fname, const gchar *cmd, g
 /* note: copied from keybindings.c.
  * Perhaps the separate Tex menu could be merged with the default build menu?
  * Then this could be done with Glade and set the accels in keybindings.c. */
-static void add_menu_accel(KeyBindingGroup *group, guint kb_id,
+static void add_menu_accel(GeanyKeyGroup *group, guint kb_id,
 	GtkAccelGroup *accel_group, GtkWidget *menuitem)
 {
-	KeyBinding *kb = &group->keys[kb_id];
+	GeanyKeyBinding *kb = &group->keys[kb_id];
 
 	if (kb->key != 0)
 		gtk_widget_add_accelerator(menuitem, "activate", accel_group,
@@ -1034,8 +1030,8 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 {
 	GtkWidget *menu, *item = NULL, *image, *separator;
 	GtkAccelGroup *accel_group = gtk_accel_group_new();
-	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
-	KeyBindingGroup *group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_BUILD);
+	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
+	GeanyKeyGroup *group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_BUILD);
 
 	menu = gtk_menu_new();
 
@@ -1048,7 +1044,7 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	image = gtk_image_new_from_stock("gtk-convert", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_compile_activate), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_compile_activate), NULL);
 	menu_items->item_compile = item;
 
 	/* build the code */
@@ -1058,7 +1054,7 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	gtk_tooltips_set_tip(tooltips, item,
 				_("Builds the current file (generate an executable file)"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_LINK, item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_build_activate), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_build_activate), NULL);
 	menu_items->item_link = item;
 
 	item = gtk_separator_menu_item_new();
@@ -1072,7 +1068,7 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	gtk_tooltips_set_tip(tooltips, item, _("Builds the current file with the "
 										   "make tool and the default target"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_MAKE, item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_make_activate),
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_ALL));
 	menu_items->item_make_all = item;
 
@@ -1083,7 +1079,7 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	gtk_tooltips_set_tip(tooltips, item, _("Builds the current file with the "
 										   "make tool and the specified target"), NULL);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_make_activate),
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_CUSTOM));
 	menu_items->item_make_custom = item;
 
@@ -1094,7 +1090,7 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	gtk_tooltips_set_tip(tooltips, item, _("Compiles the current file using the "
 										   "make tool"), NULL);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_make_activate),
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_OBJECT));
 	menu_items->item_make_object = item;
 
@@ -1107,9 +1103,16 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	gtk_widget_show(item);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_NEXTERROR, item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_next_error), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_next_error), NULL);
 	menu_items->item_next_error = item;
 
+	item = gtk_image_menu_item_new_with_mnemonic(_("_Previous Error"));
+	gtk_widget_show(item);
+	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_PREVIOUSERROR, item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_previous_error), NULL);
+	menu_items->item_previous_error = item;
+
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
@@ -1120,7 +1123,7 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	gtk_tooltips_set_tip(tooltips, item, _("Run or view the current file"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_RUN, item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_execute_activate), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_execute_activate), NULL);
 	menu_items->item_exec = item;
 
 	separator = gtk_separator_menu_item_new();
@@ -1139,7 +1142,7 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	image = gtk_image_new_from_stock("gtk-preferences", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_arguments_activate), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_arguments_activate), NULL);
 	menu_items->item_set_args = item;
 
 	menu_items->menu = menu;
@@ -1151,8 +1154,8 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 {
 	GtkWidget *menu, *item, *image, *separator;
 	GtkAccelGroup *accel_group = gtk_accel_group_new();
-	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
-	KeyBindingGroup *group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_BUILD);
+	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
+	GeanyKeyGroup *group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_BUILD);
 
 	menu = gtk_menu_new();
 
@@ -1165,7 +1168,7 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	image = gtk_image_new_from_stock("gtk-convert", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
-	g_signal_connect((gpointer) item, "activate",
+	g_signal_connect(item, "activate",
 				G_CALLBACK(on_build_tex_activate), GINT_TO_POINTER(LATEX_CMD_TO_DVI));
 	menu_items->item_compile = item;
 
@@ -1178,7 +1181,7 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	image = gtk_image_new_from_stock("gtk-convert", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
-	g_signal_connect((gpointer) item, "activate",
+	g_signal_connect(item, "activate",
 				G_CALLBACK(on_build_tex_activate), GINT_TO_POINTER(LATEX_CMD_TO_PDF));
 	menu_items->item_link = item;
 
@@ -1193,7 +1196,7 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	gtk_tooltips_set_tip(tooltips, item, _("Builds the current file with the "
 										   "make tool and the default target"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_MAKE, item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_make_activate),
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_ALL));
 	menu_items->item_make_all = item;
 
@@ -1204,7 +1207,7 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	gtk_tooltips_set_tip(tooltips, item, _("Builds the current file with the "
 										   "make tool and the specified target"), NULL);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_make_activate),
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_CUSTOM));
 	menu_items->item_make_custom = item;
 
@@ -1217,9 +1220,16 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	gtk_widget_show(item);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_NEXTERROR, item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_next_error), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_next_error), NULL);
 	menu_items->item_next_error = item;
 
+	item = gtk_image_menu_item_new_with_mnemonic(_("_Previous Error"));
+	gtk_widget_show(item);
+	/*GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_, item);*/
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_previous_error), NULL);
+	menu_items->item_previous_error = item;
+
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
@@ -1234,8 +1244,8 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	image = gtk_image_new_from_stock("gtk-find", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
-	g_signal_connect((gpointer) item, "activate",
-						G_CALLBACK(on_build_execute_activate), GINT_TO_POINTER(LATEX_CMD_VIEW_DVI));
+	g_signal_connect(item, "activate",
+					G_CALLBACK(on_build_execute_activate), GINT_TO_POINTER(LATEX_CMD_VIEW_DVI));
 	menu_items->item_exec = item;
 
 	/* PDF view */
@@ -1247,8 +1257,8 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	image = gtk_image_new_from_stock("gtk-find", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
-	g_signal_connect((gpointer) item, "activate",
-						G_CALLBACK(on_build_execute_activate), GINT_TO_POINTER(LATEX_CMD_VIEW_PDF));
+	g_signal_connect(item, "activate",
+					G_CALLBACK(on_build_execute_activate), GINT_TO_POINTER(LATEX_CMD_VIEW_PDF));
 	menu_items->item_exec2 = item;
 
 	/* separator */
@@ -1267,11 +1277,11 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	image = gtk_image_new_from_stock("gtk-preferences", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
-	g_signal_connect((gpointer) item, "activate",
+	g_signal_connect(item, "activate",
 		G_CALLBACK(on_build_arguments_activate), filetypes[GEANY_FILETYPES_LATEX]);
 	menu_items->item_set_args = item;
 
-	gtk_window_add_accel_group(GTK_WINDOW(app->window), accel_group);
+	gtk_window_add_accel_group(GTK_WINDOW(main_widgets.window), accel_group);
 
 	menu_items->menu = menu;
 	g_object_ref((gpointer)menu_items->menu);	/* to hold it after removing */
@@ -1283,7 +1293,7 @@ on_includes_arguments_tex_dialog_response  (GtkDialog *dialog,
                                             gint response,
                                             gpointer user_data)
 {
-	filetype *ft = user_data;
+	GeanyFiletype *ft = user_data;
 	g_return_if_fail(ft != NULL);
 
 	if (response == GTK_RESPONSE_ACCEPT)
@@ -1330,14 +1340,15 @@ on_includes_arguments_tex_dialog_response  (GtkDialog *dialog,
 static void show_includes_arguments_tex(void)
 {
 	GtkWidget *dialog, *label, *entries[4], *vbox, *table;
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	gint response;
-	filetype *ft = NULL;
+	GeanyFiletype *ft = NULL;
 
-	if (DOC_IDX_VALID(idx)) ft = doc_list[idx].file_type;
+	if (doc != NULL)
+		ft = doc->file_type;
 	g_return_if_fail(ft != NULL);
 
-	dialog = gtk_dialog_new_with_buttons(_("Set Arguments"), GTK_WINDOW(app->window),
+	dialog = gtk_dialog_new_with_buttons(_("Set Arguments"), GTK_WINDOW(main_widgets.window),
 										GTK_DIALOG_DESTROY_WITH_PARENT,
 										GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
@@ -1448,7 +1459,7 @@ on_includes_arguments_dialog_response  (GtkDialog *dialog,
                                         gint response,
                                         gpointer user_data)
 {
-	filetype *ft = user_data;
+	GeanyFiletype *ft = user_data;
 
 	g_return_if_fail(ft != NULL);
 
@@ -1499,14 +1510,15 @@ static void show_includes_arguments_gen(void)
 	GtkWidget *dialog, *label, *entries[3], *vbox;
 	GtkWidget *ft_table = NULL;
 	gint row = 0;
-	gint idx = document_get_cur_idx();
 	gint response;
-	filetype *ft = NULL;
+	GeanyDocument *doc = document_get_current();
+	GeanyFiletype *ft = NULL;
 
-	if (DOC_IDX_VALID(idx)) ft = doc_list[idx].file_type;
+	if (doc != NULL)
+		ft = doc->file_type;
 	g_return_if_fail(ft != NULL);
 
-	dialog = gtk_dialog_new_with_buttons(_("Set Includes and Arguments"), GTK_WINDOW(app->window),
+	dialog = gtk_dialog_new_with_buttons(_("Set Includes and Arguments"), GTK_WINDOW(main_widgets.window),
 										GTK_DIALOG_DESTROY_WITH_PARENT,
 										GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
@@ -1619,7 +1631,7 @@ static void
 on_build_arguments_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	if (user_data && FILETYPE_ID((filetype*) user_data) == GEANY_FILETYPES_LATEX)
+	if (user_data && FILETYPE_ID((GeanyFiletype*) user_data) == GEANY_FILETYPES_LATEX)
 		show_includes_arguments_tex();
 	else
 		show_includes_arguments_gen();
@@ -1640,43 +1652,42 @@ static gboolean is_c_header(const gchar *fname)
 
 /* Call this whenever build menu items need to be enabled/disabled.
  * Uses current document (if there is one) when idx == -1 */
-void build_menu_update(gint idx)
+void build_menu_update(GeanyDocument *doc)
 {
-	filetype *ft;
+	GeanyFiletype *ft;
 	gboolean have_path, can_build, can_make, can_run, can_stop, can_set_args, have_errors;
 	BuildMenuItems *menu_items;
 
-	if (idx == -1)
-		idx = document_get_cur_idx();
-	if (idx == -1 ||
-		(FILETYPE_ID(doc_list[idx].file_type) == GEANY_FILETYPES_ALL &&
-			doc_list[idx].file_name == NULL))
+	if (doc == NULL)
+		doc = document_get_current();
+	if (doc == NULL ||
+		(FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_NONE &&	doc->file_name == NULL))
 	{
-		gtk_widget_set_sensitive(lookup_widget(app->window, "menu_build1"), FALSE);
-		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(lookup_widget(app->window, "menu_build1")));
+		gtk_widget_set_sensitive(lookup_widget(main_widgets.window, "menu_build1"), FALSE);
+		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(lookup_widget(main_widgets.window, "menu_build1")));
 		gtk_widget_set_sensitive(widgets.compile_button, FALSE);
 		gtk_widget_set_sensitive(widgets.run_button, FALSE);
 		return;
 	}
 	else
-		gtk_widget_set_sensitive(lookup_widget(app->window, "menu_build1"), TRUE);
+		gtk_widget_set_sensitive(lookup_widget(main_widgets.window, "menu_build1"), TRUE);
 
-	ft = doc_list[idx].file_type;
+	ft = doc->file_type;
 	g_return_if_fail(ft != NULL);
 
 	menu_items = build_get_menu_items(ft->id);
 	/* Note: don't remove the submenu first because it can now cause an X hang if
 	 * the menu is already open when called from build_exit_cb(). */
-	gtk_menu_item_set_submenu(GTK_MENU_ITEM(lookup_widget(app->window, "menu_build1")),
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(lookup_widget(main_widgets.window, "menu_build1")),
 		menu_items->menu);
 
-	have_path = (doc_list[idx].file_name != NULL);
+	have_path = (doc->file_name != NULL);
 
 	can_make = have_path && build_info.pid <= (GPid) 1;
 
 	/* disable compile and link for C/C++ header files */
 	if (ft->id == GEANY_FILETYPES_C || ft->id == GEANY_FILETYPES_CPP)
-		can_build = can_make && ! is_c_header(doc_list[idx].file_name);
+		can_build = can_make && ! is_c_header(doc->file_name);
 	else
 		can_build = can_make;
 
@@ -1710,7 +1721,7 @@ void build_menu_update(gint idx)
 		((ft->actions->can_compile ||
 		ft->actions->can_link ||
 		ft->actions->can_exec) &&
-		FILETYPE_ID(ft) != GEANY_FILETYPES_ALL);
+		FILETYPE_ID(ft) != GEANY_FILETYPES_NONE);
 	if (menu_items->item_set_args)
 		gtk_widget_set_sensitive(menu_items->item_set_args, can_set_args);
 
@@ -1725,6 +1736,8 @@ void build_menu_update(gint idx)
 		NULL) > 0;
 	if (menu_items->item_next_error)
 		gtk_widget_set_sensitive(menu_items->item_next_error, have_errors);
+	if (menu_items->item_previous_error)
+		gtk_widget_set_sensitive(menu_items->item_previous_error, have_errors);
 }
 
 
@@ -1783,18 +1796,18 @@ static void set_stop_button(gboolean stop)
 
 
 /* Creates the relevant build menu if necessary.
- * If filetype_idx is -1, the current filetype is used, or GEANY_FILETYPES_ALL */
+ * If filetype_idx is -1, the current filetype is used, or GEANY_FILETYPES_NONE */
 BuildMenuItems *build_get_menu_items(gint filetype_idx)
 {
 	BuildMenuItems *items;
 
 	if (filetype_idx == -1)
 	{
-		gint idx = document_get_cur_idx();
-		filetype *ft = NULL;
+		GeanyDocument *doc = document_get_current();
+		GeanyFiletype *ft = NULL;
 
-		if (DOC_IDX_VALID(idx))
-			ft = doc_list[idx].file_type;
+		if (doc != NULL)
+			ft = doc->file_type;
 		filetype_idx = FILETYPE_ID(ft);
 	}
 
@@ -1818,16 +1831,18 @@ static void
 on_build_compile_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 
-	if (! DOC_IDX_VALID(idx)) return;
+	if (doc == NULL)
+		return;
 
-	if (doc_list[idx].changed) document_save_file(idx, FALSE);
+	if (doc->changed)
+		document_save_file(doc, FALSE);
 
-	if (FILETYPE_ID(doc_list[idx].file_type) == GEANY_FILETYPES_LATEX)
-		build_compile_tex_file(idx, 0);
+	if (FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_LATEX)
+		build_compile_tex_file(doc, 0);
 	else
-		build_compile_file(idx);
+		build_compile_file(doc);
 }
 
 
@@ -1835,21 +1850,22 @@ static void
 on_build_tex_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 
-	if (! DOC_IDX_VALID(idx))
+	if (doc == NULL)
 		return;
 
-	if (doc_list[idx].changed) document_save_file(idx, FALSE);
+	if (doc->changed)
+		document_save_file(doc, FALSE);
 
 	switch (GPOINTER_TO_INT(user_data))
 	{
 		case LATEX_CMD_TO_DVI:
 		case LATEX_CMD_TO_PDF:
-			build_compile_tex_file(idx, GPOINTER_TO_INT(user_data)); break;
+			build_compile_tex_file(doc, GPOINTER_TO_INT(user_data)); break;
 		case LATEX_CMD_VIEW_DVI:
 		case LATEX_CMD_VIEW_PDF:
-			build_view_tex_file(idx, GPOINTER_TO_INT(user_data)); break;
+			build_view_tex_file(doc, GPOINTER_TO_INT(user_data)); break;
 	}
 }
 
@@ -1858,30 +1874,32 @@ static void
 on_build_build_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 
-	if (! DOC_IDX_VALID(idx)) return;
+	if (doc == NULL)
+		return;
 
-	if (doc_list[idx].changed) document_save_file(idx, FALSE);
+	if (doc->changed)
+		document_save_file(doc, FALSE);
 
-	if (FILETYPE_ID(doc_list[idx].file_type) == GEANY_FILETYPES_LATEX)
-		build_compile_tex_file(idx, 1);
+	if (FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_LATEX)
+		build_compile_tex_file(doc, 1);
 	else
-		build_link_file(idx);
+		build_link_file(doc);
 }
 
 
 static void
 on_make_custom_input_response(const gchar *input)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 
-	if (doc_list[idx].changed)
-		document_save_file(idx, FALSE);
+	if (doc->changed)
+		document_save_file(doc, FALSE);
 
 	setptr(build_info.custom_target, g_strdup(input));
 
-	build_make_file(idx, GBO_MAKE_CUSTOM);
+	build_make_file(doc, GBO_MAKE_CUSTOM);
 }
 
 
@@ -1905,10 +1923,10 @@ static void
 on_build_make_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	gint build_opts = GPOINTER_TO_INT(user_data);
 
-	g_return_if_fail(DOC_IDX_VALID(idx) && doc_list[idx].file_name != NULL);
+	g_return_if_fail(doc != NULL && doc->file_name != NULL);
 
 	switch (build_opts)
 	{
@@ -1922,15 +1940,16 @@ on_build_make_activate                 (GtkMenuItem     *menuitem,
 		/* fall through */
 		case GBO_MAKE_ALL:
 		{
-			if (doc_list[idx].changed) document_save_file(idx, FALSE);
+			if (doc->changed)
+				document_save_file(doc, FALSE);
 
-			build_make_file(idx, build_opts);
+			build_make_file(doc, build_opts);
 		}
 	}
 }
 
 
-static gboolean use_html_builtin(gint idx, filetype *ft)
+static gboolean use_html_builtin(GeanyDocument *doc, GeanyFiletype *ft)
 {
 	gboolean use_builtin = FALSE;
 	if (ft->id == GEANY_FILETYPES_HTML)
@@ -1948,7 +1967,7 @@ static gboolean use_html_builtin(gint idx, filetype *ft)
 
 	if (use_builtin)
 	{
-		gchar *uri = g_strconcat("file:///", g_path_skip_root(doc_list[idx].file_name), NULL);
+		gchar *uri = g_strconcat("file:///", g_path_skip_root(doc->file_name), NULL);
 		utils_start_browser(uri);
 		g_free(uri);
 
@@ -1962,11 +1981,11 @@ static void
 on_build_execute_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	filetype_id ft_id;
-	filetype *ft;
+	GeanyFiletype *ft;
 
-	if (! DOC_IDX_VALID(idx))
+	if (doc == NULL)
 		return;
 
 	/* make the process "stopable" */
@@ -1976,29 +1995,26 @@ on_build_execute_activate              (GtkMenuItem     *menuitem,
 		return;
 	}
 
-	ft_id = FILETYPE_ID(doc_list[idx].file_type);
+	ft_id = FILETYPE_ID(doc->file_type);
 	ft = filetypes[ft_id];
 	if (ft_id == GEANY_FILETYPES_LATEX)
 	{	/* run LaTeX file */
-		if (build_view_tex_file(idx, GPOINTER_TO_INT(user_data)) == (GPid) 0)
+		if (build_view_tex_file(doc, GPOINTER_TO_INT(user_data)) == (GPid) 0)
 		{
 			ui_set_statusbar(TRUE, _("Failed to execute the view program"));
 		}
 	}
 	/* use_html_builtin() checks for HTML builtin request and returns FALSE if not */
-	else if (! use_html_builtin(idx, ft))
+	else if (! use_html_builtin(doc, ft))
 	{	/* run everything else */
 
 		/* save the file only if the run command uses it */
-		if (doc_list[idx].changed &&
+		if (doc->changed &&
 			NZV(ft->programs->run_cmd) &&	/* can happen when project is open */
 			strstr(ft->programs->run_cmd, "%f") != NULL)
-				document_save_file(idx, FALSE);
+				document_save_file(doc, FALSE);
 
-		if (build_run_cmd(idx) == (GPid) 0)
-		{
-			ui_set_statusbar(TRUE, _("Failed to execute the terminal program"));
-		}
+		build_run_cmd(doc);
 	}
 }
 
@@ -2027,7 +2043,7 @@ static void kill_process(GPid *pid)
 	else
 	{
 		*pid = 0;
-		build_menu_update(-1);
+		build_menu_update(NULL);
 	}
 }
 
@@ -2046,8 +2062,22 @@ on_build_next_error                    (GtkMenuItem     *menuitem,
 }
 
 
+static void
+on_build_previous_error                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	if (ui_tree_view_find_previous(GTK_TREE_VIEW(msgwindow.tree_compiler),
+		msgwin_goto_compiler_file_line))
+	{
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_COMPILER);
+	}
+	else
+		ui_set_statusbar(FALSE, _("No more build errors."));
+}
+
+
 void build_init()
 {
-	widgets.compile_button = lookup_widget(app->window, "toolbutton13");
-	widgets.run_button = lookup_widget(app->window, "toolbutton26");
+	widgets.compile_button = lookup_widget(main_widgets.window, "toolbutton_compile");
+	widgets.run_button = lookup_widget(main_widgets.window, "toolbutton_run");
 }
