@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: build.h 2287 2008-02-27 13:17:29Z eht16 $
+ * $Id: build.h 2727 2008-06-26 16:17:28Z ntrel $
  */
 
 
@@ -32,18 +32,18 @@ typedef enum	/* Geany Build Options */
 	GBO_MAKE_ALL,
 	GBO_MAKE_CUSTOM,
 	GBO_MAKE_OBJECT
-} build_type;
+} GeanyBuildType;
 
-typedef struct BuildInfo
+typedef struct GeanyBuildInfo
 {
-	build_type	type;	/* current action(one of the above enumeration) */
-	GPid		pid;	/* process id of the spawned process */
-	gchar		*dir;
-	guint		file_type_id;
-	gchar		*custom_target;
-} BuildInfo;
+	GeanyBuildType	 type;	/* current action(one of the above enumeration) */
+	GPid			 pid;	/* process id of the spawned process */
+	gchar			*dir;
+	guint			 file_type_id;
+	gchar			*custom_target;
+} GeanyBuildInfo;
 
-extern BuildInfo build_info;
+extern GeanyBuildInfo build_info;
 
 
 typedef struct BuildMenuItems
@@ -55,6 +55,7 @@ typedef struct BuildMenuItems
 	GtkWidget		*item_make_custom;
 	GtkWidget		*item_make_object;
 	GtkWidget		*item_next_error;
+	GtkWidget		*item_previous_error;
 	GtkWidget		*item_exec;
 	GtkWidget		*item_exec2;
 	GtkWidget		*item_set_args;
@@ -69,7 +70,7 @@ void build_finalize(void);
 
 gboolean build_parse_make_dir(const gchar *string, gchar **prefix);
 
-void build_menu_update(gint idx);
+void build_menu_update(GeanyDocument *doc);
 
 BuildMenuItems *build_get_menu_items(gint filetype_idx);
 
