@@ -18,12 +18,16 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.h 2488 2008-04-16 11:34:34Z ntrel $
+ * $Id: callbacks.h 3094 2008-10-15 11:59:32Z ntrel $
  */
 
+
+#include "geany.h" /* necessary for interface.c */
+
+
 typedef struct
 {
-	gint last_doc_idx;
+	GeanyDocument *last_doc;
 } CallbacksData;
 
 extern CallbacksData callbacks_data;
@@ -56,36 +60,12 @@ void
 on_open1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_toolbutton19_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton6_clicked                 (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
 void
 on_change_font1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_toolbutton8_clicked                 (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton9_clicked                 (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton10_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton15_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton23_clicked                (GtkToolButton   *toolbutton,
+on_toolbutton_close_clicked            (GtkToolButton   *toolbutton,
                                         gpointer         user_data);
 
 void
@@ -123,11 +103,11 @@ on_replace_tabs_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_toolbutton18_clicked                (GtkToolButton   *toolbutton,
+on_toolbutton_search_clicked           (GtkToolButton   *toolbutton,
                                         gpointer         user_data);
 
 void
-on_entry1_activate                     (GtkEntry        *entry,
+on_toolbar_search_entry_activate       (GtkEntry        *entry,
                                         gpointer         user_data);
 
 gboolean
@@ -175,14 +155,6 @@ void
 on_delete1_activate                    (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_toolbutton20_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbutton21_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
 void
 on_zoom_in1_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -192,7 +164,7 @@ on_zoom_out1_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_entry1_changed                      (GtkEditable     *editable,
+on_toolbar_search_entry_changed        (GtkEditable     *editable,
                                         gpointer         user_data);
 
 void
@@ -219,10 +191,6 @@ void
 on_invisible1_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_toolbutton13_clicked                (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
 void
 on_find_usage1_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -271,10 +239,10 @@ void
 on_find_previous1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 void
-on_find_nextsel1_activate             (GtkMenuItem     *menuitem,
+on_find_nextsel1_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 void
-on_find_prevsel1_activate             (GtkMenuItem     *menuitem,
+on_find_prevsel1_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
@@ -289,6 +257,22 @@ void
 on_toolbutton_new_clicked              (GtkToolButton   *toolbutton,
                                         gpointer         user_data);
 
+void
+on_toolbutton_open_clicked             (GtkToolButton   *toolbutton,
+                                        gpointer         user_data);
+
+void
+on_toolbutton_save_clicked             (GtkToolButton   *toolbutton,
+                                        gpointer         user_data);
+
+void
+on_toolbutton_quit_clicked             (GtkToolButton   *toolbutton,
+                                        gpointer         user_data);
+
+void
+on_toolbutton_reload_clicked           (GtkToolButton   *toolbutton,
+                                        gpointer         user_data);
+
 void
 on_go_to_line_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -316,7 +300,7 @@ on_website1_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_line_breaking1_toggled              (GtkCheckMenuItem *checkmenuitem,
+on_line_wrapping1_toggled              (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
 
 void
@@ -402,7 +386,7 @@ on_menu_unfold_all1_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_entry_goto_line_activate            (GtkEntry        *entry,
+on_toolbutton_goto_entry_activate      (GtkEntry        *entry,
                                         gpointer         user_data);
 
 void
@@ -585,11 +569,43 @@ void
 on_page_setup1_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_tools1_activate                     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 gboolean
 on_escape_key_press_event              (GtkWidget       *widget,
                                         GdkEventKey     *event,
                                         gpointer         user_data);
+
+void
+on_line_breaking1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+
+void
+on_replace_spaces_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_previous_message1_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_search1_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_close_other_documents1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_reload_configuration1_activate (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_find_document_usage1_activate       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_tabs_and_spaces1_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+void
+on_debug_messages1_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
