@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: encodings.c 2287 2008-02-27 13:17:29Z eht16 $
+ *  $Id: encodings.c 2987 2008-09-23 15:19:46Z ntrel $
  */
 
 /*
@@ -37,19 +37,26 @@
 #include "geany.h"
 #include "utils.h"
 #include "support.h"
+#include "document.h"
 #include "msgwindow.h"
 #include "encodings.h"
 #include "callbacks.h"
+#include "ui_utils.h"
 
 
 #ifdef HAVE_REGCOMP
-# include <regex.h>
+# ifdef HAVE_REGEX_H
+#  include <regex.h>
+# else
+#  include "gnuregex.h"
+# endif
 /* <meta http-equiv="content-type" content="text/html; charset=UTF-8" /> */
 # define PATTERN_HTMLMETA "<meta[ \t\n\r\f]http-equiv[ \t\n\r\f]*=[ \t\n\r\f]*\"content-type\"[ \t\n\r\f]+content[ \t\n\r\f]*=[ \t\n\r\f]*\"text/x?html;[ \t\n\r\f]*charset=([a-z0-9_-]+)\"[ \t\n\r\f]*/?>"
-/* " geany_encoding=utf-8 " */
-# define PATTERN_GEANY "[\t ]geany_encoding=([a-z0-9-]+)[\t ]"
+/* " geany_encoding=utf-8 " or " coding: utf-8 " */
+# define PATTERN_CODING "coding[\t ]*[:=][\t ]*([a-z0-9-]+)[\t ]*"
 /* precompiled regexps */
 static regex_t pregs[2];
+static gboolean pregs_loaded = FALSE;
 #endif
 
 
@@ -83,17 +90,19 @@ static void init_encodings(void)
 	fill(5, EASTEUROPEAN, GEANY_ENCODING_WINDOWS_1250, "WINDOWS-1250", _("Central European"));
 	fill(6, EASTEUROPEAN, GEANY_ENCODING_IBM_855, "IBM855", _("Cyrillic"));
 	fill(7, EASTEUROPEAN, GEANY_ENCODING_ISO_8859_5, "ISO-8859-5", _("Cyrillic"));
+	/* ISO-IR-111 not available on Windows */
 	fill(8, EASTEUROPEAN, GEANY_ENCODING_ISO_IR_111, "ISO-IR-111", _("Cyrillic"));
-	fill(9, EASTEUROPEAN, GEANY_ENCODING_KOI8_R, "KOI8R", _("Cyrillic"));
+	fill(9, EASTEUROPEAN, GEANY_ENCODING_KOI8_R, "KOI8-R", _("Cyrillic"));
 	fill(10, EASTEUROPEAN, GEANY_ENCODING_WINDOWS_1251, "WINDOWS-1251", _("Cyrillic"));
 	fill(11, EASTEUROPEAN, GEANY_ENCODING_CP_866, "CP866", _("Cyrillic/Russian"));
-	fill(12, EASTEUROPEAN, GEANY_ENCODING_KOI8_U, "KOI8U", _("Cyrillic/Ukrainian"));
+	fill(12, EASTEUROPEAN, GEANY_ENCODING_KOI8_U, "KOI8-U", _("Cyrillic/Ukrainian"));
 	fill(13, EASTEUROPEAN, GEANY_ENCODING_ISO_8859_16, "ISO-8859-16", _("Romanian"));
 
 	fill(0, MIDDLEEASTERN, GEANY_ENCODING_IBM_864, "IBM864", _("Arabic"));
 	fill(1, MIDDLEEASTERN, GEANY_ENCODING_ISO_8859_6, "ISO-8859-6", _("Arabic"));
 	fill(2, MIDDLEEASTERN, GEANY_ENCODING_WINDOWS_1256, "WINDOWS-1256", _("Arabic"));
 	fill(3, MIDDLEEASTERN, GEANY_ENCODING_IBM_862, "IBM862", _("Hebrew"));
+	/* not available at all, ? */
 	fill(4, MIDDLEEASTERN, GEANY_ENCODING_ISO_8859_8_I, "ISO-8859-8-I", _("Hebrew"));
 	fill(5, MIDDLEEASTERN, GEANY_ENCODING_WINDOWS_1255, "WINDOWS-1255", _("Hebrew"));
 	fill(6, MIDDLEEASTERN, GEANY_ENCODING_ISO_8859_8, "ISO-8859-8", _("Hebrew Visual"));
@@ -120,6 +129,7 @@ static void init_encodings(void)
 	fill(0, EASTASIAN, GEANY_ENCODING_GB18030, "GB18030", _("Chinese Simplified"));
 	fill(1, EASTASIAN, GEANY_ENCODING_GB2312, "GB2312", _("Chinese Simplified"));
 	fill(2, EASTASIAN, GEANY_ENCODING_GBK, "GBK", _("Chinese Simplified"));
+	/* maybe not available on Linux */
 	fill(3, EASTASIAN, GEANY_ENCODING_HZ, "HZ", _("Chinese Simplified"));
 	fill(4, EASTASIAN, GEANY_ENCODING_BIG5, "BIG5", _("Chinese Traditional"));
 	fill(5, EASTASIAN, GEANY_ENCODING_BIG5_HKSCS, "BIG5-HKSCS", _("Chinese Traditional"));
@@ -224,23 +234,29 @@ static GtkWidget *radio_items[GEANY_ENCODINGS_MAX];
 void encodings_select_radio_item(const gchar *charset)
 {
 	gint i;
+
+	if (ignore_callback)
+		return;
+
 	g_return_if_fail(charset != NULL);
 
 	i = 0;
 	while (i < GEANY_ENCODINGS_MAX)
 	{
-		if (utils_str_equal(charset, encodings[i].charset)) break;
+		if (utils_str_equal(charset, encodings[i].charset))
+			break;
 		i++;
 	}
-	if (i == GEANY_ENCODINGS_MAX) i = GEANY_ENCODING_UTF_8; /* fallback to UTF-8 */
+	if (i == GEANY_ENCODINGS_MAX)
+		i = GEANY_ENCODING_UTF_8; /* fallback to UTF-8 */
 
-	/* app->ignore_callback has to be set by the caller */
+	/* ignore_callback has to be set by the caller */
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(radio_items[i]), TRUE);
 }
 
 
 #ifdef HAVE_REGCOMP
-/* Regexp detection of file enconding declared in the file itself.
+/* Regexp detection of file encoding declared in the file itself.
  * Idea and parts of code taken from Bluefish, thanks.
  * regex_compile() is used to compile regular expressions on program init and keep it in memory
  * for faster access when opening a file. Pre-compiled regexps will be freed on program exit.
@@ -266,7 +282,7 @@ static gchar *regex_match(regex_t *preg, const gchar *buffer, gsize size)
 	gchar *encoding = NULL;
 	regmatch_t pmatch[10];
 
-	if (buffer == NULL || preg->buffer == NULL)
+	if (! pregs_loaded || buffer == NULL)
 		return NULL;
 
 	if (size > 512)
@@ -275,7 +291,7 @@ static gchar *regex_match(regex_t *preg, const gchar *buffer, gsize size)
 	retval = regexec(preg, (tmp_buf != NULL) ? tmp_buf : buffer, 10, pmatch, 0);
 	if (retval == 0 && pmatch[0].rm_so != -1 && pmatch[1].rm_so != -1)
 	{
-		encoding = g_strndup(&buffer[pmatch[1].rm_so], pmatch[1].rm_eo-pmatch[1].rm_so);
+		encoding = g_strndup(&buffer[pmatch[1].rm_so], pmatch[1].rm_eo - pmatch[1].rm_so);
 		geany_debug("Detected encoding by regex search: %s", encoding);
 
 		setptr(encoding, g_utf8_strup(encoding, -1));
@@ -289,10 +305,10 @@ static gchar *regex_match(regex_t *preg, const gchar *buffer, gsize size)
 void encodings_finalize(void)
 {
 #ifdef HAVE_REGCOMP
-	guint i;
-	for (i = 0; i < G_N_ELEMENTS(pregs); i++)
+	if (pregs_loaded)
 	{
-		if (pregs[i].buffer == NULL)
+		guint i;
+		for (i = 0; i < G_N_ELEMENTS(pregs); i++)
 		{
 			regfree(&pregs[i]);
 		}
@@ -315,13 +331,17 @@ void encodings_init(void)
 	init_encodings();
 
 #ifdef HAVE_REGCOMP
-	regex_compile(&pregs[0], PATTERN_HTMLMETA);
-	regex_compile(&pregs[1], PATTERN_GEANY);
+	if (! pregs_loaded)
+	{
+		regex_compile(&pregs[0], PATTERN_HTMLMETA);
+		regex_compile(&pregs[1], PATTERN_CODING);
+		pregs_loaded = TRUE;
+	}
 #endif
 
 	/* create encodings submenu in document menu */
-	menu[0] = lookup_widget(app->window, "set_encoding1_menu");
-	menu[1] = lookup_widget(app->window, "menu_reload_as1_menu");
+	menu[0] = lookup_widget(main_widgets.window, "set_encoding1_menu");
+	menu[1] = lookup_widget(main_widgets.window, "menu_reload_as1_menu");
 	cb_func[0] = G_CALLBACK(on_encoding_change);
 	cb_func[1] = G_CALLBACK(on_reload_as_activate);
 
@@ -395,7 +415,7 @@ void encodings_init(void)
 							item = gtk_menu_item_new_with_label(label);
 						gtk_widget_show(item);
 						gtk_container_add(GTK_CONTAINER(submenu), item);
-						g_signal_connect((gpointer) item, "activate",
+						g_signal_connect(item, "activate",
 										cb_func[k], GINT_TO_POINTER(encodings[j].idx));
 						g_free(label);
 						break;
