@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.h 2306 2008-03-05 17:09:57Z eht16 $
+ * $Id: filetypes.h 3018 2008-09-28 16:32:49Z eht16 $
  */
 
 
@@ -29,19 +29,23 @@
 #include "ScintillaWidget.h"
 
 
-/* each group should be alpha-sorted, based on filetype::name (not enum name) */
+/* Each group should be alpha-sorted, based on filetype::name (not enum name).
+ * Warning: remember to break the plugin ABI when adding items. */
+/* TODO: for a stable ABI put GEANY_FILETYPES_NONE first, and use a sorted duplicate
+ * filetypes array for GUI elements. */
 typedef enum
 {
 	/* normally compiled languages */
-	GEANY_FILETYPES_GROUP_COMPILED = 0,
-	GEANY_FILETYPES_ASM = GEANY_FILETYPES_GROUP_COMPILED,
+	GEANY_FILETYPES_ASM = 0,
 	GEANY_FILETYPES_C,
 	GEANY_FILETYPES_CPP,
 	GEANY_FILETYPES_CS,
 	GEANY_FILETYPES_CAML,
 	GEANY_FILETYPES_D,
+	GEANY_FILETYPES_F77,
 	GEANY_FILETYPES_FORTRAN,
 	GEANY_FILETYPES_BASIC,	/* FreeBasic */
+	GEANY_FILETYPES_GLSL,
 	GEANY_FILETYPES_HASKELL,
 	GEANY_FILETYPES_HAXE,
 	GEANY_FILETYPES_JAVA,
@@ -49,8 +53,7 @@ typedef enum
 	GEANY_FILETYPES_VHDL,
 
 	/* script languages */
-	GEANY_FILETYPES_GROUP_SCRIPT,
-	GEANY_FILETYPES_FERITE = GEANY_FILETYPES_GROUP_SCRIPT,
+	GEANY_FILETYPES_FERITE,
 	GEANY_FILETYPES_JS,
 	GEANY_FILETYPES_LUA,
 	GEANY_FILETYPES_MAKE,
@@ -58,32 +61,43 @@ typedef enum
 	GEANY_FILETYPES_PERL,
 	GEANY_FILETYPES_PHP,
 	GEANY_FILETYPES_PYTHON,
+	GEANY_FILETYPES_R,
 	GEANY_FILETYPES_RUBY,
 	GEANY_FILETYPES_SH,
 	GEANY_FILETYPES_TCL,
 
- 	/* markup langauges */
-	GEANY_FILETYPES_GROUP_MARKUP,
-	GEANY_FILETYPES_CSS = GEANY_FILETYPES_GROUP_MARKUP,
+	/* markup langauges */
+	GEANY_FILETYPES_CSS,
 	GEANY_FILETYPES_DOCBOOK,
 	GEANY_FILETYPES_HTML,
  	GEANY_FILETYPES_XML,
 
 	/* miscellaneous languages */
-	GEANY_FILETYPES_GROUP_MISC,
-	GEANY_FILETYPES_CONF = GEANY_FILETYPES_GROUP_MISC,
+	GEANY_FILETYPES_CONF,
 	GEANY_FILETYPES_DIFF,
 	GEANY_FILETYPES_LATEX,
 	GEANY_FILETYPES_REST,
 	GEANY_FILETYPES_SQL,
+	GEANY_FILETYPES_PO,
 
-	GEANY_FILETYPES_ALL,	/* must be last filetype, used for 'None' item. */
-	GEANY_MAX_FILE_TYPES
+	GEANY_FILETYPES_NONE,	/* must be last filetype */
+	GEANY_MAX_BUILT_IN_FILETYPES	/* Use filetypes_array->len instead */
 } filetype_id;
 
+typedef enum
+{
+	GEANY_FILETYPE_GROUP_NONE,
+	GEANY_FILETYPE_GROUP_COMPILED,
+	GEANY_FILETYPE_GROUP_SCRIPT,
+	GEANY_FILETYPE_GROUP_MARKUP,
+	GEANY_FILETYPE_GROUP_MISC
+}
+GeanyFiletypeGroupID;
+
+
 /* Safe wrapper to get the id field of a possibly NULL filetype pointer. */
 #define FILETYPE_ID(filetype_ptr) \
-	(((filetype_ptr) != NULL) ? (filetype_ptr)->id : GEANY_FILETYPES_ALL)
+	(((filetype_ptr) != NULL) ? (filetype_ptr)->id : GEANY_FILETYPES_NONE)
 
 
 struct build_actions
@@ -102,61 +116,66 @@ struct build_programs
 	gboolean modified;
 };
 
-struct filetype
+/** Represents a filetype. */
+struct GeanyFiletype
 {
-	filetype_id		  id;
-	guint	 		  uid;				/* unique id as reference for saved filetype in config file */
-	GtkWidget		 *item;				/* holds a pointer to the menu item for this filetypes */
-	langType 		  lang;				/* represents the langType of tagmanager(see the table */
-										/* in tagmanager/parsers.h), -1 represents all, -2 none */
-	gchar	 		 *name;				/* will be used as name for tagmanager */
-	gchar	 		 *title;			/* will be shown in the file open dialog */
-	gchar	 		 *extension;
-	gchar			**pattern;
+	filetype_id		  id;				/**< Index in @c filetypes_array. */
+	/** Represents the langType of tagmanager (see the table
+	 * in tagmanager/parsers.h), -1 represents all, -2 none. */
+	langType 		  lang;
+	gchar	 		 *name;				/**< Used as name for tagmanager. E.g. "C". */
+	gchar	 		 *title;			/**< Shown in the file open dialog. E.g. "C source file". */
+	gchar	 		 *extension;		/**< Default file extension for new files. */
+	gchar			**pattern;			/**< Array of filename-matching wildcard strings. */
 	gchar	 		 *context_action_cmd;
 	gchar	 		 *comment_open;
 	gchar	 		 *comment_close;
 	gboolean  		  comment_use_indent;
 	struct build_programs	*programs;
-	struct build_actions	*actions;
+	struct build_actions	*actions;	/* TODO: make private */
+	GeanyFiletypeGroupID	group;
+	gchar			 *error_regex_string;
+
+	struct GeanyFiletypePrivate	*priv;	/* must be last, append fields before this item */
 };
 
-extern filetype *filetypes[GEANY_MAX_FILE_TYPES];
+extern GPtrArray *filetypes_array;
+
+/* Wrap filetypes_array so it can be used with C array syntax.
+ * Example: filetypes[GEANY_FILETYPES_C]->name = ...; */
+#define filetypes	((GeanyFiletype **)filetypes_array->pdata)
 
 
-/* If uid is valid, return corresponding filetype, otherwise NULL. */
-filetype *filetypes_get_from_uid(gint uid);
+GeanyFiletype *filetypes_lookup_by_name(const gchar *name);
+
 
-/* Calls filetypes_init_types() and creates the filetype menu. */
 void filetypes_init(void);
 
-/* Create the filetype array and fill it with the known filetypes. */
 void filetypes_init_types(void);
 
-/* Detect the filetype for document idx, checking for a shebang, then filename extension. */
-filetype *filetypes_detect_from_file(gint idx);
+GeanyFiletype *filetypes_detect_from_document(GeanyDocument *doc);
 
-filetype *filetypes_detect_from_extension(const gchar *utf8_filename);
+GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename);
 
-/* Detect filetype based on the filename extension.
- * utf8_filename can include the full path. */
-filetype *filetypes_detect_from_filename(const gchar *utf8_filename);
+GeanyFiletype *filetypes_detect_from_file(const gchar *utf8_filename);
 
-/* frees the array and all related pointers */
 void filetypes_free_types(void);
 
-void filetypes_load_config(gint ft_id);
+void filetypes_load_config(gint ft_id, gboolean reload);
 
 void filetypes_save_commands(void);
 
-void filetypes_select_radio_item(const filetype *ft);
+void filetypes_select_radio_item(const GeanyFiletype *ft);
 
-GtkFileFilter *filetypes_create_file_filter(filetype *ft);
+GtkFileFilter *filetypes_create_file_filter(const GeanyFiletype *ft);
 
 GtkFileFilter *filetypes_create_file_filter_all_source(void);
 
 gchar *filetypes_get_conf_extension(gint filetype_idx);
 
-gboolean filetype_has_tags(filetype *ft);
+gboolean filetype_has_tags(GeanyFiletype *ft);
+
+gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
+		gchar **filename, gint *line);
 
 #endif
