@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: geanyobject.c 2287 2008-02-27 13:17:29Z eht16 $
+ * $Id: geanyobject.c 2953 2008-09-16 15:47:26Z ntrel $
  */
 
 /* GObject used for connecting and emitting signals when certain events happen,
@@ -86,36 +86,45 @@ static void create_signals(GObjectClass *g_object_class)
 		G_SIGNAL_RUN_FIRST,
 		G_STRUCT_OFFSET (GeanyObjectClass, document_new),
 		NULL, NULL,
-		gtk_marshal_NONE__INT,
+		gtk_marshal_NONE__POINTER,
 		G_TYPE_NONE, 1,
-		G_TYPE_INT);
+		G_TYPE_POINTER);
 	geany_object_signals[GCB_DOCUMENT_OPEN] = g_signal_new (
 		"document-open",
 		G_OBJECT_CLASS_TYPE (g_object_class),
 		G_SIGNAL_RUN_FIRST,
 		G_STRUCT_OFFSET (GeanyObjectClass, document_open),
 		NULL, NULL,
-		gtk_marshal_NONE__INT,
+		gtk_marshal_NONE__POINTER,
 		G_TYPE_NONE, 1,
-		G_TYPE_INT);
+		G_TYPE_POINTER);
 	geany_object_signals[GCB_DOCUMENT_SAVE] = g_signal_new (
 		"document-save",
 		G_OBJECT_CLASS_TYPE (g_object_class),
 		G_SIGNAL_RUN_FIRST,
 		G_STRUCT_OFFSET (GeanyObjectClass, document_save),
 		NULL, NULL,
-		gtk_marshal_NONE__INT,
+		gtk_marshal_NONE__POINTER,
 		G_TYPE_NONE, 1,
-		G_TYPE_INT);
+		G_TYPE_POINTER);
 	geany_object_signals[GCB_DOCUMENT_ACTIVATE] = g_signal_new (
 		"document-activate",
 		G_OBJECT_CLASS_TYPE (g_object_class),
 		G_SIGNAL_RUN_FIRST,
 		G_STRUCT_OFFSET (GeanyObjectClass, document_activate),
 		NULL, NULL,
-		gtk_marshal_NONE__INT,
+		gtk_marshal_NONE__POINTER,
 		G_TYPE_NONE, 1,
-		G_TYPE_INT);
+		G_TYPE_POINTER);
+	geany_object_signals[GCB_DOCUMENT_CLOSE] = g_signal_new (
+		"document-close",
+		G_OBJECT_CLASS_TYPE (g_object_class),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET (GeanyObjectClass, document_close),
+		NULL, NULL,
+		gtk_marshal_NONE__POINTER,
+		G_TYPE_NONE, 1,
+		G_TYPE_POINTER);
 
 	geany_object_signals[GCB_PROJECT_OPEN] = g_signal_new (
 		"project-open",
@@ -123,7 +132,7 @@ static void create_signals(GObjectClass *g_object_class)
 		G_SIGNAL_RUN_FIRST,
 		G_STRUCT_OFFSET (GeanyObjectClass, project_open),
 		NULL, NULL,
-		gtk_marshal_NONE__INT,
+		gtk_marshal_NONE__POINTER,
 		G_TYPE_NONE, 1,
 		G_TYPE_POINTER);
 	geany_object_signals[GCB_PROJECT_SAVE] = g_signal_new (
@@ -143,6 +152,16 @@ static void create_signals(GObjectClass *g_object_class)
 		NULL, NULL,
 		gtk_marshal_NONE__NONE,
 		G_TYPE_NONE, 0);
+
+	geany_object_signals[GCB_UPDATE_EDITOR_MENU] = g_signal_new (
+		"update-editor-menu",
+		G_OBJECT_CLASS_TYPE (g_object_class),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET (GeanyObjectClass, update_editor_menu),
+		NULL, NULL,
+		gtk_marshal_NONE__STRING_INT_POINTER,
+		G_TYPE_NONE, 3,
+		G_TYPE_STRING, G_TYPE_INT, G_TYPE_POINTER);
 }
 
 
