@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: geanyobject.h 2267 2008-02-20 11:24:23Z eht16 $
+ * $Id: geanyobject.h 2953 2008-09-16 15:47:26Z ntrel $
  */
 
 
@@ -38,9 +38,11 @@ typedef enum
 	GCB_DOCUMENT_OPEN,
 	GCB_DOCUMENT_SAVE,
 	GCB_DOCUMENT_ACTIVATE,
+	GCB_DOCUMENT_CLOSE,
 	GCB_PROJECT_OPEN,
 	GCB_PROJECT_SAVE,
 	GCB_PROJECT_CLOSE,
+	GCB_UPDATE_EDITOR_MENU,
 	GCB_MAX
 } GeanyCallbackId;
 
@@ -70,13 +72,15 @@ struct _GeanyObjectClass
 {
 	GObjectClass parent_class;
 
-	void (*document_new)(gint idx);
-	void (*document_open)(gint idx);
-	void (*document_save)(gint idx);
-	void (*document_activate)(gint idx);
+	void (*document_new)(GeanyDocument *doc);
+	void (*document_open)(GeanyDocument *doc);
+	void (*document_save)(GeanyDocument *doc);
+	void (*document_activate)(GeanyDocument *doc);
+	void (*document_close)(GeanyDocument *doc);
 	void (*project_open)(GKeyFile *keyfile);
 	void (*project_save)(GKeyFile *keyfile);
 	void (*project_close)(void);
+	void (*update_editor_menu)(const gchar *word, gint click_pos, GeanyDocument *doc);
 };
 
 GType		geany_object_get_type	(void);
