@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: geanywraplabel.c 2187 2008-01-23 14:32:52Z eht16 $
+ *  $Id: geanywraplabel.c 2789 2008-07-18 13:45:23Z eht16 $
  */
 
 /*
@@ -58,7 +58,6 @@ typedef struct
 
 static void geany_wrap_label_class_init		(GeanyWrapLabelClass *klass);
 static void geany_wrap_label_init			(GeanyWrapLabel *self);
-static void geany_wrap_label_finalize		(GObject *object);
 static void geany_wrap_label_size_request	(GtkWidget *widget, GtkRequisition *req);
 static void geany_wrap_label_size_allocate	(GtkWidget *widget, GtkAllocation *alloc);
 static void geany_wrap_label_set_wrap_width	(GtkWidget *widget, gsize width);
@@ -92,11 +91,8 @@ GType geany_wrap_label_get_type()
 
 static void geany_wrap_label_class_init(GeanyWrapLabelClass *klass)
 {
-	GObjectClass *g_object_class = G_OBJECT_CLASS(klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
 
-	g_object_class->finalize = geany_wrap_label_finalize;
-
 	parent_class = g_type_class_peek_parent(klass);
 	widget_class->size_request = geany_wrap_label_size_request;
 	widget_class->size_allocate = geany_wrap_label_size_allocate;
@@ -113,20 +109,6 @@ static void geany_wrap_label_init(GeanyWrapLabel *self)
 }
 
 
-static void geany_wrap_label_finalize(GObject *object)
-{
-	GeanyWrapLabel *self;
-
-	g_return_if_fail(object != NULL);
-	g_return_if_fail(IS_GEANY_WRAP_LABEL(object));
-
-	self = GEANY_WRAP_LABEL(object);
-
-	if (G_OBJECT_CLASS(parent_class)->finalize)
-		(* G_OBJECT_CLASS(parent_class)->finalize)(object);
-}
-
-
 /* Sets the point at which the text should wrap. */
 static void geany_wrap_label_set_wrap_width(GtkWidget *widget, gsize width)
 {
