@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keybindings.h 2395 2008-03-24 13:40:01Z ntrel $
+ * $Id: keybindings.h 3011 2008-09-26 18:29:42Z eht16 $
  */
 
 /**
@@ -40,39 +40,41 @@
 
 
 /** Function pointer type used for keybinding callbacks */
-typedef void (*KeyCallback) (guint key_id);
+typedef void (*GeanyKeyCallback) (guint key_id);
 
 /** Represents a single keybinding action */
 /* Note: name and label are not const strings so plugins can set them to malloc'd strings
  * and free them in cleanup(). */
-typedef struct KeyBinding
+typedef struct GeanyKeyBinding
 {
 	guint key;				/**< Key value in lower-case, such as @c GDK_a */
 	GdkModifierType mods;	/**< Modifier keys, such as @c GDK_CONTROL_MASK */
 	gchar *name;			/**< Key name for the configuration file, such as @c "menu_new" */
 	gchar *label;			/**< Label used in the preferences dialog keybindings tab */
-	KeyCallback callback;	/**< Callback function called when the key combination is pressed */
+	GeanyKeyCallback callback;	/**< Callback function called when the key combination is pressed */
 	GtkWidget *menu_item;	/**< Menu item widget for setting the menu accelerator */
-} KeyBinding;
+} GeanyKeyBinding;
 
 
 /** A collection of keybindings grouped together. */
-typedef struct KeyBindingGroup
+typedef struct GeanyKeyGroup
 {
 	const gchar *name;		/**< Group name used in the configuration file, such as @c "html_chars" */
 	const gchar *label;		/**< Group label used in the preferences dialog keybindings tab */
-	gsize count;			/**< Count of KeyBinding structs in @a keys */
-	KeyBinding *keys;		/**< Fixed array of KeyBinding structs */
+	gsize count;			/**< Count of GeanyKeyBinding structs in @a keys */
+	GeanyKeyBinding *keys;	/**< Fixed array of GeanyKeyBinding structs */
 }
-KeyBindingGroup;
+GeanyKeyGroup;
 
-extern GPtrArray *keybinding_groups;	/* array of KeyBindingGroup pointers */
+extern GPtrArray *keybinding_groups;	/* array of GeanyKeyGroup pointers */
 
 extern const gchar keybindings_keyfile_group_name[];
 
 
-/* Note: keybinding_groups is not in the API, so we don't need to increment the ABI when
- * appending keybindings or keygroups, as the _COUNT item shouldn't be used by plugins. */
+/* Note: we don't need to increment the plugin ABI when appending keybindings or keygroups,
+ * just make sure to only insert keybindings/groups immediately before the _COUNT item, so
+ * the existing enum values stay the same.
+ * The _COUNT item should not be used by plugins, as it may well change. */
 
 /** Keybinding group IDs */
 enum
@@ -207,6 +209,8 @@ enum
 	GEANY_KEYS_SEARCH_FINDPREVSEL,
 	GEANY_KEYS_SEARCH_NEXTMESSAGE,
 	GEANY_KEYS_SEARCH_FINDUSAGE,
+	GEANY_KEYS_SEARCH_PREVIOUSMESSAGE,
+	GEANY_KEYS_SEARCH_FINDDOCUMENTUSAGE,
 	GEANY_KEYS_SEARCH_COUNT
 };
 
@@ -222,6 +226,10 @@ enum
 	GEANY_KEYS_GOTO_PREVIOUSMARKER,
 	GEANY_KEYS_GOTO_TAGDEFINITION,
 	GEANY_KEYS_GOTO_TAGDECLARATION,
+	GEANY_KEYS_GOTO_LINESTART,
+	GEANY_KEYS_GOTO_LINEEND,
+	GEANY_KEYS_GOTO_PREVWORDSTART,
+	GEANY_KEYS_GOTO_NEXTWORDSTART,
 	GEANY_KEYS_GOTO_COUNT
 };
 
@@ -245,6 +253,7 @@ enum
 	GEANY_KEYS_FOCUS_VTE,
 	GEANY_KEYS_FOCUS_SEARCHBAR,
 	GEANY_KEYS_FOCUS_SIDEBAR,
+	GEANY_KEYS_FOCUS_COMPILER,
 	GEANY_KEYS_FOCUS_COUNT
 };
 
@@ -265,9 +274,13 @@ enum
 enum
 {
 	GEANY_KEYS_DOCUMENT_REPLACETABS,
+	GEANY_KEYS_DOCUMENT_REPLACESPACES,
+	GEANY_KEYS_DOCUMENT_TOGGLEFOLD,
 	GEANY_KEYS_DOCUMENT_FOLDALL,
 	GEANY_KEYS_DOCUMENT_UNFOLDALL,
 	GEANY_KEYS_DOCUMENT_RELOADTAGLIST,
+	GEANY_KEYS_DOCUMENT_LINEWRAP,
+	GEANY_KEYS_DOCUMENT_LINEBREAK,
 	GEANY_KEYS_DOCUMENT_COUNT
 };
 
@@ -283,6 +296,7 @@ enum
 	GEANY_KEYS_BUILD_RUN,
 	GEANY_KEYS_BUILD_RUN2,
 	GEANY_KEYS_BUILD_OPTIONS,
+	GEANY_KEYS_BUILD_PREVIOUSERROR,
 	GEANY_KEYS_BUILD_COUNT
 };
 
@@ -307,13 +321,13 @@ void keybindings_load_keyfile(void);
 
 void keybindings_free(void);
 
-void keybindings_set_item(KeyBindingGroup *group, gsize key_id,
-		KeyCallback callback, guint key, GdkModifierType mod,
+void keybindings_set_item(GeanyKeyGroup *group, gsize key_id,
+		GeanyKeyCallback callback, guint key, GdkModifierType mod,
 		gchar *name, gchar *label, GtkWidget *menu_item);
 
 void keybindings_send_command(guint group_id, guint key_id);
 
-KeyBinding *keybindings_lookup_item(guint group_id, guint key_id);
+GeanyKeyBinding *keybindings_lookup_item(guint group_id, guint key_id);
 
 /* just write the content of the keys array to the config file */
 void keybindings_write_to_file(void);
