@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: keyfile.h 2267 2008-02-20 11:24:23Z eht16 $
+ * $Id: keyfile.h 2738 2008-06-30 16:14:55Z eht16 $
  */
 
 
@@ -30,7 +30,7 @@ void configuration_save(void);
 
 gboolean configuration_load(void);
 
-gboolean configuration_open_files(void);
+void configuration_open_files(void);
 
 void configuration_reload_default_session(void);
 
@@ -42,8 +42,6 @@ void configuration_save_session_files(GKeyFile *config);
 
 void configuration_read_filetype_extensions(void);
 
-void configuration_read_snippets(void);
-
 /* set some settings which are already read from the config file, but need other things, like the
  * realisation of the main window */
 void configuration_apply_settings(void);
