@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.c 2434 2008-04-01 13:12:29Z ntrel $
+ * $Id: msgwindow.c 3056 2008-10-07 18:51:50Z eht16 $
  */
 
 /*
@@ -33,7 +33,6 @@
 #include "support.h"
 #include "prefs.h"
 #include "callbacks.h"
-#include "msgwindow.h"
 #include "ui_utils.h"
 #include "utils.h"
 #include "document.h"
@@ -42,6 +41,8 @@
 #include "main.h"
 #include "vte.h"
 #include "navqueue.h"
+#include "editor.h"
+#include "msgwindow.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -52,12 +53,12 @@
 typedef struct
 {
 	const gchar *string;	/* line data */
-	const gchar *dir;		/* working directory when string was generated */
 	const gchar *pattern;	/* pattern to split the error message into some fields */
 	guint min_fields;		/* used to detect errors after parsing */
 	guint line_idx;			/* idx of the field where the line is */
 	gint file_idx;			/* idx of the field where the filename is or -1 */
-} ParseData;
+}
+ParseData;
 
 MessageWindow msgwindow;
 
@@ -74,14 +75,12 @@ static void on_scribble_populate(GtkTextView *textview, GtkMenu *arg1, gpointer
 
 void msgwin_init()
 {
-	msgwindow.notebook = lookup_widget(app->window, "notebook_info");
-	msgwindow.tree_status = lookup_widget(app->window, "treeview3");
-	msgwindow.tree_msg = lookup_widget(app->window, "treeview4");
-	msgwindow.tree_compiler = lookup_widget(app->window, "treeview5");
+	msgwindow.notebook = lookup_widget(main_widgets.window, "notebook_info");
+	msgwindow.tree_status = lookup_widget(main_widgets.window, "treeview3");
+	msgwindow.tree_msg = lookup_widget(main_widgets.window, "treeview4");
+	msgwindow.tree_compiler = lookup_widget(main_widgets.window, "treeview5");
 	msgwindow.find_in_files_dir = NULL;
 
-	gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), FALSE);
-
 	prepare_status_tree_view();
 	prepare_msg_tree_view();
 	prepare_compiler_tree_view();
@@ -89,7 +88,7 @@ void msgwin_init()
 	msgwindow.popup_msg_menu = create_message_popup_menu(MSG_MESSAGE);
 	msgwindow.popup_compiler_menu = create_message_popup_menu(MSG_COMPILER);
 
-	g_signal_connect(G_OBJECT(lookup_widget(app->window, "textview_scribble")),
+	g_signal_connect(lookup_widget(main_widgets.window, "textview_scribble"),
 		"populate-popup", G_CALLBACK(on_scribble_populate), NULL);
 }
 
@@ -109,6 +108,7 @@ static void prepare_status_tree_view(void)
 
 	msgwindow.store_status = gtk_list_store_new(1, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_status), GTK_TREE_MODEL(msgwindow.store_status));
+	g_object_unref(msgwindow.store_status);
 
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(_("Status messages"), renderer, "text", 0, NULL);
@@ -116,11 +116,11 @@ static void prepare_status_tree_view(void)
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_status), FALSE);
 
-	pfd = pango_font_description_from_string(prefs.msgwin_font);
+	pfd = pango_font_description_from_string(interface_prefs.msgwin_font);
 	gtk_widget_modify_font(msgwindow.tree_status, pfd);
 	pango_font_description_free(pfd);
 
-	g_signal_connect(G_OBJECT(msgwindow.tree_status), "button-press-event",
+	g_signal_connect(msgwindow.tree_status, "button-press-event",
 				G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_STATUS));
 }
 
@@ -134,10 +134,11 @@ static void prepare_msg_tree_view(void)
 	GtkTreeSelection *selection;
 	PangoFontDescription *pfd;
 
-	/* doc idx, line, fg, str */
-	msgwindow.store_msg = gtk_list_store_new(4, G_TYPE_INT, G_TYPE_INT,
+	/* line, doc, fg, str */
+	msgwindow.store_msg = gtk_list_store_new(4, G_TYPE_INT, G_TYPE_POINTER,
 		GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_msg), GTK_TREE_MODEL(msgwindow.store_msg));
+	g_object_unref(msgwindow.store_msg);
 
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
@@ -146,19 +147,19 @@ static void prepare_msg_tree_view(void)
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_msg), FALSE);
 
-	pfd = pango_font_description_from_string(prefs.msgwin_font);
+	pfd = pango_font_description_from_string(interface_prefs.msgwin_font);
 	gtk_widget_modify_font(msgwindow.tree_msg, pfd);
 	pango_font_description_free(pfd);
 
 	/* use button-release-event so the selection has changed
 	 * (connect_after button-press-event doesn't work) */
-	g_signal_connect(G_OBJECT(msgwindow.tree_msg), "button-release-event",
+	g_signal_connect(msgwindow.tree_msg, "button-release-event",
 					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_MESSAGE));
 
 	/* selection handling */
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_msg));
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
-	/*g_signal_connect(G_OBJECT(selection), "changed",G_CALLBACK(on_msg_tree_selection_changed), NULL);*/
+	/*g_signal_connect(selection, "changed",G_CALLBACK(on_msg_tree_selection_changed), NULL);*/
 }
 
 
@@ -172,6 +173,7 @@ static void prepare_compiler_tree_view(void)
 
 	msgwindow.store_compiler = gtk_list_store_new(2, GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_compiler), GTK_TREE_MODEL(msgwindow.store_compiler));
+	g_object_unref(msgwindow.store_compiler);
 
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(NULL, renderer, "foreground-gdk", 0, "text", 1, NULL);
@@ -179,19 +181,19 @@ static void prepare_compiler_tree_view(void)
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_compiler), FALSE);
 
-	pfd = pango_font_description_from_string(prefs.msgwin_font);
+	pfd = pango_font_description_from_string(interface_prefs.msgwin_font);
 	gtk_widget_modify_font(msgwindow.tree_compiler, pfd);
 	pango_font_description_free(pfd);
 
 	/* use button-release-event so the selection has changed
 	 * (connect_after button-press-event doesn't work) */
-	g_signal_connect(G_OBJECT(msgwindow.tree_compiler), "button-release-event",
+	g_signal_connect(msgwindow.tree_compiler, "button-release-event",
 					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_COMPILER));
 
 	/* selection handling */
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_compiler));
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
-	/*g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(on_msg_tree_selection_changed), NULL);*/
+	/*g_signal_connect(selection, "changed", G_CALLBACK(on_msg_tree_selection_changed), NULL);*/
 }
 
 
@@ -213,6 +215,13 @@ static const GdkColor *get_color(gint msg_color)
 }
 
 
+/**
+ *  Adds a new message in the compiler tab treeview in the messages window.
+ *
+ *  @param msg_color A color to be used for the text. It must be an element of #MsgColors.
+ *  @param format Printf()-style format string.
+ *  @param ... Arguments for the @c format string.
+ **/
 void msgwin_compiler_add_fmt(gint msg_color, const gchar *format, ...)
 {
 	gchar string[512];
@@ -225,7 +234,6 @@ void msgwin_compiler_add_fmt(gint msg_color, const gchar *format, ...)
 }
 
 
-/* adds string to the compiler textview */
 void msgwin_compiler_add(gint msg_color, const gchar *msg)
 {
 	GtkTreeIter iter;
@@ -245,22 +253,34 @@ void msgwin_compiler_add(gint msg_color, const gchar *msg)
 
 	/* calling build_menu_update for every build message would be overkill */
 	gtk_widget_set_sensitive(build_get_menu_items(-1)->item_next_error, TRUE);
+	gtk_widget_set_sensitive(build_get_menu_items(-1)->item_previous_error, TRUE);
 }
 
 
 void msgwin_show_hide(gboolean show)
 {
 	ui_prefs.msgwindow_visible = show;
-	app->ignore_callback = TRUE;
+	ignore_callback = TRUE;
 	gtk_check_menu_item_set_active(
-		GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_messages_window1")),
+		GTK_CHECK_MENU_ITEM(lookup_widget(main_widgets.window, "menu_show_messages_window1")),
 		show);
-	app->ignore_callback = FALSE;
-	ui_widget_show_hide(lookup_widget(app->window, "scrolledwindow1"), show);
+	ignore_callback = FALSE;
+	ui_widget_show_hide(lookup_widget(main_widgets.window, "scrolledwindow1"), show);
 }
 
 
-void msgwin_msg_add_fmt(gint msg_color, gint line, gint idx, const gchar *format, ...)
+/**
+ *  Adds a new message in the messages tab treeview in the messages window.
+ *  If @c line and @c doc are set, clicking on this line jumps into the file which is specified
+ *  by @c doc into the line specified with @c line.
+ *
+ *  @param msg_color A color to be used for the text. It must be an element of #MsgColors.
+ *  @param line The document's line where the message belongs to. Set to -1 to ignore.
+ *  @param doc The document. Set to @c NULL to ignore.
+ *  @param format Printf()-style format string.
+ *  @param ... Arguments for the @c format string.
+ **/
+void msgwin_msg_add_fmt(gint msg_color, gint line, GeanyDocument *doc, const gchar *format, ...)
 {
 	gchar string[512];
 	va_list args;
@@ -269,12 +289,12 @@ void msgwin_msg_add_fmt(gint msg_color, gint line, gint idx, const gchar *format
 	g_vsnprintf(string, 512, format, args);
 	va_end(args);
 
-	msgwin_msg_add(msg_color, line, idx, string);
+	msgwin_msg_add(msg_color, line, doc, string);
 }
 
 
 /* adds string to the msg treeview */
-void msgwin_msg_add(gint msg_color, gint line, gint idx, const gchar *string)
+void msgwin_msg_add(gint msg_color, gint line, GeanyDocument *doc, const gchar *string)
 {
 	GtkTreeIter iter;
 	const GdkColor *color = get_color(msg_color);
@@ -284,23 +304,26 @@ void msgwin_msg_add(gint msg_color, gint line, gint idx, const gchar *string)
 
 	/* work around a strange problem when adding very long lines(greater than 4000 bytes)
 	 * cut the string to a maximum of 1024 bytes and discard the rest */
-	/** TODO find the real cause for the display problem / if it is GtkTreeView file a bug report */
+	/* TODO: find the real cause for the display problem / if it is GtkTreeView file a bug report */
 	if (strlen(string) > 1024)
 		tmp = g_strndup(string, 1024);
 	else
 		tmp = g_strdup(string);
 
 	gtk_list_store_append(msgwindow.store_msg, &iter);
-	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, idx, 2, color, 3, tmp, -1);
-
-	gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), TRUE);
+	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, doc, 2, color, 3, tmp, -1);
 
 	g_free(tmp);
 }
 
 
-/* Log a status message *without* setting the status bar.
- * (Use ui_set_statusbar() to display text on the statusbar) */
+/**
+ *  Log a status message *without* setting the status bar.
+ *  (Use ui_set_statusbar() to display text on the statusbar)
+ *
+ *  @param format Printf()-style format string.
+ *  @param ... Arguments for the @c format string.
+ **/
 void msgwin_status_add(const gchar *format, ...)
 {
 	GtkTreeIter iter;
@@ -330,7 +353,7 @@ void msgwin_status_add(const gchar *format, ...)
 		GtkTreePath *path = gtk_tree_model_get_path(gtk_tree_view_get_model(GTK_TREE_VIEW(msgwindow.tree_status)), &iter);
 
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(msgwindow.tree_status), path, NULL, FALSE, 0.0, 0.0);
-		if (prefs.switch_msgwin_pages) gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
+		if (prefs.switch_to_status) gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
 		gtk_tree_path_free(path);
 	}
 }
@@ -389,6 +412,60 @@ on_compiler_treeview_copy_activate     (GtkMenuItem     *menuitem,
 }
 
 
+static void on_compiler_treeview_copy_all_activate(GtkMenuItem *menuitem, gpointer user_data)
+{
+	GtkListStore *store = msgwindow.store_compiler;
+	GtkTreeIter iter;
+	GString *str = g_string_new("");
+	gint str_idx = 1;
+	gboolean valid;
+
+	switch (GPOINTER_TO_INT(user_data))
+	{
+		case MSG_STATUS:
+		store = msgwindow.store_status;
+		str_idx = 0;
+		break;
+
+		case MSG_COMPILER:
+		/* default values */
+		break;
+
+		case MSG_MESSAGE:
+		store = msgwindow.store_msg;
+		str_idx = 3;
+		break;
+	}
+
+	/* walk through the list and copy every line into a string */
+	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	while (valid)
+	{
+		gchar *line;
+
+		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, str_idx, &line, -1);
+		if (NZV(line))
+		{
+			g_string_append(str, line);
+			g_string_append_c(str, '\n');
+		}
+		g_free(line);
+
+		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+	}
+
+	/* copy the string into the clipboard */
+	if (str->len > 0)
+	{
+		gtk_clipboard_set_text(
+			gtk_clipboard_get(gdk_atom_intern("CLIPBOARD", FALSE)),
+			str->str,
+			str->len);
+	}
+	g_string_free(str, TRUE);
+}
+
+
 static void
 on_hide_message_window                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -399,22 +476,31 @@ on_hide_message_window                 (GtkMenuItem     *menuitem,
 
 static GtkWidget *create_message_popup_menu(gint type)
 {
-	GtkWidget *message_popup_menu, *clear, *copy;
+	GtkWidget *message_popup_menu, *clear, *copy, *image;
 
 	message_popup_menu = gtk_menu_new();
 
 	clear = gtk_image_menu_item_new_from_stock("gtk-clear", NULL);
 	gtk_widget_show(clear);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), clear);
-	g_signal_connect((gpointer)clear, "activate",
+	g_signal_connect(clear, "activate",
 		G_CALLBACK(on_message_treeview_clear_activate), GINT_TO_POINTER(type));
 
 	copy = gtk_image_menu_item_new_from_stock("gtk-copy", NULL);
 	gtk_widget_show(copy);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
-	g_signal_connect((gpointer)copy, "activate",
+	g_signal_connect(copy, "activate",
 		G_CALLBACK(on_compiler_treeview_copy_activate), GINT_TO_POINTER(type));
 
+	copy = gtk_image_menu_item_new_with_mnemonic(_("Copy _All"));
+	gtk_widget_show(copy);
+	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
+	image = gtk_image_new_from_stock("gtk-copy", GTK_ICON_SIZE_MENU);
+	gtk_widget_show(image);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy), image);
+	g_signal_connect(copy, "activate",
+		G_CALLBACK(on_compiler_treeview_copy_all_activate), GINT_TO_POINTER(type));
+
 	msgwin_menu_add_common_items(GTK_MENU(message_popup_menu));
 
 	return message_popup_menu;
@@ -439,8 +525,7 @@ void msgwin_menu_add_common_items(GtkMenu *menu)
 	item = gtk_menu_item_new_with_mnemonic(_("_Hide Message Window"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate",
-		G_CALLBACK(on_hide_message_window), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_hide_message_window), NULL);
 }
 
 
@@ -478,7 +563,6 @@ gboolean msgwin_goto_compiler_file_line()
 	gchar *string;
 	gboolean ret = FALSE;
 	GdkColor *color;
-	gint old_idx = document_get_cur_idx();
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_compiler));
 	if (gtk_tree_selection_get_selected(selection, &model, &iter))
@@ -496,7 +580,6 @@ gboolean msgwin_goto_compiler_file_line()
 		if (string != NULL)
 		{
 			gint line;
-			gint idx;
 			gchar *filename, *dir;
 			GtkTreePath *path;
 
@@ -511,18 +594,20 @@ gboolean msgwin_goto_compiler_file_line()
 			if (filename != NULL && line > -1)
 			{
 				gchar *utf8_filename = utils_get_utf8_from_locale(filename);
-				idx = document_find_by_filename(utf8_filename, FALSE);
+				GeanyDocument *doc = document_find_by_filename(utf8_filename);
+				GeanyDocument *old_doc = document_get_current();
+
 				g_free(utf8_filename);
 
-				if (idx < 0)	/* file not already open */
-					idx = document_open_file(filename, FALSE, NULL, NULL);
+				if (doc == NULL)	/* file not already open */
+					doc = document_open_file(filename, FALSE, NULL, NULL);
 
-				if (DOC_IDX_VALID(idx))
+				if (doc != NULL)
 				{
-					if (! doc_list[idx].changed)	/* if modified, line may be wrong */
-						document_set_indicator(idx, line - 1);
+					if (! doc->changed)	/* if modified, line may be wrong */
+						editor_set_indicator_on_line(doc->editor, line - 1);
 
-					ret = navqueue_goto_line(old_idx, idx, line);
+					ret = navqueue_goto_line(old_doc, doc, line);
 				}
 			}
 			g_free(filename);
@@ -533,6 +618,25 @@ gboolean msgwin_goto_compiler_file_line()
 }
 
 
+static void make_absolute(gchar **filename, const gchar *dir)
+{
+	guint skip_dot_slash = 0;	/* number of characters to skip at the beginning of the filename */
+
+	if (*filename == NULL)
+		return;
+
+	/* skip some characters at the beginning of the filename, at the moment only "./"
+	 * can be extended if other "trash" is known */
+	if (strncmp(*filename, "./", 2) == 0)
+		skip_dot_slash = 2;
+
+	/* add directory */
+	if (! utils_is_absolute_path(*filename))
+		setptr(*filename, g_strconcat(dir, G_DIR_SEPARATOR_S,
+			*filename + skip_dot_slash, NULL));
+}
+
+
 /* try to parse the file and line number where the error occured described in line
  * and when something useful is found, it stores the line number in *line and the
  * relevant file with the error in *filename.
@@ -542,12 +646,11 @@ static void parse_file_line(ParseData *data, gchar **filename, gint *line)
 {
 	gchar *end = NULL;
 	gchar **fields;
-	guint skip_dot_slash = 0;	/* number of characters to skip at the beginning of the filename */
 
 	*filename = NULL;
 	*line = -1;
 
-	g_return_if_fail(data->dir != NULL && data->string != NULL);
+	g_return_if_fail(data->string != NULL);
 
 	fields = g_strsplit_set(data->string, data->pattern, data->min_fields);
 
@@ -571,48 +674,24 @@ static void parse_file_line(ParseData *data, gchar **filename, gint *line)
 	if (data->file_idx == -1)
 	{
 		/* we have no filename in the error message, so take the current one and hope it's correct */
-		document *doc = document_get_current();
+		GeanyDocument *doc = document_get_current();
 		if (doc != NULL)
 			*filename = g_strdup(doc->file_name);
 		g_strfreev(fields);
 		return;
 	}
 
-	/* skip some characters at the beginning of the filename, at the moment only "./"
-	 * can be extended if other "trash" is known */
-	if (strncmp(fields[data->file_idx], "./", 2) == 0) skip_dot_slash = 2;
-
-	/* get the build directory to get the path to look for other files */
-	if (! utils_is_absolute_path(fields[data->file_idx]))
-		*filename = g_strconcat(data->dir, G_DIR_SEPARATOR_S,
-			fields[data->file_idx] + skip_dot_slash, NULL);
-	else
-		*filename = g_strdup(fields[data->file_idx]);
-
+	*filename = g_strdup(fields[data->file_idx]);
 	g_strfreev(fields);
 }
 
 
-/* try to parse the file and line number where the error occured described in string
- * and when something useful is found, it stores the line number in *line and the
- * relevant file with the error in *filename.
- * *line will be -1 if no error was found in string.
- * *filename must be freed unless it is NULL. */
-void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir, gchar **filename, gint *line)
+void parse_compiler_error_line(const gchar *string,
+		gchar **filename, gint *line)
 {
-	ParseData data = {NULL, NULL, NULL, 0, 0, 0};
+	ParseData data = {NULL, NULL, 0, 0, 0};
 
 	data.string = string;
-	data.dir = build_info.dir;
-
-	*filename = NULL;
-	*line = -1;
-
-	if (dir != NULL)
-		data.dir = dir;
-
-	g_return_if_fail(build_info.dir != NULL || dir != NULL);
-	if (string == NULL) return;
 
 	switch (build_info.file_type_id)
 	{
@@ -662,6 +741,7 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir, gch
 		}
 		case GEANY_FILETYPES_BASIC:
 		case GEANY_FILETYPES_PASCAL:
+		case GEANY_FILETYPES_CS:
 		{
 			/* getdrive.bas(52) error 18: Syntax error in '? GetAllDrives'
 			 * bandit.pas(149,3) Fatal: Syntax error, ";" expected but "ELSE" found */
@@ -733,11 +813,12 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir, gch
 			 * empty.h:4: Warnung: type defaults to `int' in declaration of `foo'
 			 * empty.c:21: error: conflicting types for `foo'
 			 * Only parse file and line, so that linker errors will also work (with -g) */
+		case GEANY_FILETYPES_F77:
 		case GEANY_FILETYPES_FORTRAN:
 		case GEANY_FILETYPES_LATEX:
 			/* ./kommtechnik_2b.tex:18: Emergency stop. */
 		case GEANY_FILETYPES_MAKE:	/* Assume makefile is building with gcc */
-		case GEANY_FILETYPES_ALL:
+		case GEANY_FILETYPES_NONE:
 		default:	/* The default is a GNU gcc type error */
 		{
 			if (build_info.file_type_id == GEANY_FILETYPES_JAVA &&
@@ -768,6 +849,38 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir, gch
 }
 
 
+/* try to parse the file and line number where the error occured described in string
+ * and when something useful is found, it stores the line number in *line and the
+ * relevant file with the error in *filename.
+ * *line will be -1 if no error was found in string.
+ * *filename must be freed unless it is NULL. */
+void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
+		gchar **filename, gint *line)
+{
+	GeanyFiletype *ft;
+
+	*filename = NULL;
+	*line = -1;
+
+	if (string == NULL)
+		return;
+
+	if (dir == NULL)
+		dir = build_info.dir;
+	g_return_if_fail(dir != NULL);
+
+	ft = filetypes[build_info.file_type_id];
+
+	/* try parsing with a custom regex */
+	if (!filetypes_parse_error_message(ft, string, filename, line))
+	{
+		/* fallback to default old-style parsing */
+		parse_compiler_error_line(string, filename, line);
+	}
+	make_absolute(filename, dir);
+}
+
+
 gboolean msgwin_goto_messages_file_line()
 {
 	GtkTreeIter iter;
@@ -778,14 +891,16 @@ gboolean msgwin_goto_messages_file_line()
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_msg));
 	if (gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
-		gint idx, line, old_idx = document_get_cur_idx();
+		gint line;
 		gchar *string;
+		GeanyDocument *doc;
+		GeanyDocument *old_doc = document_get_current();
 
-		gtk_tree_model_get(model, &iter, 0, &line, 1, &idx, 3, &string, -1);
-		if (line >= 0 && idx >= 0)
+		gtk_tree_model_get(model, &iter, 0, &line, 1, &doc, 3, &string, -1);
+		/* doc may have been closed, so check doc->index: */
+		if (line >= 0 && DOC_VALID(doc))
 		{
-			if (DOC_IDX_VALID(idx))
-				ret = navqueue_goto_line(old_idx, idx, line);
+			ret = navqueue_goto_line(old_doc, doc, line);
 		}
 		else if (line < 0 && string != NULL)
 		{
@@ -794,9 +909,9 @@ gboolean msgwin_goto_messages_file_line()
 			if (filename != NULL && line > -1)
 			{
 				/* use document_open_file to find an already open file, or open it in place */
-				idx = document_open_file(filename, FALSE, NULL, NULL);
-				if (DOC_IDX_VALID(idx))
-					ret = navqueue_goto_line(old_idx, idx, line);
+				doc = document_open_file(filename, FALSE, NULL, NULL);
+				if (doc != NULL)
+					ret = navqueue_goto_line(old_doc, doc, line);
 			}
 			g_free(filename);
 		}
@@ -818,17 +933,18 @@ static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *
 	*filename = NULL;
 	*line = -1;
 
-	if (string == NULL || msgwindow.find_in_files_dir == NULL) return;
+	if (string == NULL || msgwindow.find_in_files_dir == NULL)
+		return;
 
 	/* conflict:3:conflicting types for `foo' */
 	data.string = string;
-	data.dir = msgwindow.find_in_files_dir;
 	data.pattern = ":";
 	data.min_fields = 3;
 	data.line_idx = 1;
 	data.file_idx = 0;
 
 	parse_file_line(&data, filename, line);
+	make_absolute(filename, msgwindow.find_in_files_dir);
 }
 
 
@@ -882,13 +998,21 @@ static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *
 }
 
 
-void msgwin_switch_tab(MessageWindowTabNum tabnum, gboolean show)
+/**
+ *  Switches to the given notebook tab of the messages window and shows the messages window
+ *  if it was previously hidden and @c show is set to @a TRUE.
+ *
+ *  @param tabnum An index of a tab in the messages window. Valid values are all elements of
+ *                #MessageWindowTabNum.
+ *  @param show Whether to show the messages window at all if it was hidden before.
+ **/
+void msgwin_switch_tab(gint tabnum, gboolean show)
 {
 	GtkWidget *widget = NULL;	/* widget to focus */
 
 	switch (tabnum)
 	{
-		case MSG_SCRATCH: widget = lookup_widget(app->window, "textview_scribble"); break;
+		case MSG_SCRATCH: widget = lookup_widget(main_widgets.window, "textview_scribble"); break;
 #ifdef HAVE_VTE
 		case MSG_VTE: widget = (vte_info.have_vte) ? vc->vte : NULL; break;
 #endif
@@ -904,22 +1028,26 @@ void msgwin_switch_tab(MessageWindowTabNum tabnum, gboolean show)
 		gtk_widget_grab_focus(widget);
 }
 
-
-void msgwin_clear_tab(MessageWindowTabNum tabnum)
+/**
+ *  Removes all messages from a tab specified by @c tabnum in the messages window.
+ *
+ *  @param tabnum An index of a tab in the messages window which should be cleared.
+ *                Valid values are @a MSG_STATUS, @a MSG_COMPILER and @a MSG_MESSAGE.
+ **/
+void msgwin_clear_tab(gint tabnum)
 {
 	GtkListStore *store = NULL;
 
 	switch (tabnum)
 	{
 		case MSG_MESSAGE:
-			gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), FALSE);
 			store = msgwindow.store_msg;
 			break;
 
 		case MSG_COMPILER:
-			gtk_widget_set_sensitive(build_get_menu_items(-1)->item_next_error, FALSE);
-			store = msgwindow.store_compiler;
-			break;
+			gtk_list_store_clear(msgwindow.store_compiler);
+			build_menu_update(NULL);	/* update next error items */
+			return;
 
 		case MSG_STATUS: store = msgwindow.store_status; break;
 		default: return;
