@@ -18,29 +18,40 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.h 2150 2008-01-06 18:11:57Z eht16 $
+ * $Id: msgwindow.h 2705 2008-06-18 17:03:08Z ntrel $
  */
 
+/**
+ *  @file msgwindow.h
+ *  Message window functions (status, compiler, messages windows).
+ *  Also compiler error message parsing and grep file and line parsing.
+ **/
+
 
 #ifndef GEANY_MSGWINDOW_H
 #define GEANY_MSGWINDOW_H 1
 
 
-enum
+/**
+ * Various colors for use in the compiler and messages treeviews when adding messages.
+ **/
+enum MsgColors
 {
-	COLOR_RED,
-	COLOR_DARK_RED,
-	COLOR_BLACK,
-	COLOR_BLUE
+	COLOR_RED,		/**< Color red */
+	COLOR_DARK_RED,	/**< Color dark red */
+	COLOR_BLACK,	/**< Color black */
+	COLOR_BLUE		/**< Color blue */
 };
 
+/** Indices of the notebooks in the messages window. */
 typedef enum
 {
-	MSG_STATUS = 0, // force it to start at 0 to keep in sync with the notebook page numbers
-	MSG_COMPILER,
-	MSG_MESSAGE,
-	MSG_SCRATCH,
-	MSG_VTE,
+	/* force it to start at 0 to keep in sync with the notebook page numbers */
+	MSG_STATUS = 0,	/**< Index of the status message tab */
+	MSG_COMPILER,	/**< Index of the compiler tab */
+	MSG_MESSAGE,	/**< Index of the messages tab */
+	MSG_SCRATCH,	/**< Index of the scratch tab */
+	MSG_VTE			/**< Index of the VTE tab */
 } MessageWindowTabNum;
 
 
@@ -63,20 +74,20 @@ typedef struct
 extern MessageWindow msgwindow;
 
 
-void msgwin_init();
+void msgwin_init(void);
 
-void msgwin_finalize();
+void msgwin_finalize(void);
 
 void msgwin_show_hide(gboolean show);
 
-void msgwin_switch_tab(MessageWindowTabNum tabnum, gboolean show);
+void msgwin_switch_tab(gint tabnum, gboolean show);
 
-void msgwin_clear_tab(MessageWindowTabNum tabnum);
+void msgwin_clear_tab(gint tabnum);
 
+void msgwin_msg_add_fmt(gint msg_color, gint line, GeanyDocument *doc, const gchar *format, ...)
+			G_GNUC_PRINTF (4, 5);
 
-void msgwin_msg_add_fmt(gint msg_color, gint line, gint idx, const gchar *format, ...) G_GNUC_PRINTF (4, 5);
-
-void msgwin_msg_add(gint msg_color, gint line, gint idx, const gchar *string);
+void msgwin_msg_add(gint msg_color, gint line, GeanyDocument *doc, const gchar *string);
 
 void msgwin_compiler_add_fmt(gint msg_color, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
 
@@ -87,11 +98,11 @@ void msgwin_status_add(const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
 void msgwin_menu_add_common_items(GtkMenu *menu);
 
-gboolean msgwin_goto_compiler_file_line();
+gboolean msgwin_goto_compiler_file_line(void);
 
 void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 									  gchar **filename, gint *line);
 
-gboolean msgwin_goto_messages_file_line();
+gboolean msgwin_goto_messages_file_line(void);
 
 #endif
