@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: notebook.c 2375 2008-03-21 13:05:47Z ntrel $
+ * $Id: notebook.c 3017 2008-09-28 15:17:00Z eht16 $
  */
 
 /*
@@ -27,8 +27,9 @@
 
 #include "geany.h"
 #include "notebook.h"
-#include "prefs.h"
 #include "document.h"
+#include "editor.h"
+#include "documentprivate.h"
 #include "ui_utils.h"
 #include "treeviews.h"
 #include "support.h"
@@ -77,23 +78,143 @@ notebook_tab_close_clicked_cb(GtkButton *button, gpointer user_data);
 static void setup_tab_dnd(void);
 
 
-static void focus_sci(GtkWidget *widget, gpointer user_data)
+static gboolean focus_sci(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 
-	if (! DOC_IDX_VALID(idx)) return;
+	if (doc != NULL)
+		gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 
-	gtk_widget_grab_focus(GTK_WIDGET(doc_list[idx].sci));
+	return FALSE;
+}
+
+
+static gboolean gtk_notebook_show_arrows(GtkNotebook *notebook)
+{
+	return notebook->scrollable;
+#if 0
+	/* To get this working we would need to define at least the first two fields of
+	 * GtkNotebookPage since it is a private field. The better way would be to
+	 * subclass GtkNotebook.
+struct _FakeGtkNotebookPage
+{
+	GtkWidget *child;
+	GtkWidget *tab_label;
+};
+ */
+	gboolean show_arrow = FALSE;
+	GList *children;
+
+	if (! notebook->scrollable)
+		return FALSE;
+
+	children = notebook->children;
+	while (children)
+	{
+		struct _FakeGtkNotebookPage *page = children->data;
+
+		if (page->tab_label && ! gtk_widget_get_child_visible(page->tab_label))
+			show_arrow = TRUE;
+
+		children = children->next;
+	}
+	return show_arrow;
+#endif
+}
+
+
+static gboolean is_position_on_tab_bar(GtkNotebook *notebook, GdkEventButton *event)
+{
+	GtkWidget *page;
+	GtkWidget *tab;
+	GtkWidget *nb;
+	GtkPositionType tab_pos;
+	gint scroll_arrow_hlength, scroll_arrow_vlength;
+	gdouble x, y;
+
+	page = gtk_notebook_get_nth_page(notebook, 0);
+	g_return_val_if_fail(page != NULL, FALSE);
+
+	tab = gtk_notebook_get_tab_label(notebook, page);
+	g_return_val_if_fail(tab != NULL, FALSE);
+
+	tab_pos = gtk_notebook_get_tab_pos(notebook);
+	nb = GTK_WIDGET(notebook);
+
+#if GTK_CHECK_VERSION(2, 10, 0)
+	gtk_widget_style_get(GTK_WIDGET(notebook), "scroll-arrow-hlength", &scroll_arrow_hlength,
+		"scroll-arrow-vlength", &scroll_arrow_vlength, NULL);
+#else
+	scroll_arrow_hlength = scroll_arrow_vlength = 16;
+#endif
+
+	if (! gdk_event_get_coords((GdkEvent*) event, &x, &y))
+	{
+		x = event->x;
+		y = event->y;
+	}
+
+	switch (tab_pos)
+	{
+		case GTK_POS_TOP:
+		case GTK_POS_BOTTOM:
+		{
+			if (event->y >= 0 && event->y <= tab->allocation.height)
+			{
+				if (! gtk_notebook_show_arrows(notebook) || (
+					x > scroll_arrow_hlength &&
+					x < nb->allocation.width - scroll_arrow_hlength))
+					return TRUE;
+			}
+			break;
+		}
+		case GTK_POS_LEFT:
+		case GTK_POS_RIGHT:
+		{
+			if (event->x >= 0 && event->x <= tab->allocation.width)
+			{
+				if (! gtk_notebook_show_arrows(notebook) || (
+					y > scroll_arrow_vlength &&
+					y < nb->allocation.height - scroll_arrow_vlength))
+					return TRUE;
+			}
+		}
+	}
+
+	return FALSE;
+}
+
+
+static gboolean notebook_tab_bar_click_cb(GtkWidget *widget, GdkEventButton *event,
+										  gpointer user_data)
+{
+	if (event->type == GDK_2BUTTON_PRESS)
+	{
+		/* accessing ::event_window is a little hacky but we need to make sure the click
+		 * was in the tab bar and not inside the child */
+		if (event->window != GTK_NOTEBOOK(main_widgets.notebook)->event_window)
+			return FALSE;
+
+		if (is_position_on_tab_bar(GTK_NOTEBOOK(widget), event))
+		{
+			document_new_file(NULL, NULL, NULL);
+			return TRUE;
+		}
+	}
+	return FALSE;
 }
 
 
 void notebook_init()
 {
+	g_signal_connect_after(main_widgets.notebook, "button-press-event",
+		G_CALLBACK(notebook_tab_bar_click_cb), NULL);
+
 	/* focus the current document after clicking on a tab */
-	g_signal_connect_after(G_OBJECT(app->notebook), "button-release-event",
+	g_signal_connect_after(main_widgets.notebook, "button-release-event",
 		G_CALLBACK(focus_sci), NULL);
 
-	g_signal_connect(G_OBJECT(app->notebook), "drag-data-received",
+	g_signal_connect(main_widgets.notebook, "drag-data-received",
 		G_CALLBACK(on_window_drag_data_received), NULL);
 
 	setup_tab_dnd();
@@ -102,7 +223,7 @@ void notebook_init()
 
 static void setup_tab_dnd()
 {
-	GtkWidget *notebook = app->notebook;
+	GtkWidget *notebook = main_widgets.notebook;
 
 	/* Due to a segfault with manual tab DnD setup on GTK 2.10, we must
 	*  use the built in gtk_notebook_set_tab_reorderable from GTK 2.10.
@@ -112,15 +233,13 @@ static void setup_tab_dnd()
 	if (gtk_check_version(2, 10, 0) == NULL) /* null means version ok */
 	{
 #if GTK_CHECK_VERSION(2, 10, 0)
-		g_signal_connect(G_OBJECT(notebook), "page-reordered",
-			G_CALLBACK(notebook_page_reordered_cb), NULL);
+		g_signal_connect(notebook, "page-reordered", G_CALLBACK(notebook_page_reordered_cb), NULL);
 #endif
 		return;
 	}
 
 	/* Set up drag movement callback */
-	g_signal_connect(G_OBJECT(notebook), "drag-motion",
-		G_CALLBACK(notebook_drag_motion_cb), NULL);
+	g_signal_connect(notebook, "drag-motion", G_CALLBACK(notebook_drag_motion_cb), NULL);
 
 	/* There is a bug on GTK 2.6 with drag reordering of notebook tabs.
 	 * Clicking (not dragging) on a notebook tab, then making a selection in the
@@ -135,7 +254,7 @@ static void setup_tab_dnd()
 	{
 		/* workaround GTK+2.6 drag start bug when over sci widget: */
 		gtk_widget_add_events(notebook, GDK_POINTER_MOTION_MASK);
-		g_signal_connect(G_OBJECT(notebook), "motion-notify-event",
+		g_signal_connect(notebook, "motion-notify-event",
 			G_CALLBACK(notebook_motion_notify_event_cb), NULL);
 	}
 #endif
@@ -158,8 +277,8 @@ notebook_motion_notify_event_cb(GtkWidget *widget, GdkEventMotion *event,
 	gpointer user_data)
 {
 	static gboolean drag_enabled = TRUE; /* stores current state */
-	GtkWidget *page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(app->notebook),
-			gtk_notebook_get_current_page(GTK_NOTEBOOK(app->notebook)));
+	GtkWidget *page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(main_widgets.notebook),
+			gtk_notebook_get_current_page(GTK_NOTEBOOK(main_widgets.notebook)));
 
 	if (page == NULL || event->x < 0 || event->y < 0) return FALSE;
 
@@ -261,17 +380,13 @@ notebook_find_tab_num_at_pos(GtkNotebook *notebook, gint x, gint y)
 		}
 
 		/* subtract notebook pos to remove possible border padding */
-		max_x = tab->allocation.x + tab->allocation.width -
-			GTK_WIDGET(notebook)->allocation.x;
-		max_y = tab->allocation.y + tab->allocation.height -
-			GTK_WIDGET(notebook)->allocation.y;
+		max_x = tab->allocation.x + tab->allocation.width - GTK_WIDGET(notebook)->allocation.x;
+		max_y = tab->allocation.y + tab->allocation.height - GTK_WIDGET(notebook)->allocation.y;
 
-		if (((tab_pos == GTK_POS_TOP)
-			|| (tab_pos == GTK_POS_BOTTOM))
-			&&(x<=max_x)) return page_num;
-		else if (((tab_pos == GTK_POS_LEFT)
-			|| (tab_pos == GTK_POS_RIGHT))
-			&& (y<=max_y)) return page_num;
+		if (((tab_pos == GTK_POS_TOP) || (tab_pos == GTK_POS_BOTTOM)) && (x<=max_x))
+			return page_num;
+		else if (((tab_pos == GTK_POS_LEFT) || (tab_pos == GTK_POS_RIGHT)) && (y<=max_y))
+			return page_num;
 
 		page_num++;
 	}
@@ -279,14 +394,14 @@ notebook_find_tab_num_at_pos(GtkNotebook *notebook, gint x, gint y)
 }
 
 
-/* call this after the number of tabs in app->notebook changes. */
+/* call this after the number of tabs in main_widgets.notebook changes. */
 static void tab_count_changed(void)
 {
-	switch (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)))
+	switch (gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)))
 	{
 		case 0:
 		/* Enables DnD for dropping files into the empty notebook widget */
-		gtk_drag_dest_set(app->notebook, GTK_DEST_DEFAULT_ALL,
+		gtk_drag_dest_set(main_widgets.notebook, GTK_DEST_DEFAULT_ALL,
 			files_drop_targets,	G_N_ELEMENTS(files_drop_targets),
 			GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
 		break;
@@ -295,51 +410,57 @@ static void tab_count_changed(void)
 		/* Disables DnD for dropping files into the notebook widget and enables the DnD for moving file
 		 * tabs. Files can still be dropped into the notebook widget because it will be handled by the
 		 * active Scintilla Widget (only dropping to the tab bar is not possible but it should be ok) */
-		gtk_drag_dest_set(app->notebook, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
+		gtk_drag_dest_set(main_widgets.notebook, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
 			drag_targets, G_N_ELEMENTS(drag_targets), GDK_ACTION_MOVE);
 		break;
 	}
 }
 
-gboolean notebook_tab_label_cb(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+
+static gboolean notebook_tab_label_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	/* toggle additional widgets on double click */
 	if (event->type == GDK_2BUTTON_PRESS)
+	{
 		on_menu_toggle_all_additional_widgets1_activate(NULL, NULL);
+		return TRUE; /* stop other handlers like notebook_tab_bar_click_cb() */
+	}
 	/* close tab on middle click */
 	if (event->button == 2)
-		document_remove(gtk_notebook_page_num(GTK_NOTEBOOK(app->notebook), GTK_WIDGET(user_data)));
+	{
+		document_remove_page(gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook),
+			GTK_WIDGET(data)));
+		return TRUE; /* stop other handlers like notebook_tab_bar_click_cb() */
+	}
 
 	return FALSE;
 }
 
 
 /* Returns page number of notebook page, or -1 on error */
-gint notebook_new_tab(gint doc_idx)
+gint notebook_new_tab(GeanyDocument *this)
 {
 	GtkWidget *hbox, *ebox;
 	gint tabnum;
 	gchar *title;
-	document *this = &(doc_list[doc_idx]);
 	GtkWidget *page;
 
-	g_return_val_if_fail(doc_idx >= 0 && this != NULL, -1);
+	g_return_val_if_fail(this != NULL, -1);
 
-	page = GTK_WIDGET(this->sci);
-	title = g_path_get_basename(DOC_FILENAME(doc_idx));
+	page = GTK_WIDGET(this->editor->sci);
+	title = g_path_get_basename(DOC_FILENAME(this));
 
-	this->tab_label = gtk_label_new(title);
+	this->priv->tab_label = gtk_label_new(title);
 
 	ebox = gtk_event_box_new();
 	GTK_WIDGET_SET_FLAGS(ebox, GTK_NO_WINDOW);
-	g_signal_connect(G_OBJECT(ebox), "button-press-event",
-		G_CALLBACK(notebook_tab_label_cb), page);
+	g_signal_connect(ebox, "button-press-event", G_CALLBACK(notebook_tab_label_cb), page);
 
 	hbox = gtk_hbox_new(FALSE, 2);
-	gtk_container_add(GTK_CONTAINER(ebox), this->tab_label);
+	gtk_container_add(GTK_CONTAINER(ebox), this->priv->tab_label);
 	gtk_box_pack_start(GTK_BOX(hbox), ebox, FALSE, FALSE, 0);
 
-	if (prefs.show_tab_cross)
+	if (file_prefs.show_tab_cross)
 	{
 		GtkWidget *image, *btn, *align;
 		GtkRcStyle *rcstyle;
@@ -367,21 +488,20 @@ gint notebook_new_tab(gint doc_idx)
 
 		gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 0);
 
-		g_signal_connect(G_OBJECT(btn), "clicked",
-			G_CALLBACK(notebook_tab_close_clicked_cb), page);
+		g_signal_connect(btn, "clicked", G_CALLBACK(notebook_tab_close_clicked_cb), page);
 	}
 
 	gtk_widget_show_all(hbox);
 
-	this->tabmenu_label = gtk_label_new(title);
-	gtk_misc_set_alignment(GTK_MISC(this->tabmenu_label), 0.0, 0);
+	this->priv->tabmenu_label = gtk_label_new(title);
+	gtk_misc_set_alignment(GTK_MISC(this->priv->tabmenu_label), 0.0, 0);
 
-	if (prefs.tab_order_ltr)
-		tabnum = gtk_notebook_append_page_menu(GTK_NOTEBOOK(app->notebook), page,
-			hbox, this->tabmenu_label);
+	if (file_prefs.tab_order_ltr)
+		tabnum = gtk_notebook_append_page_menu(GTK_NOTEBOOK(main_widgets.notebook), page,
+			hbox, this->priv->tabmenu_label);
 	else
-		tabnum = gtk_notebook_insert_page_menu(GTK_NOTEBOOK(app->notebook), page,
-			hbox, this->tabmenu_label, 0);
+		tabnum = gtk_notebook_insert_page_menu(GTK_NOTEBOOK(main_widgets.notebook), page,
+			hbox, this->priv->tabmenu_label, 0);
 
 	tab_count_changed();
 
@@ -389,7 +509,7 @@ gint notebook_new_tab(gint doc_idx)
 #if GTK_CHECK_VERSION(2, 10, 0)
 	if (gtk_check_version(2, 10, 0) == NULL) /* null means version ok */
 	{
-		gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(app->notebook), page, TRUE);
+		gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(main_widgets.notebook), page, TRUE);
 	}
 #endif
 	g_free(title);
@@ -400,25 +520,26 @@ gint notebook_new_tab(gint doc_idx)
 static void
 notebook_tab_close_clicked_cb(GtkButton *button, gpointer user_data)
 {
-	gint cur_page = gtk_notebook_page_num(GTK_NOTEBOOK(app->notebook),
+	gint cur_page = gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook),
 		GTK_WIDGET(user_data));
-	document_remove(cur_page);
+
+	document_remove_page(cur_page);
 }
 
 
 /* Always use this instead of gtk_notebook_remove_page(). */
 void notebook_remove_page(gint page_num)
 {
-	gint curpage = gtk_notebook_get_current_page(GTK_NOTEBOOK(app->notebook));
+	gint curpage = gtk_notebook_get_current_page(GTK_NOTEBOOK(main_widgets.notebook));
 
 	/* Focus the next page, not the previous */
-	if (curpage == page_num && prefs.tab_order_ltr)
+	if (curpage == page_num && file_prefs.tab_order_ltr)
 	{
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), curpage + 1);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook), curpage + 1);
 	}
 
 	/* now remove the page (so we don't temporarily switch to the previous page) */
-	gtk_notebook_remove_page(GTK_NOTEBOOK(app->notebook), page_num);
+	gtk_notebook_remove_page(GTK_NOTEBOOK(main_widgets.notebook), page_num);
 
 	tab_count_changed();
 }
