@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: notebook.h 2287 2008-02-27 13:17:29Z eht16 $
+ * $Id: notebook.h 2705 2008-06-18 17:03:08Z ntrel $
  */
 
 #ifndef GEANY_NOTEBOOK_H
@@ -27,7 +27,7 @@
 void notebook_init(void);
 
 /* Returns page number of notebook page, or -1 on error */
-gint notebook_new_tab(gint doc_idx);
+gint notebook_new_tab(GeanyDocument *doc);
 
 /* Always use this instead of gtk_notebook_remove_page(). */
 void notebook_remove_page(gint page_num);
