@@ -19,10 +19,12 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugins.c 2496 2008-04-17 17:48:12Z eht16 $
+ * $Id: plugins.c 3095 2008-10-15 12:19:24Z ntrel $
  */
 
 /* Code to manage, load and unload plugins. */
+/** @file plugins.c
+ * Plugin utility functions. */
 
 #include "geany.h"
 
@@ -55,6 +57,7 @@
 #include "highlighting.h"
 #include "keybindings.h"
 #include "navqueue.h"
+#include "main.h"
 
 
 #ifdef G_OS_WIN32
@@ -64,19 +67,29 @@
 #endif
 
 
+typedef struct GeanyPluginPrivate
+{
+	GeanyAutoSeparator	toolbar_separator;
+}
+GeanyPluginPrivate;
+
+
 typedef struct Plugin
 {
 	GModule 	*module;
 	gchar		*filename;				/* plugin filename (/path/libname.so) */
+	PluginInfo		info;				/* plugin name, description, etc */
 	PluginFields	fields;
+	GeanyPlugin		public;				/* fields the plugin can read */
+	GeanyPluginPrivate	priv;			/* GeanyPlugin type private data */
+
 	gulong		*signal_ids;			/* signal IDs to disconnect when unloading */
 	gsize		signal_ids_len;
-	KeyBindingGroup	*key_group;
+	GeanyKeyGroup	*key_group;
 
-	PluginInfo*	(*info) (void);			/* Returns plugin name, description */
-	void	(*init) (GeanyData *data);	/* Called when the plugin is enabled */
-	void	(*configure) (GtkWidget *parent);	/* plugin configure dialog, optionally */
-	void	(*cleanup) (void);			/* Called when the plugin is disabled or when Geany exits */
+	void		(*init) (GeanyData *data);			/* Called when the plugin is enabled */
+	GtkWidget*	(*configure) (GtkDialog *dialog);	/* plugin configure dialog, optional */
+	void		(*cleanup) (void);					/* Called when the plugin is disabled or when Geany exits */
 }
 Plugin;
 
@@ -88,32 +101,40 @@ static GList *active_plugin_list = NULL; /* list of only actually loaded plugins
 static gchar **active_plugins_pref = NULL; 	/* list of plugin filenames to load at startup */
 static GList *failed_plugins_list = NULL;	/* plugins the user wants active but can't be used */
 
-static GtkWidget *separator = NULL;
+static GtkWidget *menu_separator = NULL;
 
 static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data);
 
+void plugin_add_toolbar_item(GeanyPlugin *plugin, GtkToolItem *item);
 
-enum
-{
-	PLUGIN_FREE_NON_ACTIVE,
-	PLUGIN_FREE_ALL
+
+static PluginFuncs plugin_funcs = {
+	&plugin_add_toolbar_item
 };
 
-
 static DocumentFuncs doc_funcs = {
 	&document_new_file,
-	&document_get_cur_idx,
-	&document_get_n_idx,
-	&document_find_by_filename,
 	&document_get_current,
+	&document_get_from_page,
+	&document_find_by_filename,
+	&document_find_by_real_path,
 	&document_save_file,
 	&document_open_file,
 	&document_open_files,
-	&document_remove,
+	&document_remove_page,
 	&document_reload_file,
 	&document_set_encoding,
 	&document_set_text_changed,
-	&document_set_filetype
+	&document_set_filetype,
+	&document_close
+};
+
+static EditorFuncs editor_funcs = {
+	&editor_set_indicator,
+	&editor_set_indicator_on_line,
+	&editor_clear_indicators,
+	&editor_get_indent_prefs,
+	&editor_create_widget
 };
 
 static ScintillaFuncs sci_funcs = {
@@ -146,10 +167,13 @@ static ScintillaFuncs sci_funcs = {
 	&sci_get_line_is_visible,
 	&sci_ensure_line_is_visible,
 	&sci_scroll_caret,
-	&sci_find_bracematch,
+	&sci_find_matching_brace,
 	&sci_get_style_at,
 	&sci_get_char_at,
 	&sci_get_current_line,
+	&sci_has_selection,
+	&sci_get_tab_width,
+	&sci_indicator_clear
 };
 
 static TemplateFuncs template_funcs = {
@@ -179,6 +203,7 @@ static UIUtilsFuncs uiutils_funcs = {
 	&ui_table_add_row,
 	&ui_path_box_new,
 	&ui_button_new_with_image,
+	&ui_add_document_sensitive
 };
 
 static DialogFuncs dialog_funcs = {
@@ -193,7 +218,10 @@ static SupportFuncs support_funcs = {
 
 static MsgWinFuncs msgwin_funcs = {
 	&msgwin_status_add,
-	&msgwin_compiler_add_fmt
+	&msgwin_compiler_add_fmt,
+	&msgwin_msg_add_fmt,
+	&msgwin_clear_tab,
+	&msgwin_switch_tab
 };
 
 static EncodingFuncs encoding_funcs = {
@@ -226,23 +254,19 @@ static HighlightingFuncs highlighting_funcs = {
 
 
 static FiletypeFuncs filetype_funcs = {
-	&filetypes_detect_from_filename,
-	&filetypes_get_from_uid
+	&filetypes_detect_from_file,
+	&filetypes_lookup_by_name
 };
 
 static NavQueueFuncs navqueue_funcs = {
 	&navqueue_goto_line
 };
 
-static GeanyData geany_data = {
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
+static MainFuncs main_funcs = {
+	&main_reload_configuration
+};
 
+static GeanyFunctions geany_functions = {
 	&doc_funcs,
 	&sci_funcs,
 	&template_funcs,
@@ -257,20 +281,34 @@ static GeanyData geany_data = {
 	&search_funcs,
 	&highlighting_funcs,
 	&filetype_funcs,
-	&navqueue_funcs
+	&navqueue_funcs,
+	&editor_funcs,
+	&main_funcs,
+	&plugin_funcs
 };
 
+static GeanyData geany_data;
+
 
 static void
 geany_data_init(void)
 {
-	geany_data.app = app;
-	geany_data.tools_menu = lookup_widget(app->window, "tools1_menu");
-	geany_data.doc_array = doc_array;
-	geany_data.filetypes = filetypes;
-	geany_data.prefs = &prefs;
-	geany_data.editor_prefs = &editor_prefs;
-	geany_data.build_info = &build_info;
+	GeanyData gd = {
+		app,
+		&main_widgets,
+		documents_array,
+		filetypes_array,
+		&prefs,
+		&interface_prefs,
+		&toolbar_prefs,
+		&editor_prefs,
+		&file_prefs,
+		&search_prefs,
+		&tool_prefs,
+		&template_prefs,
+		&build_info
+	};
+	memcpy(&geany_data, &gd, sizeof(GeanyData));
 }
 
 
@@ -338,13 +376,19 @@ static gboolean
 plugin_check_version(GModule *module)
 {
 	gint (*version_check)(gint) = NULL;
-	gint result;
 
-	g_module_symbol(module, "version_check", (void *) &version_check);
+	g_module_symbol(module, "plugin_version_check", (void *) &version_check);
 
-	if (version_check)
+	if (! version_check)
 	{
-		result = version_check(abi_version);
+		geany_debug("Plugin \"%s\" has no plugin_version_check() function - ignoring plugin!",
+				g_module_name(module));
+		return FALSE;
+	}
+	else
+	{
+		gint result = version_check(GEANY_ABI_VERSION);
+
 		if (result < 0)
 		{
 			ui_set_statusbar(TRUE, _("The plugin \"%s\" is not binary compatible with this "
@@ -364,9 +408,9 @@ plugin_check_version(GModule *module)
 }
 
 
-static void add_callbacks(Plugin *plugin, GeanyCallback *callbacks)
+static void add_callbacks(Plugin *plugin, PluginCallback *callbacks)
 {
-	GeanyCallback *cb;
+	PluginCallback *cb;
 	guint i, len = 0;
 
 	while (TRUE)
@@ -396,16 +440,31 @@ static void add_callbacks(Plugin *plugin, GeanyCallback *callbacks)
 static void
 add_kb_group(Plugin *plugin)
 {
+	guint i;
+
 	g_return_if_fail(NZV(plugin->key_group->name));
 	g_return_if_fail(! g_str_equal(plugin->key_group->name, keybindings_keyfile_group_name));
 
+	for (i = 0; i < plugin->key_group->count; i++)
+	{
+		GeanyKeyBinding *kb = &plugin->key_group->keys[i];
+
+		if (!NZV(kb->name))
+		{
+			geany_debug("Plugin \"%s\" has not set a name for keybinding %d"
+				" - ignoring all keybindings!",
+				plugin->info.name, i);
+			plugin->key_group->count = 0;
+			break;
+		}
+	}
 	if (plugin->key_group->count == 0)
 	{
-		plugin->key_group = NULL;	/* Ignore the group */
+		plugin->key_group = NULL;	/* Ignore the group (maybe the plugin has optional KB) */
 		return;
 	}
 
-	plugin->key_group->label = plugin->info()->name;
+	plugin->key_group->label = plugin->info.name;
 
 	g_ptr_array_add(keybinding_groups, plugin->key_group);
 }
@@ -414,18 +473,52 @@ add_kb_group(Plugin *plugin)
 static void
 plugin_init(Plugin *plugin)
 {
-	GeanyCallback *callbacks;
+	GeanyPlugin **p_geany_plugin;
+	PluginCallback *callbacks;
+	PluginInfo **p_info;
+	PluginFields **plugin_fields;
+	GeanyData **p_geany_data;
+	GeanyFunctions **p_geany_functions;
 
-	if (plugin->init)
-		plugin->init(&geany_data);
+	/* set these symbols before plugin_init() is called */
+	g_module_symbol(plugin->module, "geany_plugin", (void *) &p_geany_plugin);
+	if (p_geany_plugin)
+		*p_geany_plugin = &plugin->public;
+	g_module_symbol(plugin->module, "plugin_info", (void *) &p_info);
+	if (p_info)
+		*p_info = &plugin->info;
+	g_module_symbol(plugin->module, "geany_data", (void *) &p_geany_data);
+	if (p_geany_data)
+		*p_geany_data = &geany_data;
+	g_module_symbol(plugin->module, "geany_functions", (void *) &p_geany_functions);
+	if (p_geany_functions)
+		*p_geany_functions = &geany_functions;
+	g_module_symbol(plugin->module, "plugin_fields", (void *) &plugin_fields);
+	if (plugin_fields)
+		*plugin_fields = &plugin->fields;
+
+	/* start the plugin */
+	g_return_if_fail(plugin->init);
+	plugin->init(&geany_data);
+
+	/* store some function pointers for later use */
+	g_module_symbol(plugin->module, "plugin_configure", (void *) &plugin->configure);
+	g_module_symbol(plugin->module, "plugin_cleanup", (void *) &plugin->cleanup);
+	if (plugin->cleanup == NULL)
+	{
+		if (app->debug_mode)
+			g_warning("Plugin '%s' has no plugin_cleanup() function - there may be memory leaks!",
+				plugin->info.name);
+	}
+
+	/* now read any plugin-owned data that might have been set in plugin_init() */
 
 	if (plugin->fields.flags & PLUGIN_IS_DOCUMENT_SENSITIVE)
 	{
-		gboolean enable = gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) ? TRUE : FALSE;
-		gtk_widget_set_sensitive(plugin->fields.menu_item, enable);
+		ui_add_document_sensitive(plugin->fields.menu_item);
 	}
 
-	g_module_symbol(plugin->module, "geany_callbacks", (void *) &callbacks);
+	g_module_symbol(plugin->module, "plugin_callbacks", (void *) &callbacks);
 	if (callbacks)
 		add_callbacks(plugin, callbacks);
 
@@ -434,15 +527,16 @@ plugin_init(Plugin *plugin)
 	if (plugin->key_group)
 		add_kb_group(plugin);
 
+	/* remember which plugins are active */
 	active_plugin_list = g_list_append(active_plugin_list, plugin);
 
 	geany_debug("Loaded:   %s (%s)", plugin->filename,
-		NVL(plugin->info()->name, "<Unknown>"));
+		NVL(plugin->info.name, "<Unknown>"));
 }
 
 
-/* Load and init a plugin.
- * init_plugin decides whether the plugin's init() function should be called or not. If it is
+/* Load and optionally init a plugin.
+ * init_plugin decides whether the plugin's plugin_init() function should be called or not. If it is
  * called, the plugin will be started, if not the plugin will be read only (for the list of
  * available plugins in the plugin manager).
  * When add_to_list is set, the plugin will be added to the plugin manager's plugin_list. */
@@ -451,9 +545,7 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 {
 	Plugin *plugin;
 	GModule *module;
-	PluginInfo* (*info)(void);
-	PluginFields **plugin_fields;
-	GeanyData **p_geany_data;
+	void (*plugin_set_info)(PluginInfo*);
 
 	g_return_val_if_fail(fname, NULL);
 	g_return_val_if_fail(g_module_supported(), NULL);
@@ -473,11 +565,12 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 	 * causing a segfault. Without that flag the module will safely fail to load.
 	 * G_MODULE_BIND_LOCAL also helps find undefined symbols e.g. app when it would
 	 * otherwise not be detected due to the shadowing of Geany's app variable.
-	 * Also without G_MODULE_BIND_LOCAL calling info() in a plugin will be shadowed. */
+	 * Also without G_MODULE_BIND_LOCAL calling public functions e.g. the old info()
+	 * function from a plugin will be shadowed. */
 	module = g_module_open(fname, G_MODULE_BIND_LOCAL);
 	if (! module)
 	{
-		g_warning("%s", g_module_error());
+		geany_debug("Can't load plugin: %s", g_module_error());
 		return NULL;
 	}
 
@@ -497,38 +590,48 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 		return NULL;
 	}
 
-	g_module_symbol(module, "info", (void *) &info);
-	if (info == NULL)
+	g_module_symbol(module, "plugin_set_info", (void *) &plugin_set_info);
+	if (plugin_set_info == NULL)
 	{
-		geany_debug("Unknown plugin info for \"%s\"!", fname);
+		geany_debug("No plugin_set_info() defined for \"%s\" - ignoring plugin!", fname);
 
 		if (! g_module_close(module))
 			g_warning("%s: %s", fname, g_module_error());
 		return NULL;
 	}
-	geany_debug("Initializing plugin '%s'", info()->name);
 
 	plugin = g_new0(Plugin, 1);
-	plugin->info = info;
+
+	/* read plugin name, etc. */
+	plugin_set_info(&plugin->info);
+	if (!NZV(plugin->info.name))
+	{
+		geany_debug("No plugin name set in plugin_set_info() for \"%s\" - ignoring plugin!",
+			fname);
+
+		if (! g_module_close(module))
+			g_warning("%s: %s", fname, g_module_error());
+		g_free(plugin);
+		return NULL;
+	}
+
+	g_module_symbol(module, "plugin_init", (void *) &plugin->init);
+	if (plugin->init == NULL)
+	{
+		geany_debug("Plugin '%s' has no plugin_init() function - ignoring plugin!",
+			plugin->info.name);
+
+		if (! g_module_close(module))
+			g_warning("%s: %s", fname, g_module_error());
+		g_free(plugin);
+		return NULL;
+	}
+	geany_debug("Initializing plugin '%s'", plugin->info.name);
+
 	plugin->filename = g_strdup(fname);
 	plugin->module = module;
-
-	g_module_symbol(module, "geany_data", (void *) &p_geany_data);
-	if (p_geany_data)
-		*p_geany_data = &geany_data;
-	g_module_symbol(module, "plugin_fields", (void *) &plugin_fields);
-	if (plugin_fields)
-		*plugin_fields = &plugin->fields;
-
-	g_module_symbol(module, "init", (void *) &plugin->init);
-	g_module_symbol(module, "configure", (void *) &plugin->configure);
-	g_module_symbol(module, "cleanup", (void *) &plugin->cleanup);
-	if (plugin->init != NULL && plugin->cleanup == NULL)
-	{
-		if (app->debug_mode)
-			g_warning("Plugin '%s' has no cleanup() function - there may be memory leaks!",
-				info()->name);
-	}
+	plugin->public.info = &plugin->info;
+	plugin->public.priv = &plugin->priv;
 
 	if (init_plugin)
 		plugin_init(plugin);
@@ -559,11 +662,13 @@ static gboolean is_active_plugin(Plugin *plugin)
 }
 
 
+/* Clean up anything used by an active plugin  */
 static void
-plugin_unload(Plugin *plugin)
+plugin_cleanup(Plugin *plugin)
 {
+	GtkWidget *widget;
 
-	if (is_active_plugin(plugin) && plugin->cleanup)
+	if (plugin->cleanup)
 		plugin->cleanup();
 
 	remove_callbacks(plugin);
@@ -571,22 +676,24 @@ plugin_unload(Plugin *plugin)
 	if (plugin->key_group)
 		g_ptr_array_remove_fast(keybinding_groups, plugin->key_group);
 
-	active_plugin_list = g_list_remove(active_plugin_list, plugin);
+	widget = plugin->priv.toolbar_separator.widget;
+	if (widget)
+		gtk_widget_destroy(widget);
+
 	geany_debug("Unloaded: %s", plugin->filename);
 }
 
 
 static void
-plugin_free(Plugin *plugin, gpointer data)
+plugin_free(Plugin *plugin)
 {
 	g_return_if_fail(plugin);
 	g_return_if_fail(plugin->module);
 
-	/* don't do anything when closing the plugin manager and it is an active plugin */
-	if (GPOINTER_TO_INT(data) == PLUGIN_FREE_NON_ACTIVE && is_active_plugin(plugin))
-		return;
+	if (is_active_plugin(plugin))
+		plugin_cleanup(plugin);
 
-	plugin_unload(plugin);
+	active_plugin_list = g_list_remove(active_plugin_list, plugin);
 
 	if (plugin->module != NULL && ! g_module_close(plugin->module))
 		g_warning("%s: %s", plugin->filename, g_module_error());
@@ -605,15 +712,14 @@ load_active_plugins()
 {
 	guint i, len;
 
-	len = g_strv_length(active_plugins_pref);
-	if (active_plugins_pref == NULL || len == 0)
+	if (active_plugins_pref == NULL || (len = g_strv_length(active_plugins_pref)) == 0)
 		return;
 
 	for (i = 0; i < len; i++)
 	{
 		const gchar *fname = active_plugins_pref[i];
 
-		if (NZV(fname))
+		if (NZV(fname) && g_file_test(fname, G_FILE_TEST_EXISTS))
 		{
 			if (plugin_new(fname, TRUE, FALSE) == NULL)
 				failed_plugins_list = g_list_append(failed_plugins_list, g_strdup(fname));
@@ -652,7 +758,7 @@ static gchar *get_plugin_path()
 	gchar *install_dir = g_win32_get_package_installation_directory("geany", NULL);
 	gchar *path;
 
-	path = g_strconcat(install_dir, "\\plugins", NULL);
+	path = g_strconcat(install_dir, "\\lib", NULL);
 
 	return path;
 }
@@ -679,6 +785,30 @@ static void load_all_plugins(void)
 }
 
 
+static void on_tools_menu_show(GtkWidget *menu_item, G_GNUC_UNUSED gpointer user_data)
+{
+	GList *item, *list = gtk_container_get_children(GTK_CONTAINER(menu_item));
+	guint i = 0;
+	gboolean have_plugin_menu_items = FALSE;
+
+	for (item = list; item != NULL; item = g_list_next(item))
+	{
+		if (item->data == menu_separator)
+		{
+			if (i < g_list_length(list) - 1)
+			{
+				have_plugin_menu_items = TRUE;
+				break;
+			}
+		}
+		i++;
+	}
+	g_list_free(list);
+
+	ui_widget_show_hide(menu_separator, have_plugin_menu_items);
+}
+
+
 void plugins_init()
 {
 	GtkWidget *widget;
@@ -688,19 +818,18 @@ void plugins_init()
 
 	widget = gtk_separator_menu_item_new();
 	gtk_widget_show(widget);
-	gtk_container_add(GTK_CONTAINER(geany_data.tools_menu), widget);
+	gtk_container_add(GTK_CONTAINER(main_widgets.tools_menu), widget);
 
 	widget = gtk_menu_item_new_with_mnemonic(_("_Plugin Manager"));
-	gtk_container_add(GTK_CONTAINER (geany_data.tools_menu), widget);
+	gtk_container_add(GTK_CONTAINER(main_widgets.tools_menu), widget);
 	gtk_widget_show(widget);
-	g_signal_connect((gpointer) widget, "activate", G_CALLBACK(pm_show_dialog), NULL);
+	g_signal_connect(widget, "activate", G_CALLBACK(pm_show_dialog), NULL);
 
-	separator = gtk_separator_menu_item_new();
-	gtk_container_add(GTK_CONTAINER(geany_data.tools_menu), separator);
+	menu_separator = gtk_separator_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(main_widgets.tools_menu), menu_separator);
+	g_signal_connect(main_widgets.tools_menu, "show", G_CALLBACK(on_tools_menu_show), NULL);
 
 	load_active_plugins();
-
-	plugins_update_tools_menu();
 }
 
 
@@ -771,7 +900,7 @@ void plugins_free(void)
 	}
 	if (active_plugin_list != NULL)
 	{
-		g_list_foreach(active_plugin_list, (GFunc) plugin_free,	GINT_TO_POINTER(PLUGIN_FREE_ALL));
+		g_list_foreach(active_plugin_list, (GFunc) plugin_free,	NULL);
 		g_list_free(active_plugin_list);
 	}
 	g_strfreev(active_plugins_pref);
@@ -781,42 +910,6 @@ void plugins_free(void)
 }
 
 
-void plugins_update_document_sensitive(gboolean enabled)
-{
-	GList *item;
-
-	for (item = active_plugin_list; item != NULL; item = g_list_next(item))
-	{
-		Plugin *plugin = item->data;
-
-		if (plugin->fields.flags & PLUGIN_IS_DOCUMENT_SENSITIVE)
-			gtk_widget_set_sensitive(plugin->fields.menu_item, enabled);
-	}
-}
-
-
-static gint
-plugin_has_menu(Plugin *a, Plugin *b)
-{
-	if (a->fields.menu_item != NULL)
-		return 0;
-
-	return 1;
-}
-
-
-void plugins_update_tools_menu()
-{
-	gboolean found;
-
-	if (separator == NULL)
-		return;
-
-	found = (g_list_find_custom(active_plugin_list, NULL, (GCompareFunc) plugin_has_menu) != NULL);
-	ui_widget_show_hide(separator, found);
-}
-
-
 /* Plugin Manager */
 
 enum
@@ -835,7 +928,9 @@ typedef struct
 	GtkListStore *store;
 	GtkWidget *description_label;
 	GtkWidget *configure_button;
-} PluginManagerWidgets;
+}
+PluginManagerWidgets;
+
 static PluginManagerWidgets pm_widgets;
 
 
@@ -854,7 +949,7 @@ void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data)
 			gchar *text;
 			PluginInfo *pi;
 
-			pi = p->info();
+			pi = &p->info;
 			text = g_strdup_printf(
 				_("Plugin: %s %s\nDescription: %s\nAuthor(s): %s"),
 				pi->name, pi->version, pi->description, pi->author);
@@ -892,7 +987,7 @@ static void pm_plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer
 	/* unload plugin module */
 	if (!state)
 		keybindings_write_to_file();	/* save shortcuts (only need this group, but it doesn't take long) */
-	plugin_free(p, GINT_TO_POINTER(PLUGIN_FREE_ALL));
+	plugin_free(p);
 
 	/* reload plugin module and initialize it if item is checked */
 	p = plugin_new(file_name, state, TRUE);
@@ -924,7 +1019,7 @@ static void pm_prepare_treeview(GtkWidget *tree, GtkListStore *store)
     column = gtk_tree_view_column_new_with_attributes(
 		_("Active"), checkbox_renderer, "active", PLUGIN_COLUMN_CHECK, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
-	g_signal_connect((gpointer) checkbox_renderer, "toggled", G_CALLBACK(pm_plugin_toggled), NULL);
+	g_signal_connect(checkbox_renderer, "toggled", G_CALLBACK(pm_plugin_toggled), NULL);
 
 	text_renderer = gtk_cell_renderer_text_new();
     column = gtk_tree_view_column_new_with_attributes(
@@ -945,7 +1040,7 @@ static void pm_prepare_treeview(GtkWidget *tree, GtkListStore *store)
 	/* selection handling */
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
 	gtk_tree_selection_set_mode(sel, GTK_SELECTION_SINGLE);
-	g_signal_connect((gpointer) sel, "changed", G_CALLBACK(pm_selection_changed), NULL);
+	g_signal_connect(sel, "changed", G_CALLBACK(pm_selection_changed), NULL);
 
 	list = g_list_first(plugin_list);
 	if (list == NULL)
@@ -959,16 +1054,52 @@ static void pm_prepare_treeview(GtkWidget *tree, GtkListStore *store)
 	{
 		for (; list != NULL; list = list->next)
 		{
+			Plugin *p = list->data;
+
 			gtk_list_store_append(store, &iter);
 			gtk_list_store_set(store, &iter,
-				PLUGIN_COLUMN_CHECK, is_active_plugin(list->data),
-				PLUGIN_COLUMN_NAME, ((Plugin*)list->data)->info()->name,
-				PLUGIN_COLUMN_FILE, ((Plugin*)list->data)->filename,
-				PLUGIN_COLUMN_PLUGIN, list->data,
+				PLUGIN_COLUMN_CHECK, is_active_plugin(p),
+				PLUGIN_COLUMN_NAME, p->info.name,
+				PLUGIN_COLUMN_FILE, p->filename,
+				PLUGIN_COLUMN_PLUGIN, p,
 				-1);
 		}
 	}
 	gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(store));
+	g_object_unref(store);
+}
+
+
+static void configure_plugin(Plugin *p)
+{
+	GtkWidget *parent = pm_widgets.dialog;
+	GtkWidget *prefs_page, *dialog, *vbox;
+
+	dialog = gtk_dialog_new_with_buttons(p->info.name,
+		GTK_WINDOW(parent), GTK_DIALOG_DESTROY_WITH_PARENT,
+		GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+	gtk_widget_set_name(dialog, "GeanyDialog");
+
+	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
+	gtk_widget_show(vbox);
+
+	prefs_page = p->configure(GTK_DIALOG(dialog));
+
+	if (! GTK_IS_WIDGET(prefs_page))
+	{
+		geany_debug("Invalid widget returned from plugin_configure() in plugin \"%s\"!",
+			p->info.name);
+	}
+	else
+	{
+		gtk_container_add(GTK_CONTAINER(vbox), prefs_page);
+
+		/* run the dialog */
+		while (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_APPLY);
+	}
+	gtk_widget_destroy(dialog);
 }
 
 
@@ -986,18 +1117,31 @@ void pm_on_configure_button_clicked(GtkButton *button, gpointer user_data)
 
 		if (p != NULL)
 		{
-			p->configure(pm_widgets.dialog);
+			configure_plugin(p);
 		}
 	}
 }
 
 
+static void
+free_non_active_plugin(gpointer data, gpointer user_data)
+{
+	Plugin *plugin = data;
+
+	/* don't do anything when closing the plugin manager and it is an active plugin */
+	if (is_active_plugin(plugin))
+		return;
+
+	plugin_free(plugin);
+}
+
+
 static void pm_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
 	if (plugin_list != NULL)
 	{
 		/* remove all non-active plugins from the list */
-		g_list_foreach(plugin_list, (GFunc) plugin_free, GINT_TO_POINTER(PLUGIN_FREE_NON_ACTIVE));
+		g_list_foreach(plugin_list, free_non_active_plugin, NULL);
 		g_list_free(plugin_list);
 		plugin_list = NULL;
 	}
@@ -1007,12 +1151,12 @@ static void pm_dialog_response(GtkDialog *dialog, gint response, gpointer user_d
 
 static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 {
-	GtkWidget *vbox, *vbox2, *label_vbox, *hbox, *swin, *label, *label2;
+	GtkWidget *vbox, *vbox2, *label_vbox, *hbox, *swin, *label, *label2, *desc_win;
 
 	/* before showing the dialog, we need to create the list of available plugins */
 	load_all_plugins();
 
-	pm_widgets.dialog = gtk_dialog_new_with_buttons(_("Plugins"), GTK_WINDOW(app->window),
+	pm_widgets.dialog = gtk_dialog_new_with_buttons(_("Plugins"), GTK_WINDOW(main_widgets.window),
 						GTK_DIALOG_DESTROY_WITH_PARENT,
 						GTK_STOCK_OK, GTK_RESPONSE_CANCEL, NULL);
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(pm_widgets.dialog));
@@ -1032,18 +1176,24 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(swin), GTK_SHADOW_IN);
 	gtk_container_add(GTK_CONTAINER(swin), pm_widgets.tree);
 
-	label = gtk_label_new(_("Below is a list of available plugins. "
+	label = geany_wrap_label_new(_("Below is a list of available plugins. "
 		"Select the plugins which should be loaded when Geany is started."));
 
 	pm_widgets.configure_button = gtk_button_new_from_stock(GTK_STOCK_PREFERENCES);
 	gtk_widget_set_sensitive(pm_widgets.configure_button, FALSE);
-	g_signal_connect((gpointer) pm_widgets.configure_button, "clicked",
+	g_signal_connect(pm_widgets.configure_button, "clicked",
 		G_CALLBACK(pm_on_configure_button_clicked), NULL);
 
 	label2 = gtk_label_new(_("<b>Plugin details:</b>"));
 	gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label2), 0, 0.5);
+
 	pm_widgets.description_label = geany_wrap_label_new("");
+	desc_win = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(desc_win),
+		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(desc_win),
+		pm_widgets.description_label);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), label2, TRUE, TRUE, 0);
@@ -1051,18 +1201,57 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 
 	label_vbox = gtk_vbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(label_vbox), hbox, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(label_vbox), pm_widgets.description_label, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(label_vbox), desc_win, FALSE, FALSE, 0);
 
 	vbox2 = gtk_vbox_new(FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(vbox2), swin, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox2), label_vbox, FALSE, FALSE, 0);
 
-	g_signal_connect((gpointer) pm_widgets.dialog, "response",
-		G_CALLBACK(pm_dialog_response), NULL);
+	g_signal_connect(pm_widgets.dialog, "response", G_CALLBACK(pm_dialog_response), NULL);
 
 	gtk_container_add(GTK_CONTAINER(vbox), vbox2);
 	gtk_widget_show_all(pm_widgets.dialog);
 }
 
+
+/** Insert a toolbar item before the Quit button, or after the previous plugin toolbar item.
+ * A separator is added on the first call to this function, and will be shown when @a item is
+ * shown; hidden when @a item is hidden.
+ * @note You should still destroy @a item yourself, usually in @ref plugin_cleanup().
+ * @param plugin Must be @ref geany_plugin.
+ * @param item The item to add. */
+void plugin_add_toolbar_item(GeanyPlugin *plugin, GtkToolItem *item)
+{
+	GtkToolbar *toolbar = GTK_TOOLBAR(main_widgets.toolbar);
+	gint pos;
+	GeanyAutoSeparator *autosep;
+
+	g_return_if_fail(plugin);
+	autosep = &plugin->priv->toolbar_separator;
+
+	if (!autosep->widget)
+	{
+		GtkToolItem *sep;
+
+		pos = ui_get_toolbar_insert_position();
+		/* pos should be valid even if the quit btn is hidden */
+		g_return_if_fail(pos >= 0);
+		gtk_toolbar_insert(toolbar, item, pos);
+
+		sep = gtk_separator_tool_item_new();
+		gtk_toolbar_insert(toolbar, sep, pos + 1);
+		autosep->widget = GTK_WIDGET(sep);
+	}
+	else
+	{
+		pos = gtk_toolbar_get_item_index(toolbar, GTK_TOOL_ITEM(autosep->widget));
+		g_return_if_fail(pos >= 0);
+		gtk_toolbar_insert(toolbar, item, pos);
+	}
+	/* hide the separator widget if there are no toolbar items showing for the plugin */
+	ui_auto_separator_add_ref(autosep, GTK_WIDGET(item));
+}
+
+
 #endif
