@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: printing.c 2287 2008-02-27 13:17:29Z eht16 $
+ * $Id: printing.c 2915 2008-09-01 11:11:33Z eht16 $
  */
 
 
@@ -33,7 +33,6 @@
 
 #include "geany.h"
 #include "printing.h"
-#include "prefs.h"
 #include "document.h"
 #include "sciwrappers.h"
 #include "editor.h"
@@ -42,6 +41,7 @@
 #include "support.h"
 #include "dialogs.h"
 #include "utils.h"
+#include "ui_utils.h"
 #include "msgwindow.h"
 
 
@@ -72,7 +72,7 @@ enum
 /* document-related variables */
 typedef struct
 {
-	gint idx;
+	GeanyDocument *doc;
 	gint font_width;
 	gint lines;
 	gint n_pages;
@@ -213,7 +213,7 @@ static gint get_page_count(GtkPrintContext *context, DocInfo *dinfo)
 		gint lines = 1;
 		gint line_width;
 
-		line_buf = sci_get_line(doc_list[dinfo->idx].sci, j);
+		line_buf = sci_get_line(dinfo->doc->editor->sci, j);
 		line_width = (g_utf8_strlen(line_buf, -1) + 1) * dinfo->font_width;
 		if (line_width > width)
 			lines = ceil(line_width / width);
@@ -240,8 +240,8 @@ static void add_page_header(PangoLayout *layout, cairo_t *cr, DocInfo *dinfo, gi
 	gint ph_height = dinfo->line_height * 3;
 	gchar *data;
 	gchar *datetime;
-	gchar *tmp_file_name = (doc_list[dinfo->idx].file_name != NULL) ?
-		doc_list[dinfo->idx].file_name : GEANY_STRING_UNTITLED;
+	gchar *tmp_file_name = (dinfo->doc->file_name != NULL) ?
+		dinfo->doc->file_name : GEANY_STRING_UNTITLED;
 	gchar *file_name = (printing_prefs.page_header_basename) ?
 		g_path_get_basename(tmp_file_name) : g_strdup(tmp_file_name);
 
@@ -343,21 +343,18 @@ static GtkWidget *create_custom_widget(GtkPrintOperation *operation, gpointer us
 	w->check_print_linenumbers = gtk_check_button_new_with_mnemonic(_("Print line numbers"));
 	gtk_box_pack_start(GTK_BOX(page), w->check_print_linenumbers, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip(tooltips, w->check_print_linenumbers, _("Add line numbers to the printed page."), NULL);
-	gtk_button_set_focus_on_click(GTK_BUTTON(w->check_print_linenumbers), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w->check_print_linenumbers), printing_prefs.print_line_numbers);
 
 	w->check_print_pagenumbers = gtk_check_button_new_with_mnemonic(_("Print page numbers"));
 	gtk_box_pack_start(GTK_BOX(page), w->check_print_pagenumbers, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip(tooltips, w->check_print_pagenumbers, _("Add page numbers at the bottom of each page. It takes 2 lines of the page."), NULL);
-	gtk_button_set_focus_on_click(GTK_BUTTON(w->check_print_pagenumbers), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w->check_print_pagenumbers), printing_prefs.print_page_numbers);
 
 	w->check_print_pageheader = gtk_check_button_new_with_mnemonic(_("Print page header"));
 	gtk_box_pack_start(GTK_BOX(page), w->check_print_pageheader, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip(tooltips, w->check_print_pageheader, _("Adds a little header to every page containing the page number, the filename and the current date(see below). It takes 3 lines of the page."), NULL);
-	gtk_button_set_focus_on_click(GTK_BUTTON(w->check_print_pageheader), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w->check_print_pageheader), printing_prefs.print_page_header);
-	g_signal_connect((gpointer) w->check_print_pageheader, "toggled", G_CALLBACK(on_page_header_toggled), w);
+	g_signal_connect(w->check_print_pageheader, "toggled", G_CALLBACK(on_page_header_toggled), w);
 
 	frame33 = gtk_frame_new(NULL);
 	gtk_box_pack_start(GTK_BOX(page), frame33, FALSE, FALSE, 0);
@@ -374,7 +371,6 @@ static GtkWidget *create_custom_widget(GtkPrintOperation *operation, gpointer us
 	w->check_print_basename = gtk_check_button_new_with_mnemonic(_("Use the basename of the printed file"));
 	gtk_box_pack_start(GTK_BOX(vbox30), w->check_print_basename, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip(tooltips, w->check_print_basename, _("Print only the basename(without the path) of the printed file."), NULL);
-	gtk_button_set_focus_on_click(GTK_BUTTON(w->check_print_basename), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w->check_print_basename), printing_prefs.page_header_basename);
 
 	hbox10 = gtk_hbox_new(FALSE, 5);
@@ -415,10 +411,10 @@ static void begin_print(GtkPrintOperation *operation, GtkPrintContext *context,
 	if (dinfo == NULL)
 		return;
 
-	desc = pango_font_description_from_string(prefs.editor_font);
+	desc = pango_font_description_from_string(interface_prefs.editor_font);
 
 	/* init dinfo fields */
-	dinfo->lines = sci_get_line_count(doc_list[dinfo->idx].sci);
+	dinfo->lines = sci_get_line_count(dinfo->doc->editor->sci);
 	dinfo->lines_per_page = 0;
 	dinfo->cur_line = 0;
 	dinfo->cur_pos = 0;
@@ -434,7 +430,7 @@ static void begin_print(GtkPrintOperation *operation, GtkPrintContext *context,
 	dinfo->n_pages = get_page_count(context, dinfo);
 
 	/* read all styles from Scintilla */
-	style_max = pow(2, scintilla_send_message(doc_list[dinfo->idx].sci, SCI_GETSTYLEBITS, 0, 0));
+	style_max = pow(2, scintilla_send_message(dinfo->doc->editor->sci, SCI_GETSTYLEBITS, 0, 0));
 	/* if the lexer uses only the first 32 styles(style bits = 5),
 	 * we need to add the pre-defined styles */
 	if (style_max == 32)
@@ -442,21 +438,21 @@ static void begin_print(GtkPrintOperation *operation, GtkPrintContext *context,
 	for (i = 0; i < style_max; i++)
 	{
 		dinfo->styles[i][FORE] = ROTATE_RGB(scintilla_send_message(
-			doc_list[dinfo->idx].sci, SCI_STYLEGETFORE, i, 0));
+			dinfo->doc->editor->sci, SCI_STYLEGETFORE, i, 0));
 		if (i == STYLE_LINENUMBER)
 		{	/* ignore background colour for line number margin to avoid trouble with wrapped lines */
 			dinfo->styles[STYLE_LINENUMBER][BACK] = ROTATE_RGB(scintilla_send_message(
-				doc_list[dinfo->idx].sci, SCI_STYLEGETBACK, STYLE_DEFAULT, 0));
+				dinfo->doc->editor->sci, SCI_STYLEGETBACK, STYLE_DEFAULT, 0));
 		}
 		else
 		{
 			dinfo->styles[i][BACK] = ROTATE_RGB(scintilla_send_message(
-				doc_list[dinfo->idx].sci, SCI_STYLEGETBACK, i, 0));
+				dinfo->doc->editor->sci, SCI_STYLEGETBACK, i, 0));
 		}
 		dinfo->styles[i][BOLD] =
-			scintilla_send_message(doc_list[dinfo->idx].sci, SCI_STYLEGETBOLD, i, 0);
+			scintilla_send_message(dinfo->doc->editor->sci, SCI_STYLEGETBOLD, i, 0);
 		dinfo->styles[i][ITALIC] =
-			scintilla_send_message(doc_list[dinfo->idx].sci, SCI_STYLEGETITALIC, i, 0);
+			scintilla_send_message(dinfo->doc->editor->sci, SCI_STYLEGETITALIC, i, 0);
 	}
 
 	if (dinfo->n_pages >= 0)
@@ -470,6 +466,7 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 					  gint page_nr, gpointer user_data)
 {
 	DocInfo *dinfo = user_data;
+	GeanyEditor *editor;
 	cairo_t *cr;
 	gdouble width, height;
 	gdouble x, y;
@@ -480,6 +477,8 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 	if (dinfo == NULL || page_nr >= dinfo->n_pages)
 		return;
 
+	editor = dinfo->doc->editor;
+
 #ifdef GEANY_PRINT_DEBUG
 	geany_debug("draw_page = %d, pages = %d, (real) lines_per_page = %d",
 		page_nr, dinfo->n_pages, dinfo->lines_per_page);
@@ -550,8 +549,8 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 			/* data */
 			else
 			{
-				style = sci_get_style_at(doc_list[dinfo->idx].sci, dinfo->cur_pos);
-				c = sci_get_char_at(doc_list[dinfo->idx].sci, dinfo->cur_pos);
+				style = sci_get_style_at(dinfo->doc->editor->sci, dinfo->cur_pos);
+				c = sci_get_char_at(dinfo->doc->editor->sci, dinfo->cur_pos);
 				if (c == '\0' || style == -1)
 				{	/* if c gets 0, we are probably out of document boundaries,
 					 * so stop to break out of outer loop */
@@ -560,18 +559,18 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 				}
 				dinfo->cur_pos++;
 
-
 				/* convert tabs to spaces which seems to be better than using Pango tabs */
 				if (c == '\t')
 				{
-					gchar *s = g_strnfill(editor_prefs.tab_width, ' ');
+					gint tab_width = sci_get_tab_width(editor->sci);
+					gchar *s = g_strnfill(tab_width, ' ');
 					g_string_append(str, s);
 					g_free(s);
 				}
 				/* don't add line breaks, they are handled manually below */
 				else if (c == '\r' || c == '\n')
 				{
-					gchar c_next = sci_get_char_at(doc_list[dinfo->idx].sci, dinfo->cur_pos);
+					gchar c_next = sci_get_char_at(dinfo->doc->editor->sci, dinfo->cur_pos);
 					at_eol = TRUE;
 					if (c == '\r' && c_next == '\n')
 						dinfo->cur_pos++; /* skip LF part of CR/LF */
@@ -587,7 +586,7 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 					 * style doesn't change since it is only one character with multiple bytes. */
 					while (c < 0)
 					{
-						c = sci_get_char_at(doc_list[dinfo->idx].sci, dinfo->cur_pos);
+						c = sci_get_char_at(dinfo->doc->editor->sci, dinfo->cur_pos);
 						if (c < 0)
 						{	/* only add the byte when it is part of the UTF-8 character
 							 * otherwise we could add e.g. a '\n' and it won't be visible in the
@@ -731,7 +730,7 @@ static void status_changed(GtkPrintOperation *op, gpointer data)
 }
 
 
-static void printing_print_gtk(gint idx)
+static void printing_print_gtk(GeanyDocument *doc)
 {
 	GtkPrintOperation *op;
 	GtkPrintOperationResult res = GTK_PRINT_OPERATION_RESULT_ERROR;
@@ -745,7 +744,7 @@ static void printing_print_gtk(gint idx)
 	widgets = g_new0(PrintWidgets, 1);
 	dinfo = g_new0(DocInfo, 1);
 	/* all other fields are initialised in begin_print() */
-	dinfo->idx = idx;
+	dinfo->doc = doc;
 
 	op = gtk_print_operation_new();
 
@@ -755,7 +754,7 @@ static void printing_print_gtk(gint idx)
 	g_signal_connect(op, "begin-print", G_CALLBACK(begin_print), dinfo);
 	g_signal_connect(op, "end-print", G_CALLBACK(end_print), dinfo);
 	g_signal_connect(op, "draw-page", G_CALLBACK(draw_page), dinfo);
-	g_signal_connect(op, "status-changed", G_CALLBACK(status_changed), doc_list[idx].file_name);
+	g_signal_connect(op, "status-changed", G_CALLBACK(status_changed), doc->file_name);
 	g_signal_connect(op, "create-custom-widget", G_CALLBACK(create_custom_widget), widgets);
 	g_signal_connect(op, "custom-widget-apply", G_CALLBACK(custom_widget_apply), widgets);
 
@@ -765,7 +764,7 @@ static void printing_print_gtk(gint idx)
 		gtk_print_operation_set_default_page_setup(op, page_setup);
 
 	res = gtk_print_operation_run(
-		op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, GTK_WINDOW(app->window), &error);
+		op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, GTK_WINDOW(main_widgets.window), &error);
 
 	if (res == GTK_PRINT_OPERATION_RESULT_APPLY)
 	{
@@ -777,7 +776,7 @@ static void printing_print_gtk(gint idx)
 	else if (res == GTK_PRINT_OPERATION_RESULT_ERROR)
 	{
 		dialogs_show_msgbox(GTK_MESSAGE_ERROR, _("Printing of %s failed (%s)."),
-							doc_list[idx].file_name, error->message);
+							doc->file_name, error->message);
 		g_error_free(error);
 	}
 
@@ -795,7 +794,7 @@ void printing_page_setup_gtk(void)
 		settings = gtk_print_settings_new();
 
 	new_page_setup = gtk_print_run_page_setup_dialog(
-		GTK_WINDOW(app->window), page_setup, settings);
+		GTK_WINDOW(main_widgets.window), page_setup, settings);
 
 	if (page_setup != NULL)
 		g_object_unref(page_setup);
@@ -806,11 +805,11 @@ void printing_page_setup_gtk(void)
 
 
 /* simple file print using an external tool */
-static void print_external(gint idx)
+static void print_external(GeanyDocument *doc)
 {
 	gchar *cmdline;
 
-	if (doc_list[idx].file_name == NULL)
+	if (doc->file_name == NULL)
 		return;
 
 	if (! NZV(printing_prefs.external_print_cmd))
@@ -821,11 +820,11 @@ static void print_external(gint idx)
 	}
 
 	cmdline = g_strdup(printing_prefs.external_print_cmd);
-	cmdline = utils_str_replace(cmdline, "%f", doc_list[idx].file_name);
+	cmdline = utils_str_replace(cmdline, "%f", doc->file_name);
 
 	if (dialogs_show_question(
 			_("The file \"%s\" will be printed with the following command:\n\n%s"),
-			doc_list[idx].file_name, cmdline))
+			doc->file_name, cmdline))
 	{
 		GError *error = NULL;
 
@@ -841,12 +840,12 @@ static void print_external(gint idx)
 		{
 			dialogs_show_msgbox(GTK_MESSAGE_ERROR,
 				_("Printing of \"%s\" failed (return code: %s)."),
-				doc_list[idx].file_name, error->message);
+				doc->file_name, error->message);
 			g_error_free(error);
 		}
 		else
 		{
-			msgwin_status_add(_("File %s printed."), doc_list[idx].file_name);
+			msgwin_status_add(_("File %s printed."), doc->file_name);
 		}
 		g_free(tmp_cmdline);
 	}
@@ -854,16 +853,16 @@ static void print_external(gint idx)
 }
 
 
-void printing_print_doc(gint idx)
+void printing_print_doc(GeanyDocument *doc)
 {
-	if (! DOC_IDX_VALID(idx))
+	if (doc == NULL)
 		return;
 
 #if GTK_CHECK_VERSION(2, 10, 0)
 	if (gtk_check_version(2, 10, 0) == NULL && printing_prefs.use_gtk_printing)
-		printing_print_gtk(idx);
+		printing_print_gtk(doc);
 	else
 #endif
-		print_external(idx);
+		print_external(doc);
 }
 
