@@ -18,10 +18,10 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: project.c 2453 2008-04-08 14:07:17Z eht16 $
+ * $Id: project.c 3037 2008-10-02 15:09:33Z eht16 $
  */
 
-/*
+/** @file project.h
  * Project Management.
  */
 
@@ -35,6 +35,7 @@
 #include "support.h"
 #include "utils.h"
 #include "ui_utils.h"
+#include "document.h"
 #include "msgwindow.h"
 #include "main.h"
 #include "keyfile.h"
@@ -42,12 +43,10 @@
 # include "win32.h"
 #endif
 #include "build.h"
-#include "document.h"
 #include "geanyobject.h"
-#include "callbacks.h"
 
 
-ProjectPrefs project_prefs = { NULL, 0 };
+ProjectPrefs project_prefs = { NULL, FALSE, FALSE };
 
 static struct
 {
@@ -98,7 +97,7 @@ void project_new()
 	GtkWidget *button;
 	GtkWidget *bbox;
 	GtkWidget *label;
-	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
+	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
 	PropertyDialogElements *e;
 	gint response;
 
@@ -107,7 +106,7 @@ void project_new()
 	g_return_if_fail(app->project == NULL);
 
 	e = g_new0(PropertyDialogElements, 1);
-	e->dialog = gtk_dialog_new_with_buttons(_("New Project"), GTK_WINDOW(app->window),
+	e->dialog = gtk_dialog_new_with_buttons(_("New Project"), GTK_WINDOW(main_widgets.window),
 										 GTK_DIALOG_DESTROY_WITH_PARENT,
 										 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
 
@@ -143,8 +142,7 @@ void project_new()
 	e->file_name = gtk_entry_new();
 	gtk_entry_set_width_chars(GTK_ENTRY(e->file_name), 30);
 	button = gtk_button_new();
-	g_signal_connect((gpointer) button, "clicked",
-				G_CALLBACK(on_file_save_button_clicked), e);
+	g_signal_connect(button, "clicked", G_CALLBACK(on_file_save_button_clicked), e);
 	image = gtk_image_new_from_stock("gtk-open", GTK_ICON_SIZE_BUTTON);
 	gtk_container_add(GTK_CONTAINER(button), image);
 	bbox = gtk_hbox_new(FALSE, 6);
@@ -169,12 +167,12 @@ void project_new()
 	gtk_container_add(GTK_CONTAINER(vbox), table);
 
 	/* signals */
-	g_signal_connect((gpointer) e->name, "changed", G_CALLBACK(on_name_entry_changed), e);
+	g_signal_connect(e->name, "changed", G_CALLBACK(on_name_entry_changed), e);
 	/* run the callback manually to initialise the base_path and file_name fields */
 	on_name_entry_changed(GTK_EDITABLE(e->name), e);
 
-	g_signal_connect((gpointer) e->file_name, "changed", G_CALLBACK(on_entries_changed), e);
-	g_signal_connect((gpointer) e->base_path, "changed", G_CALLBACK(on_entries_changed), e);
+	g_signal_connect(e->file_name, "changed", G_CALLBACK(on_entries_changed), e);
+	g_signal_connect(e->base_path, "changed", G_CALLBACK(on_entries_changed), e);
 
 	gtk_widget_show_all(e->dialog);
 
@@ -237,7 +235,7 @@ void project_open()
 	if (! close_open_project()) return;
 
 #ifdef G_OS_WIN32
-	file = win32_show_project_open_dialog(app->window, _("Open Project"), dir, FALSE, TRUE);
+	file = win32_show_project_open_dialog(main_widgets.window, _("Open Project"), dir, FALSE, TRUE);
 	if (file != NULL)
 	{
 		/* try to load the config */
@@ -255,7 +253,7 @@ void project_open()
 	}
 #else
 
-	dialog = gtk_file_chooser_dialog_new(_("Open Project"), GTK_WINDOW(app->window),
+	dialog = gtk_file_chooser_dialog_new(_("Open Project"), GTK_WINDOW(main_widgets.window),
 			GTK_FILE_CHOOSER_ACTION_OPEN,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
@@ -266,7 +264,7 @@ void project_open()
 	gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);
 	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(dialog), TRUE);
 	gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
-	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(app->window));
+	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(main_widgets.window));
 	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
 
 	/* add FileFilters */
@@ -295,18 +293,17 @@ void project_open()
 }
 
 
-/* Called when opening, closing and updating projects. */
+/* Called when creating, opening, closing and updating projects. */
 static void update_ui(void)
 {
-	ui_set_window_title(-1);
-	build_menu_update(-1);
+	ui_set_window_title(NULL);
+	build_menu_update(NULL);
 }
 
 
 /* open_default will make function reload default session files on close */
 void project_close(gboolean open_default)
 {
-	gint i, max = gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook));
 	g_return_if_fail(app->project != NULL);
 
 	ui_set_statusbar(TRUE, _("Project \"%s\" closed."), app->project->name);
@@ -326,13 +323,7 @@ void project_close(gboolean open_default)
 	if (project_prefs.project_session)
 	{
 		/* close all existing tabs first */
-		main_status.closing_all = TRUE;
-		for (i = 0; i < max; i++)
-		{
-			if (! document_remove(0))
-				break;
-		}
-		main_status.closing_all = FALSE;
+		document_close_all();
 
 		/* after closing all tabs let's open the tabs found in the default config */
 		if (open_default == TRUE && cl_options.load_session)
@@ -348,8 +339,6 @@ void project_close(gboolean open_default)
 	{
 		g_signal_emit_by_name(geany_object, "project-close");
 	}
-
-	tm_workspace_update(TM_WORK_OBJECT(app->tm_workspace), TRUE, TRUE, FALSE);
 	update_ui();
 }
 
@@ -363,9 +352,9 @@ static void create_properties_dialog(PropertyDialogElements *e)
 	GtkWidget *bbox;
 	GtkWidget *label;
 	GtkWidget *swin;
-	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
+	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
 
-	e->dialog = gtk_dialog_new_with_buttons(_("Project Properties"), GTK_WINDOW(app->window),
+	e->dialog = gtk_dialog_new_with_buttons(_("Project Properties"), GTK_WINDOW(main_widgets.window),
 										 GTK_DIALOG_DESTROY_WITH_PARENT,
 										 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
 	gtk_dialog_add_buttons(GTK_DIALOG(e->dialog), GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
@@ -452,8 +441,7 @@ static void create_properties_dialog(PropertyDialogElements *e)
 		"Options can be appended to the command. "
 		"Leave blank to use the default run command."), NULL);
 	button = gtk_button_new();
-	g_signal_connect((gpointer) button, "clicked",
-				G_CALLBACK(on_file_open_button_clicked), e->run_cmd);
+	g_signal_connect(button, "clicked", G_CALLBACK(on_file_open_button_clicked), e->run_cmd);
 	image = gtk_image_new_from_stock("gtk-open", GTK_ICON_SIZE_BUTTON);
 	gtk_container_add(GTK_CONTAINER(button), image);
 	bbox = gtk_hbox_new(FALSE, 6);
@@ -543,8 +531,6 @@ void project_properties()
 	{
 		if (! update_config(e))
 			goto retry;
-		/* successfully updated properties */
-		update_ui();
 	}
 
 	gtk_widget_destroy(e->dialog);
@@ -581,7 +567,7 @@ static gboolean update_config(const PropertyDialogElements *e)
 	const gchar *name, *file_name, *base_path;
 	gchar *locale_filename;
 	gint name_len;
-	gint err_code;
+	gint err_code = 0;
 	gboolean new_project = FALSE;
 	GeanyProject *p;
 
@@ -603,22 +589,14 @@ static gboolean update_config(const PropertyDialogElements *e)
 	}
 
 	file_name = gtk_entry_get_text(GTK_ENTRY(e->file_name));
-	if (strlen(file_name) == 0)
+	if (! NZV(file_name))
 	{
 		SHOW_ERR(_("You have specified an invalid project filename."));
 		gtk_widget_grab_focus(e->file_name);
 		return FALSE;
 	}
 
-	/* finally test whether the given project file can be written */
 	locale_filename = utils_get_locale_from_utf8(file_name);
-	if ((err_code = utils_is_file_writeable(locale_filename)) != 0)
-	{
-		SHOW_ERR1(_("Project file could not be written (%s)."), g_strerror(err_code));
-		gtk_widget_grab_focus(e->file_name);
-		return FALSE;
-	}
-
 	base_path = gtk_entry_get_text(GTK_ENTRY(e->base_path));
 	if (NZV(base_path))
 	{	/* check whether the given directory actually exists */
@@ -633,22 +611,36 @@ static gboolean update_config(const PropertyDialogElements *e)
 
 		if (! g_file_test(locale_path, G_FILE_TEST_IS_DIR))
 		{
-			if (dialogs_show_question_full(NULL, GTK_STOCK_OK, GTK_STOCK_CANCEL,
+			gboolean create_dir;
+
+			create_dir = dialogs_show_question_full(NULL, GTK_STOCK_OK, GTK_STOCK_CANCEL,
 				_("Create the project's base path directory?"),
 				_("The path \"%s\" does not exist."),
-				base_path))
+				base_path);
+
+			if (create_dir)
+				err_code = utils_mkdir(locale_path, TRUE);
+
+			if (! create_dir || err_code != 0)
 			{
-				utils_mkdir(locale_path, TRUE);
-			}
-			else
-			{
-				g_free(locale_path);
+				if (err_code != 0)
+					SHOW_ERR1(_("Project base directory could not be created (%s)."),
+						g_strerror(err_code));
 				gtk_widget_grab_focus(e->base_path);
+				utils_free_pointers(locale_path, locale_filename, NULL);
 				return FALSE;
 			}
 		}
 		g_free(locale_path);
 	}
+	/* finally test whether the given project file can be written */
+	if ((err_code = utils_is_file_writeable(locale_filename)) != 0)
+	{
+		SHOW_ERR1(_("Project file could not be written (%s)."), g_strerror(err_code));
+		gtk_widget_grab_focus(e->file_name);
+		g_free(locale_filename);
+		return FALSE;
+	}
 	g_free(locale_filename);
 
 	if (app->project == NULL)
@@ -658,14 +650,10 @@ static gboolean update_config(const PropertyDialogElements *e)
 	}
 	p = app->project;
 
-	if (p->name != NULL) g_free(p->name);
-	p->name = g_strdup(name);
-
-	if (p->file_name != NULL) g_free(p->file_name);
-	p->file_name = g_strdup(file_name);
-
-	if (p->base_path != NULL) g_free(p->base_path);
-	p->base_path = g_strdup(NZV(base_path) ? base_path : "./"); /* use "." if base_path is empty */
+	setptr(p->name, g_strdup(name));
+	setptr(p->file_name, g_strdup(file_name));
+	/* use "." if base_path is empty */
+	setptr(p->base_path, g_strdup(NZV(base_path) ? base_path : "./"));
 
 	if (! new_project)	/* save properties specific fields */
 	{
@@ -675,15 +663,13 @@ static gboolean update_config(const PropertyDialogElements *e)
 
 		p->make_in_base_path = gtk_toggle_button_get_active(
 			GTK_TOGGLE_BUTTON(e->make_in_base_path));
-		if (p->run_cmd != NULL) g_free(p->run_cmd);
-		p->run_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(e->run_cmd)));
+		setptr(p->run_cmd, g_strdup(gtk_entry_get_text(GTK_ENTRY(e->run_cmd))));
 
 		/* get and set the project description */
 		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(e->description));
 		gtk_text_buffer_get_start_iter(buffer, &start);
 		gtk_text_buffer_get_end_iter(buffer, &end);
-		g_free(p->description);
-		p->description = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
+		setptr(p->description, g_strdup(gtk_text_buffer_get_text(buffer, &start, &end, FALSE)));
 
 #if 0
 		/* get and set the project file patterns */
@@ -702,6 +688,8 @@ static gboolean update_config(const PropertyDialogElements *e)
 	else
 		ui_set_statusbar(TRUE, _("Project \"%s\" saved."), p->name);
 
+	update_ui();
+
 	return TRUE;
 }
 
@@ -835,8 +823,12 @@ static void on_name_entry_changed(GtkEditable *editable, PropertyDialogElements
 	{
 		base_path = g_strconcat(project_dir, G_DIR_SEPARATOR_S,
 			name, G_DIR_SEPARATOR_S, NULL);
-		file_name = g_strconcat(project_dir, G_DIR_SEPARATOR_S,
-			name, "." GEANY_PROJECT_EXT, NULL);
+		if (project_prefs.project_file_in_basedir)
+			file_name = g_strconcat(project_dir, G_DIR_SEPARATOR_S, name, G_DIR_SEPARATOR_S,
+				name, "." GEANY_PROJECT_EXT, NULL);
+		else
+			file_name = g_strconcat(project_dir, G_DIR_SEPARATOR_S,
+				name, "." GEANY_PROJECT_EXT, NULL);
 	}
 	else
 	{
@@ -915,9 +907,7 @@ static gboolean load_config(const gchar *filename)
 		/* save current (non-project) session (it could has been changed since program startup) */
 		configuration_save_default_session();
 		/* now close all open files */
-		/** TODO make this a general, non-callback function, use it also in project_close()
-		  * and remove include of callbacks.h */
-		on_close_all1_activate(NULL, NULL);
+		document_close_all();
 		/* read session files so they can be opened with configuration_open_files() */
 		configuration_load_session_files(config);
 	}
