@@ -17,12 +17,15 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sciwrappers.c 2312 2008-03-07 15:42:46Z eht16 $
+ * $Id: sciwrappers.c 3000 2008-09-25 18:29:53Z eht16 $
  */
 
-/*
- * Wrappers for the SCI_* Scintilla messages.
- * Originally from the cssed project (http://cssed.sf.net).
+/** @file sciwrappers.h
+ * Wrapper functions for the Scintilla editor widget @c SCI_* messages.
+ * You should also check the http://scintilla.org documentation, as it is more detailed.
+ * @note These functions were originally from the cssed project
+ * (http://cssed.sf.net, thanks).
+ * @see ScintillaFuncs::send_message().
  */
 
 #include <string.h>
@@ -35,9 +38,6 @@
 #define SSM(s, m, w, l) scintilla_send_message(s, m, w, l)
 
 
-/* stolen from cssed (http://cssed.sf.net), thanks */
-
-
 /* line numbers visibility */
 void sci_set_line_numbers(ScintillaObject * sci, gboolean set, gint extra_width)
 {
@@ -196,6 +196,9 @@ void sci_add_text(ScintillaObject* sci, const gchar* text)
 	}
 }
 
+/** Set all text.
+ * @param sci Scintilla widget.
+ * @param text Text. */
 void sci_set_text(ScintillaObject* sci, const gchar* text)
 {
 	if( text != NULL ){ /* if null text is passed to scintilla will segfault */
@@ -247,12 +250,18 @@ void sci_redo( ScintillaObject* sci )
 }
 
 
+/** Begin grouping a set of edits together as one Undo action.
+ * You must call sci_end_undo_action() after making your edits.
+ * @param sci Scintilla @c GtkWidget. */
 void sci_start_undo_action( ScintillaObject* sci )
 {
 	SSM( sci,SCI_BEGINUNDOACTION,0,0 );
 }
 
 
+/** End grouping a set of edits together as one Undo action.
+ * @param sci Scintilla @c GtkWidget.
+ * @see sci_start_undo_action(). */
 void sci_end_undo_action( ScintillaObject* sci )
 {
 	SSM( sci, SCI_ENDUNDOACTION,0,0);
@@ -362,30 +371,47 @@ gint sci_marker_previous(ScintillaObject* sci, gint line, gint marker_mask, gboo
 }
 
 
+/** Get line number from position.
+ * @param sci Scintilla widget.
+ * @param position Position. */
 gint sci_get_line_from_position(ScintillaObject* sci, gint position )
 {
 	return SSM(sci, SCI_LINEFROMPOSITION, position, 0);
 }
 
 
+/** Get column number relative to the start of the line that @a position is on.
+ * @param sci Scintilla widget.
+ * @param position Position. */
 gint sci_get_col_from_position(ScintillaObject* sci, gint position )
 {
 	return SSM(sci, SCI_GETCOLUMN, position, 0);
 }
 
 
+/** Get position for the start of @a line.
+ * @param sci Scintilla widget.
+ * @param line Line.
+ * @return Position. */
 gint sci_get_position_from_line(ScintillaObject* sci, gint line )
 {
 	return SSM(sci, SCI_POSITIONFROMLINE, line, 0);
 }
 
 
+/** Get cursor position.
+ * @param sci Scintilla widget.
+ * @return Position. */
 gint sci_get_current_position(ScintillaObject* sci )
 {
 	return SSM(sci, SCI_GETCURRENTPOS, 0, 0);
 }
 
 
+/** Set the cursor position.
+ * @param sci Scintilla widget.
+ * @param position Position.
+ * @param scroll_to_caret Whether to scroll the cursor in view. */
 void sci_set_current_position(ScintillaObject* sci, gint position, gboolean scroll_to_caret)
 {
 	if (scroll_to_caret)
@@ -408,18 +434,26 @@ void sci_set_current_line(ScintillaObject* sci, gint line)
 }
 
 
+/** Get total number of lines.
+ * @param sci Scintilla widget. */
 gint sci_get_line_count( ScintillaObject* sci )
 {
 	return SSM(sci, SCI_GETLINECOUNT, 0, 0);
 }
 
 
+/** Set selection start position.
+ * @param sci Scintilla widget.
+ * @param position Position. */
 void sci_set_selection_start(ScintillaObject* sci, gint position)
 {
 	SSM(sci, SCI_SETSELECTIONSTART, position, 0);
 }
 
 
+/** Set selection end position.
+ * @param sci Scintilla widget.
+ * @param position Position. */
 void sci_set_selection_end(ScintillaObject* sci, gint position)
 {
 	SSM(sci, SCI_SETSELECTIONEND, position, 0);
@@ -456,24 +490,36 @@ void sci_clear(ScintillaObject* sci)
 }
 
 
+/** Get position of selection start.
+ * @param sci Scintilla widget.
+ * @return Position. */
 gint sci_get_selection_start(ScintillaObject* sci)
 {
 	return SSM(sci, SCI_GETSELECTIONSTART,0,0);
 }
 
 
+/** Get position of selection end.
+ * @param sci Scintilla widget.
+ * @return Position. */
 gint sci_get_selection_end(ScintillaObject* sci)
 {
 	return SSM(sci, SCI_GETSELECTIONEND,0,0);
 }
 
 
+/** Replace selection.
+ * @param sci Scintilla widget.
+ * @param text Text. */
 void sci_replace_sel(ScintillaObject* sci, const gchar* text)
 {
 	SSM(sci, SCI_REPLACESEL,0, (sptr_t) text);
 }
 
 
+/** Get length of all text.
+ * @param sci Scintilla widget.
+ * @return Length. */
 gint sci_get_length(ScintillaObject* sci)
 {
 	return SSM(sci,SCI_GETLENGTH,0,0);
@@ -486,13 +532,21 @@ gint sci_get_lexer(ScintillaObject* sci)
 }
 
 
+/** Get line length.
+ * @param sci Scintilla widget.
+ * @param line Line number.
+ * @return Length. */
 gint sci_get_line_length(ScintillaObject* sci,gint line)
 {
 	return SSM(sci,SCI_LINELENGTH, line,0);
 }
 
 
-/* Returns: a NULL-terminated copy of the line text */
+/* TODO: rename/change to use buffer? Otherwise inconsistent with sci_get_text*(). */
+/** Get line contents.
+ * @param sci Scintilla widget.
+ * @param line_num Line number.
+ * @return A @c NULL-terminated copy of the line text. */
 gchar *sci_get_line(ScintillaObject* sci, gint line_num)
 {
 	gint len = sci_get_line_length(sci, line_num);
@@ -504,21 +558,29 @@ gchar *sci_get_line(ScintillaObject* sci, gint line_num)
 }
 
 
-/* the last char will be null terminated */
+/** Get all text.
+ * @param sci Scintilla widget.
+ * @param len Length of @a text buffer, usually sci_get_length() + 1.
+ * @param text Text buffer; must be allocated @a len + 1 bytes for null-termination. */
 void sci_get_text(ScintillaObject* sci, gint len, gchar* text)
 {
 	SSM( sci, SCI_GETTEXT, len, (sptr_t) text );
 }
 
 
-/* Text must be allocated sci_get_selected_text_length() + 1, because
- * the selection will be NULL-terminated. */
+/** Get selected text.
+ * @param sci Scintilla widget.
+ * @param text Text buffer; must be allocated sci_get_selected_text_length() + 1 bytes
+ * for null-termination. */
 void sci_get_selected_text(ScintillaObject* sci, gchar* text)
 {
 	SSM( sci, SCI_GETSELTEXT, 0, (sptr_t) text);
 }
 
 
+/** Get selected text length.
+ * @param sci Scintilla widget.
+ * @return Length. */
 gint sci_get_selected_text_length(ScintillaObject* sci)
 {
 	return SSM( sci, SCI_GETSELTEXT, 0, 0);
@@ -539,13 +601,19 @@ void sci_get_xy_from_position(ScintillaObject* sci,gint pos, gint* x, gint* y)
 }
 
 
-/* folding */
+/** Check if a line is visible (folding may have hidden it).
+ * @param sci Scintilla widget.
+ * @param line Line number.
+ * @return Whether @a line will be drawn on the screen. */
 gboolean sci_get_line_is_visible(ScintillaObject* sci, gint line)
 {
 	return SSM(sci,SCI_GETLINEVISIBLE, line,0);
 }
 
 
+/** Make the @a line visible (folding may have hidden it).
+ * @param sci Scintilla widget.
+ * @param line Line number. */
 void sci_ensure_line_is_visible(ScintillaObject* sci, gint line)
 {
 	SSM(sci,SCI_ENSUREVISIBLE,line,0);
@@ -614,12 +682,19 @@ void sci_set_tab_width(ScintillaObject * sci, gint width)
 }
 
 
+/** Get display tab width (this is not indent width, see GeanyIndentPrefs).
+ * @param sci Scintilla widget.
+ * @return Width. */
 gint sci_get_tab_width(ScintillaObject * sci)
 {
 	return SSM(sci, SCI_GETTABWIDTH, 0, 0);
 }
 
 
+/** Get character.
+ * @param sci Scintilla widget.
+ * @param pos Position.
+ * @return Char. */
 gchar sci_get_char_at(ScintillaObject *sci, gint pos)
 {
 	return (gchar) SSM(sci, SCI_GETCHARAT, pos, 0);
@@ -631,10 +706,9 @@ void sci_set_savepoint(ScintillaObject *sci)
 	SSM(sci, SCI_SETSAVEPOINT, 0, 0);
 }
 
-
-void sci_set_indentation_guides(ScintillaObject *sci, gboolean enable)
+void sci_set_indentation_guides(ScintillaObject *sci, gint mode)
 {
-	SSM(sci, SCI_SETINDENTATIONGUIDES, enable, 0);
+	SSM(sci, SCI_SETINDENTATIONGUIDES, mode, 0);
 }
 
 
@@ -644,8 +718,10 @@ void sci_use_popup(ScintillaObject *sci, gboolean enable)
 }
 
 
-/* you can also call this has_selection */
-gboolean sci_can_copy(ScintillaObject *sci)
+/** Check if there's a selection.
+ * @param sci Scintilla widget.
+ * @return Whether a selection is present. */
+gboolean sci_has_selection(ScintillaObject *sci)
 {
 	if (SSM(sci, SCI_GETSELECTIONEND,0,0) - SSM(sci, SCI_GETSELECTIONSTART,0,0))
 		return TRUE;
@@ -673,6 +749,8 @@ void sci_set_anchor(ScintillaObject *sci, gint pos)
 }
 
 
+/** Scroll the cursor in view.
+ * @param sci Scintilla widget. */
 void sci_scroll_caret(ScintillaObject *sci)
 {
 	SSM(sci, SCI_SCROLLCARET, 0, 0);
@@ -710,9 +788,9 @@ void sci_set_font(ScintillaObject *sci, gint style, const gchar* font, gint size
 }
 
 
-void sci_goto_line(ScintillaObject *sci, gint line, gboolean ensure_visibility)
+void sci_goto_line(ScintillaObject *sci, gint line, gboolean unfold)
 {
-	if (ensure_visibility) SSM(sci,SCI_ENSUREVISIBLE,line,0);
+	if (unfold) SSM(sci, SCI_ENSUREVISIBLE, line, 0);
 	SSM(sci, SCI_GOTOLINE, line, 0);
 }
 
@@ -723,6 +801,10 @@ void sci_marker_delete_all(ScintillaObject *sci, gint marker)
 }
 
 
+/** Get style ID for @a position.
+ * @param sci Scintilla widget.
+ * @param position Position.
+ * @return Style ID. */
 gint sci_get_style_at(ScintillaObject *sci, gint position)
 {
 	return SSM(sci, SCI_GETSTYLEAT, position, 0);
@@ -747,7 +829,11 @@ void sci_clear_cmdkey(ScintillaObject *sci, gint key)
 }
 
 
-/* text will be zero terminated and must be allocated (end - start + 1) bytes */
+/** Get text between @a start and @a end.
+ * @param sci Scintilla widget.
+ * @param start Start.
+ * @param end End.
+ * @param text Text will be zero terminated and must be allocated (end - start + 1) bytes. */
 void sci_get_text_range(ScintillaObject *sci, gint start, gint end, gchar *text)
 {
 	struct TextRange tr;
@@ -770,13 +856,17 @@ void sci_selection_duplicate(ScintillaObject *sci)
 }
 
 
+/** Insert text.
+ * @param sci Scintilla widget.
+ * @param pos Position.
+ * @param text Text. */
 void sci_insert_text(ScintillaObject *sci, gint pos, const gchar *text)
 {
 	SSM(sci, SCI_INSERTTEXT, pos, (sptr_t) text);
 }
 
 
-void sci_grap_focus(ScintillaObject *sci)
+void sci_grab_focus(ScintillaObject *sci)
 {
 	SSM(sci, SCI_GRABFOCUS, 0, 0);
 }
@@ -827,13 +917,19 @@ gboolean sci_get_readonly(ScintillaObject *sci)
 	return SSM(sci, SCI_GETREADONLY, 0, 0);
 }
 
-/* a simple convenience function to not have SSM() in the outside of this file */
- void sci_cmd(ScintillaObject * sci, gint cmd)
+/** A simple convenience function for sending Scintilla commands without any parameters.
+ * @param sci The Scintilla @c GtkWidget.
+ * @param cmd @c SCI_COMMAND.
+ * @see http://scintilla.org for the documentation. */
+void sci_cmd(ScintillaObject * sci, gint cmd)
 {
 	SSM(sci, cmd, 0, 0);
 }
 
 
+/** Get current line number.
+ * @param sci Scintilla widget.
+ * @return Line number. */
 gint sci_get_current_line(ScintillaObject *sci)
 {
 	return SSM(sci, SCI_LINEFROMPOSITION, SSM(sci, SCI_GETCURRENTPOS, 0, 0), 0);
@@ -871,6 +967,30 @@ void sci_start_styling(ScintillaObject *sci, gint pos, gint mask)
 	SSM(sci, SCI_STARTSTYLING, pos, mask);
 }
 
+gint sci_get_indicator(ScintillaObject *sci)
+{
+	return SSM(sci, SCI_GETINDICATORCURRENT, 0, 0);
+}
+
+void sci_set_indicator(ScintillaObject *sci, gint indic)
+{
+	SSM(sci, SCI_SETINDICATORCURRENT, indic, 0);
+}
+
+void sci_indicator_fill(ScintillaObject *sci, gint pos, gint len)
+{
+	SSM(sci, SCI_INDICATORFILLRANGE, pos, len);
+}
+
+/** Clear a range of text from any set indicators. Starting at @a pos, @a len characters long.
+ * @param sci Scintilla widget.
+ * @param pos Starting position.
+ * @param len Length. */
+void sci_indicator_clear(ScintillaObject *sci, gint pos, gint len)
+{
+	SSM(sci, SCI_INDICATORCLEARRANGE, pos, len);
+}
+
 void sci_select_all(ScintillaObject *sci)
 {
 	SSM(sci, SCI_SELECTALL, 0, 0);
@@ -886,7 +1006,11 @@ void sci_set_autoc_max_height(ScintillaObject *sci, gint val)
 	SSM(sci, SCI_AUTOCSETMAXHEIGHT, val, 0);
 }
 
-gint sci_find_bracematch(ScintillaObject *sci, gint pos)
+/** Find a matching brace at @a pos.
+ * @param sci Scintilla widget.
+ * @param pos Position.
+ * @return Matching brace position. */
+gint sci_find_matching_brace(ScintillaObject *sci, gint pos)
 {
 	return SSM(sci, SCI_BRACEMATCH, pos, 0);
 }
@@ -916,11 +1040,17 @@ gint sci_get_pos_at_line_sel_end(ScintillaObject *sci, gint line)
 	return SSM(sci, SCI_GETLINESELENDPOSITION, line, 0);
 }
 
+/** Get selection mode.
+ * @param sci Scintilla widget.
+ * @return Selection mode. */
 gint sci_get_selection_mode(ScintillaObject *sci)
 {
 	return SSM(sci, SCI_GETSELECTIONMODE, 0, 0);
 }
 
+/** Set selection mode.
+ * @param sci Scintilla widget.
+ * @param mode Mode. */
 void sci_set_selection_mode(ScintillaObject *sci, gint mode)
 {
 	SSM(sci, SCI_SETSELECTIONMODE, mode, 0);
@@ -952,3 +1082,24 @@ void sci_set_caret_policy_y(ScintillaObject *sci, gint policy, gint slop)
 	SSM(sci, SCI_SETYCARETPOLICY, policy, slop);
 }
 
+void sci_set_scroll_stop_at_last_line(ScintillaObject* sci, gboolean set)
+{
+	SSM(sci, SCI_SETENDATLASTLINE, set, 0);
+}
+
+void sci_cancel(ScintillaObject *sci)
+{
+	SSM( sci, SCI_CANCEL, 0, 0);
+}
+
+gint sci_get_target_end(ScintillaObject *sci)
+{
+	return SSM( sci, SCI_GETTARGETEND, 0, 0);
+}
+
+gint sci_get_position_after(ScintillaObject *sci, gint start)
+{
+	return SSM(sci, SCI_POSITIONAFTER, start, 0);
+}
+
+
