@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.c 2389 2008-03-23 16:29:43Z eht16 $
+ * $Id: search.c 3068 2008-10-12 13:07:56Z frlan $
  */
 
 /*
@@ -33,8 +33,8 @@
 #include "prefs.h"
 #include "support.h"
 #include "utils.h"
-#include "msgwindow.h"
 #include "document.h"
+#include "msgwindow.h"
 #include "sciwrappers.h"
 #include "ui_utils.h"
 #include "editor.h"
@@ -64,7 +64,8 @@ enum {
 
 GeanySearchData search_data;
 
-SearchPrefs search_prefs = {NULL};
+GeanySearchPrefs search_prefs;
+
 
 static struct
 {
@@ -133,7 +134,7 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 {
 	GtkWidget *checkbox1, *checkbox2, *check_regexp, *check_back, *checkbox5,
 			  *checkbox7, *hbox, *fbox, *mbox;
-	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
+	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
 
 	check_regexp = gtk_check_button_new_with_mnemonic(_("_Use regular expressions"));
 	g_object_set_data_full(G_OBJECT(dialog), "check_regexp",
@@ -141,7 +142,7 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_regexp), FALSE);
 	gtk_tooltips_set_tip(tooltips, check_regexp, _("Use POSIX-like regular expressions. "
 		"For detailed information about using regular expressions, please read the documentation."), NULL);
-	g_signal_connect(G_OBJECT(check_regexp), "toggled",
+	g_signal_connect(check_regexp, "toggled",
 		G_CALLBACK(on_find_replace_checkbutton_toggled), GTK_WIDGET(dialog));
 
 	if (dialog != GTK_DIALOG(widgets.find_dialog))
@@ -226,36 +227,36 @@ static void setup_find_next(const gchar *text)
  * the scintilla selection or current token instead.
  * Search flags are always zero.
  */
-void search_find_selection(gint idx, gboolean search_backwards)
+void search_find_selection(GeanyDocument *doc, gboolean search_backwards)
 {
 	gchar *s = NULL;
 #ifdef G_OS_UNIX
 	GtkClipboard *clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
 #endif
 
-	g_return_if_fail(DOC_IDX_VALID(idx));
+	g_return_if_fail(doc != NULL);
 
 #ifdef G_OS_UNIX
-	s=gtk_clipboard_wait_for_text(clipboard);
+	s = gtk_clipboard_wait_for_text(clipboard);
 	if (s)
 	{
 		if (strchr(s,'\n') || strchr(s, '\r'))
 		{
 			g_free(s);
-			s=NULL;
+			s = NULL;
 		};
 	}
 #endif
 
 	if (!s)
-		s=editor_get_default_selection(idx, TRUE, NULL);
+		s = editor_get_default_selection(doc->editor, TRUE, NULL);
 
 	if (s)
 	{
 		setup_find_next(s);	/* allow find next/prev */
 
-		if (document_find_text(idx, s, 0, search_backwards, FALSE, NULL) > -1)
-			editor_display_current_line(idx, 0.3F);
+		if (document_find_text(doc, s, 0, search_backwards, FALSE, NULL) > -1)
+			editor_display_current_line(doc->editor, 0.3F);
 		g_free(s);
 	}
 }
@@ -285,23 +286,23 @@ static void load_monospace_style()
 
 void search_show_find_dialog(void)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	gchar *sel = NULL;
 
-	g_return_if_fail(DOC_IDX_VALID(idx));
+	g_return_if_fail(doc != NULL);
 
-	sel = editor_get_default_selection(idx, prefs.search_use_current_word, NULL);
+	sel = editor_get_default_selection(doc->editor, search_prefs.use_current_word, NULL);
 
 	if (widgets.find_dialog == NULL)
 	{
 		GtkWidget *label, *entry, *sbox, *vbox;
 		GtkWidget *exp, *bbox, *button, *check_close;
-		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
+		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
 
 		load_monospace_style();
 
 		widgets.find_dialog = gtk_dialog_new_with_buttons(_("Find"),
-			GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_WINDOW(main_widgets.window), GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
 		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.find_dialog));
 		gtk_widget_set_name(widgets.find_dialog, "GeanyDialogSearch");
@@ -328,11 +329,11 @@ void search_show_find_dialog(void)
 		g_object_set_data_full(G_OBJECT(widgets.find_dialog), "entry",
 						gtk_widget_ref(entry), (GDestroyNotify)gtk_widget_unref);
 
-		g_signal_connect((gpointer) gtk_bin_get_child(GTK_BIN(entry)), "activate",
+		g_signal_connect(gtk_bin_get_child(GTK_BIN(entry)), "activate",
 				G_CALLBACK(on_find_entry_activate), NULL);
-		g_signal_connect((gpointer) widgets.find_dialog, "response",
+		g_signal_connect(widgets.find_dialog, "response",
 				G_CALLBACK(on_find_dialog_response), entry);
-		g_signal_connect((gpointer) widgets.find_dialog, "delete_event",
+		g_signal_connect(widgets.find_dialog, "delete-event",
 				G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
 		sbox = gtk_hbox_new(FALSE, 6);
@@ -351,17 +352,17 @@ void search_show_find_dialog(void)
 		gtk_tooltips_set_tip(tooltips, button,
 				_("Mark all matches in the current document."), NULL);
 		gtk_container_add(GTK_CONTAINER(bbox), button);
-		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_find_dialog_response),
+		g_signal_connect(button, "clicked", G_CALLBACK(send_find_dialog_response),
 			GINT_TO_POINTER(GEANY_RESPONSE_MARK));
 
 		button = gtk_button_new_with_mnemonic(_("In Sessi_on"));
 		gtk_container_add(GTK_CONTAINER(bbox), button);
-		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_find_dialog_response),
+		g_signal_connect(button, "clicked", G_CALLBACK(send_find_dialog_response),
 			GINT_TO_POINTER(GEANY_RESPONSE_FIND_IN_SESSION));
 
 		button = gtk_button_new_with_mnemonic(_("_In Document"));
 		gtk_container_add(GTK_CONTAINER(bbox), button);
-		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_find_dialog_response),
+		g_signal_connect(button, "clicked", G_CALLBACK(send_find_dialog_response),
 			GINT_TO_POINTER(GEANY_RESPONSE_FIND_IN_FILE));
 
 		/* close window checkbox */
@@ -406,24 +407,25 @@ static void send_replace_dialog_response(GtkButton *button, gpointer user_data)
 
 void search_show_replace_dialog(void)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	gchar *sel = NULL;
 
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
+	if (doc == NULL)
+		return;
 
-	sel = editor_get_default_selection(idx, prefs.search_use_current_word, NULL);
+	sel = editor_get_default_selection(doc->editor, search_prefs.use_current_word, NULL);
 
 	if (widgets.replace_dialog == NULL)
 	{
 		GtkWidget *label_find, *label_replace, *entry_find, *entry_replace,
 			*check_close, *button, *rbox, *fbox, *vbox, *exp, *bbox;
 		GtkSizeGroup *label_size;
-		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
+		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
 
 		load_monospace_style();
 
 		widgets.replace_dialog = gtk_dialog_new_with_buttons(_("Replace"),
-			GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_WINDOW(main_widgets.window), GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
 		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.replace_dialog));
 		gtk_box_set_spacing(GTK_BOX(vbox), 9);
@@ -460,14 +462,14 @@ void search_show_replace_dialog(void)
 		g_object_set_data_full(G_OBJECT(widgets.replace_dialog), "entry_replace",
 						gtk_widget_ref(entry_replace), (GDestroyNotify)gtk_widget_unref);
 
-		g_signal_connect((gpointer) gtk_bin_get_child(GTK_BIN(entry_find)),
+		g_signal_connect(gtk_bin_get_child(GTK_BIN(entry_find)),
 				"key-press-event", G_CALLBACK(on_widget_key_pressed_set_focus),
 				gtk_bin_get_child(GTK_BIN(entry_replace)));
-		g_signal_connect((gpointer) gtk_bin_get_child(GTK_BIN(entry_replace)), "activate",
+		g_signal_connect(gtk_bin_get_child(GTK_BIN(entry_replace)), "activate",
 				G_CALLBACK(on_replace_entry_activate), NULL);
-		g_signal_connect((gpointer) widgets.replace_dialog, "response",
+		g_signal_connect(widgets.replace_dialog, "response",
 				G_CALLBACK(on_replace_dialog_response), entry_replace);
-		g_signal_connect((gpointer) widgets.replace_dialog, "delete_event",
+		g_signal_connect(widgets.replace_dialog, "delete-event",
 				G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
 		fbox = gtk_hbox_new(FALSE, 6);
@@ -496,17 +498,17 @@ void search_show_replace_dialog(void)
 		gtk_tooltips_set_tip(tooltips, button,
 			_("Replace all matches found in the currently selected text"), NULL);
 		gtk_container_add(GTK_CONTAINER(bbox), button);
-		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_replace_dialog_response),
+		g_signal_connect(button, "clicked", G_CALLBACK(send_replace_dialog_response),
 			GINT_TO_POINTER(GEANY_RESPONSE_REPLACE_IN_SEL));
 
 		button = gtk_button_new_with_mnemonic(_("In Sessi_on"));
 		gtk_container_add(GTK_CONTAINER(bbox), button);
-		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_replace_dialog_response),
+		g_signal_connect(button, "clicked", G_CALLBACK(send_replace_dialog_response),
 			GINT_TO_POINTER(GEANY_RESPONSE_REPLACE_IN_SESSION));
 
 		button = gtk_button_new_with_mnemonic(_("_In Document"));
 		gtk_container_add(GTK_CONTAINER(bbox), button);
-		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_replace_dialog_response),
+		g_signal_connect(button, "clicked", G_CALLBACK(send_replace_dialog_response),
 			GINT_TO_POINTER(GEANY_RESPONSE_REPLACE_IN_FILE));
 
 		/* close window checkbox */
@@ -558,9 +560,9 @@ void search_show_find_in_files_dialog(const gchar *dir)
 	static GtkWidget *combo = NULL;
 	static GtkWidget *dir_combo;
 	GtkWidget *entry; /* the child GtkEntry of combo (or dir_combo) */
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	gchar *sel = NULL;
-	gchar *cur_dir;
+	gchar *cur_dir = NULL;
 
 	if (widgets.find_in_files_dialog == NULL)
 	{
@@ -568,12 +570,12 @@ void search_show_find_in_files_dialog(const gchar *dir)
 			*check_recursive, *check_extra, *entry_extra;
 		GtkWidget *dbox, *sbox, *cbox, *rbox, *rbtn, *hbox, *vbox;
 		GtkSizeGroup *size_group;
-		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
+		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
 
 		load_monospace_style();
 
 		widgets.find_in_files_dialog = gtk_dialog_new_with_buttons(
-			_("Find in Files"), GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
+			_("Find in Files"), GTK_WINDOW(main_widgets.window), GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
 		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.find_in_files_dialog));
 		gtk_box_set_spacing(GTK_BOX(vbox), 9);
@@ -694,7 +696,7 @@ void search_show_find_in_files_dialog(const gchar *dir)
 				_("Other options to pass to Grep"), NULL);
 
 		/* enable entry_extra when check_extra is checked */
-		g_signal_connect(G_OBJECT(check_extra), "toggled",
+		g_signal_connect(check_extra, "toggled",
 			G_CALLBACK(on_extra_options_toggled), entry_extra);
 
 		hbox = gtk_hbox_new(FALSE, 6);
@@ -702,39 +704,47 @@ void search_show_find_in_files_dialog(const gchar *dir)
 		gtk_box_pack_start(GTK_BOX(hbox), entry_extra, TRUE, TRUE, 0);
 		gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
-		g_signal_connect((gpointer) dir_combo, "key-press-event",
+		g_signal_connect(dir_combo, "key-press-event",
 				G_CALLBACK(on_widget_key_pressed_set_focus), combo);
-		g_signal_connect((gpointer) widgets.find_in_files_dialog, "response",
+		g_signal_connect(widgets.find_in_files_dialog, "response",
 				G_CALLBACK(on_find_in_files_dialog_response), combo);
-		g_signal_connect((gpointer) widgets.find_in_files_dialog, "delete_event",
+		g_signal_connect(widgets.find_in_files_dialog, "delete-event",
 				G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
 		gtk_widget_show_all(widgets.find_in_files_dialog);
-		sel = editor_get_default_selection(idx, prefs.search_use_current_word, NULL);
+		sel = editor_get_default_selection(doc->editor, search_prefs.use_current_word, NULL);
 	}
 
 	entry = GTK_BIN(combo)->child;
 	/* only set selection if the dialog is not already visible, or has just been created */
 	if (! sel && ! GTK_WIDGET_VISIBLE(widgets.find_in_files_dialog))
-		sel = editor_get_default_selection(idx, prefs.search_use_current_word, NULL);
+		sel = editor_get_default_selection(doc->editor, search_prefs.use_current_word, NULL);
 	if (sel)
 		gtk_entry_set_text(GTK_ENTRY(entry), sel);
 	g_free(sel);
 
 	entry = GTK_BIN(dir_combo)->child;
 	if (NZV(dir))
-		cur_dir = g_strdup(dir);
+		cur_dir = g_strdup(dir);	/* custom directory argument passed */
 	else
-		cur_dir = utils_get_current_file_dir_utf8();
+	{
+		gboolean entry_empty = ! NZV(gtk_entry_get_text(GTK_ENTRY(entry)));
+
+		if (search_prefs.use_current_file_dir || entry_empty)
+		{
+			cur_dir = utils_get_current_file_dir_utf8();
+
+			/* use default_open_path if no directory could be determined
+			 * (e.g. when no files are open) */
+			if (!cur_dir)
+				cur_dir = g_strdup(prefs.default_open_path);
+		}
+	}
 	if (cur_dir)
 	{
 		gtk_entry_set_text(GTK_ENTRY(entry), cur_dir);
 		g_free(cur_dir);
 	}
-	else
-	{	/* use default_open_path if no directory could be determined (e.g. when no files are open) */
-		gtk_entry_set_text(GTK_ENTRY(entry), prefs.default_open_path);
-	}
 
 	/* put the focus to the directory entry if it is empty */
 	if (utils_str_equal(gtk_entry_get_text(GTK_ENTRY(entry)), ""))
@@ -789,23 +799,23 @@ on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user
 }
 
 
-static gint search_mark(gint idx, const gchar *search_text, gint flags)
+static gint search_mark(GeanyDocument *doc, const gchar *search_text, gint flags)
 {
 	gint pos, line, count = 0;
 	struct TextToFind ttf;
 
-	g_return_val_if_fail(DOC_IDX_VALID(idx), 0);
+	g_return_val_if_fail(doc != NULL, 0);
 
 	ttf.chrg.cpMin = 0;
-	ttf.chrg.cpMax = sci_get_length(doc_list[idx].sci);
+	ttf.chrg.cpMax = sci_get_length(doc->editor->sci);
 	ttf.lpstrText = (gchar *)search_text;
 	while (1)
 	{
-		pos = sci_find_text(doc_list[idx].sci, flags, &ttf);
+		pos = sci_find_text(doc->editor->sci, flags, &ttf);
 		if (pos == -1) break;
 
-		line = sci_get_line_from_position(doc_list[idx].sci, pos);
-		sci_set_marker_at_line(doc_list[idx].sci, line, TRUE, 1);
+		line = sci_get_line_from_position(doc->editor->sci, pos);
+		sci_set_marker_at_line(doc->editor->sci, line, TRUE, 1);
 
 		ttf.chrg.cpMin = ttf.chrgText.cpMax + 1;
 		count++;
@@ -849,18 +859,19 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		gtk_widget_hide(widgets.find_dialog);
 	else
 	{
-		gint idx = document_get_cur_idx();
+		GeanyDocument *doc = document_get_current();
 		gboolean search_replace_escape;
 		gboolean check_close = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_close")));
 
+		if (doc == NULL)
+			return;
+
 		search_replace_escape = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_escape")));
 		search_data.backwards = FALSE;
 		search_data.search_bar = FALSE;
 
-		if (! DOC_IDX_VALID(idx)) return;
-
 		g_free(search_data.text);
 		search_data.text = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(user_data)))));
 		if (strlen(search_data.text) == 0 ||
@@ -879,10 +890,10 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		{
 			case GEANY_RESPONSE_FIND:
 			case GEANY_RESPONSE_FIND_PREVIOUS:
-				document_find_text(idx, search_data.text, search_data.flags,
+				document_find_text(doc, search_data.text, search_data.flags,
 					(response == GEANY_RESPONSE_FIND_PREVIOUS), TRUE, GTK_WIDGET(widgets.find_dialog));
 				check_close = FALSE;
-				if (prefs.suppress_search_dialogs)
+				if (search_prefs.suppress_dialogs)
 					check_close = TRUE;
 				break;
 
@@ -895,17 +906,18 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 				break;
 
 			case GEANY_RESPONSE_MARK:
-				if (DOC_IDX_VALID(idx))
-				{
-					gint count = search_mark(idx, search_data.text, search_data.flags);
+			{
+				gint count = search_mark(doc, search_data.text, search_data.flags);
 
-					if (count == 0)
-						ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), search_data.text);
-					else
-						ui_set_statusbar(FALSE, _("Found %d matches for \"%s\"."), count,
-							search_data.text);
-				}
-				break;
+				if (count == 0)
+					ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), search_data.text);
+				else
+					ui_set_statusbar(FALSE,
+						ngettext("Found %d match for \"%s\".",
+								 "Found %d matches for \"%s\".", count),
+						count, search_data.text);
+			}
+			break;
 		}
 		if (check_close)
 			gtk_widget_hide(widgets.find_dialog);
@@ -923,7 +935,7 @@ on_replace_entry_activate(GtkEntry *entry, gpointer user_data)
 static void
 on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	GtkWidget *entry_find = lookup_widget(GTK_WIDGET(widgets.replace_dialog), "entry_find");
 	GtkWidget *entry_replace = lookup_widget(GTK_WIDGET(widgets.replace_dialog), "entry_replace");
 	gint search_flags_re;
@@ -971,28 +983,28 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	{
 		case GEANY_RESPONSE_REPLACE_AND_FIND:
 		{
-			gint rep = document_replace_text(idx, find, replace, search_flags_re,
+			gint rep = document_replace_text(doc, find, replace, search_flags_re,
 				search_backwards_re);
 			if (rep != -1)
-				document_find_text(idx, find, search_flags_re, search_backwards_re,
+				document_find_text(doc, find, search_flags_re, search_backwards_re,
 					TRUE, NULL);
 			break;
 		}
 		case GEANY_RESPONSE_REPLACE:
 		{
-			document_replace_text(idx, find, replace, search_flags_re,
+			document_replace_text(doc, find, replace, search_flags_re,
 				search_backwards_re);
 			break;
 		}
 		case GEANY_RESPONSE_FIND:
 		{
-			document_find_text(idx, find, search_flags_re, search_backwards_re, TRUE,
+			document_find_text(doc, find, search_flags_re, search_backwards_re, TRUE,
 							   GTK_WIDGET(dialog));
 			break;
 		}
 		case GEANY_RESPONSE_REPLACE_IN_FILE:
 		{
-			if (! document_replace_all(idx, find, replace, search_flags_re,
+			if (! document_replace_all(doc, find, replace, search_flags_re,
 				search_replace_escape_re))
 			{
 				utils_beep();
@@ -1004,28 +1016,28 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			guint n, count = 0;
 
 			/* replace in all documents following notebook tab order */
-			for (n = 0; (gint) n < gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)); n++)
+			for (n = 0; (gint) n < gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)); n++)
 			{
-				gint ix = document_get_n_idx(n);
+				GeanyDocument *tmp_doc = document_get_from_page(n);
 
-				if (! doc_list[ix].is_valid) continue;
-
-				if (document_replace_all(ix, find, replace, search_flags_re,
+				if (document_replace_all(tmp_doc, find, replace, search_flags_re,
 					search_replace_escape_re)) count++;
 			}
 			if (count == 0)
 				utils_beep();
 
-			ui_set_statusbar(FALSE, _("Replaced text in %u files."), count);
+			ui_set_statusbar(FALSE,
+				ngettext("Replaced text in %u file.",
+						 "Replaced text in %u files.", count), count);
 			/* show which docs had replacements: */
 			gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
 
-			ui_save_buttons_toggle(doc_list[idx].changed);	/* update save all */
+			ui_save_buttons_toggle(doc->changed);	/* update save all */
 			break;
 		}
 		case GEANY_RESPONSE_REPLACE_IN_SEL:
 		{
-			document_replace_sel(idx, find, replace, search_flags_re, search_replace_escape_re);
+			document_replace_sel(doc, find, replace, search_flags_re, search_replace_escape_re);
 			break;
 		}
 	}
@@ -1161,11 +1173,11 @@ search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *op
 
 	if (! search_text || ! *search_text || ! dir) return TRUE;
 
-	command_grep = g_find_program_in_path(prefs.tools_grep_cmd);
+	command_grep = g_find_program_in_path(tool_prefs.grep_cmd);
 	if (command_grep == NULL)
 	{
 		ui_set_statusbar(TRUE, _("Cannot execute grep tool '%s';"
-			" check the path setting in Preferences."), prefs.tools_grep_cmd);
+			" check the path setting in Preferences."), tool_prefs.grep_cmd);
 		return FALSE;
 	}
 
@@ -1182,7 +1194,7 @@ search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *op
 	}
 	g_strfreev(opts_argv);
 
-	i++;	/* correct for prefs.tools_grep_cmd */
+	i++;	/* correct for tool_prefs.grep_cmd */
 	argv_prefix[i++] = g_strdup("--");
 	argv_prefix[i++] = g_strdup(search_text);
 
@@ -1230,9 +1242,9 @@ search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *op
 		g_child_watch_add(child_pid, search_close_pid, NULL);
 
 		str = g_strdup_printf(_("%s %s -- %s (in directory: %s)"),
-			prefs.tools_grep_cmd, opts, search_text, dir);
+			tool_prefs.grep_cmd, opts, search_text, dir);
 		utf8_str = utils_get_utf8_from_locale(str);
-		msgwin_msg_add(COLOR_BLUE, -1, -1, utf8_str);
+		msgwin_msg_add(COLOR_BLUE, -1, NULL, utf8_str);
 		utils_free_pointers(str, utf8_str, NULL);
 		ret = TRUE;
 	}
@@ -1291,7 +1303,7 @@ static gboolean search_read_io              (GIOChannel *source,
 
 		while (g_io_channel_read_line(source, &msg, NULL, NULL, NULL) && msg)
 		{
-			msgwin_msg_add(COLOR_BLACK, -1, -1, g_strstrip(msg));
+			msgwin_msg_add(COLOR_BLACK, -1, NULL, g_strstrip(msg));
 			g_free(msg);
 		}
 	}
@@ -1304,6 +1316,7 @@ static gboolean search_read_io              (GIOChannel *source,
 
 static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 {
+	/* TODO: port this also to Windows API */
 #ifdef G_OS_UNIX
 	const gchar *msg = _("Search failed.");
 	gint color = COLOR_DARK_RED;
@@ -1317,16 +1330,21 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 				gint count = gtk_tree_model_iter_n_children(
 					GTK_TREE_MODEL(msgwindow.store_msg), NULL) - 1;
 
-				msgwin_msg_add_fmt(COLOR_BLUE, -1, -1,
-					_("Search completed with %d matches."), count);
-				ui_set_statusbar(FALSE, _("Search completed with %d matches."), count);
+				msgwin_msg_add_fmt(COLOR_BLUE, -1, NULL,
+					ngettext("Search completed with %d match.",
+							 "Search completed with %d matches.", count),
+					count);
+				ui_set_statusbar(FALSE,
+						ngettext("Search completed with %d match.",
+								 "Search completed with %d matches.", count),
+						count);
 				break;
 			}
 			case 1:
 				msg = _("No matches found.");
 				color = COLOR_BLUE;
 			default:
-				msgwin_msg_add(color, -1, -1, msg);
+				msgwin_msg_add(color, -1, NULL, msg);
 				ui_set_statusbar(FALSE, "%s", msg);
 				break;
 		}
@@ -1338,24 +1356,24 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 }
 
 
-static gint find_document_usage(gint idx, const gchar *search_text, gint flags)
+static gint find_document_usage(GeanyDocument *doc, const gchar *search_text, gint flags)
 {
 	gchar *buffer, *short_file_name;
 	struct TextToFind ttf;
 	gint count = 0;
 
-	g_return_val_if_fail(DOC_IDX_VALID(idx), 0);
+	g_return_val_if_fail(doc != NULL, 0);
 
-	short_file_name = g_path_get_basename(DOC_FILENAME(idx));
+	short_file_name = g_path_get_basename(DOC_FILENAME(doc));
 
 	ttf.chrg.cpMin = 0;
-	ttf.chrg.cpMax = sci_get_length(doc_list[idx].sci);
+	ttf.chrg.cpMax = sci_get_length(doc->editor->sci);
 	ttf.lpstrText = (gchar *)search_text;
 	while (1)
 	{
 		gint pos, line, start, find_len;
 
-		pos = sci_find_text(doc_list[idx].sci, flags, &ttf);
+		pos = sci_find_text(doc->editor->sci, flags, &ttf);
 		if (pos == -1)
 			break;	/* no more matches */
 		find_len = ttf.chrgText.cpMax - ttf.chrgText.cpMin;
@@ -1363,9 +1381,9 @@ static gint find_document_usage(gint idx, const gchar *search_text, gint flags)
 			break;	/* Ignore regex ^ or $ */
 
 		count++;
-		line = sci_get_line_from_position(doc_list[idx].sci, pos);
-		buffer = sci_get_line(doc_list[idx].sci, line);
-		msgwin_msg_add_fmt(COLOR_BLACK, line + 1, idx,
+		line = sci_get_line_from_position(doc->editor->sci, pos);
+		buffer = sci_get_line(doc->editor->sci, line);
+		msgwin_msg_add_fmt(COLOR_BLACK, line + 1, doc,
 			"%s:%d : %s", short_file_name, line + 1, g_strstrip(buffer));
 		g_free(buffer);
 
@@ -1379,41 +1397,45 @@ static gint find_document_usage(gint idx, const gchar *search_text, gint flags)
 
 void search_find_usage(const gchar *search_text, gint flags, gboolean in_session)
 {
-	gint idx;
+	GeanyDocument *doc;
 	gboolean found = FALSE;
 
-	idx = document_get_cur_idx();
-	g_return_if_fail(DOC_IDX_VALID(idx));
+	doc = document_get_current();
+	g_return_if_fail(doc != NULL);
 
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_MESSAGE);
 	gtk_list_store_clear(msgwindow.store_msg);
 
 	if (! in_session)
 	{	/* use current document */
-		found = (find_document_usage(idx, search_text, flags) > 0);
+		found = (find_document_usage(doc, search_text, flags) > 0);
 	}
 	else
 	{
 		guint i;
-		for (i = 0; i < doc_array->len; i++)
+		for (i = 0; i < documents_array->len; i++)
 		{
-			if (doc_list[i].is_valid)
-				if (find_document_usage(i, search_text, flags) > 0) found = TRUE;
+			if (documents[i]->is_valid)
+				if (find_document_usage(documents[i], search_text, flags) > 0)
+					found = TRUE;
 		}
 	}
 
 	if (! found) /* no matches were found */
 	{
 		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), search_text);
-		msgwin_msg_add_fmt(COLOR_BLUE, -1, -1, _("No matches found for \"%s\"."), search_text);
+		msgwin_msg_add_fmt(COLOR_BLUE, -1, NULL, _("No matches found for \"%s\"."), search_text);
 	}
 	else
 	{
 		gint count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(msgwindow.store_msg), NULL);
 
-		ui_set_statusbar(FALSE, _("Found %d matches for \"%s\"."), count, search_text);
-		msgwin_msg_add_fmt(COLOR_BLUE, -1, -1, _("Found %d matches for \"%s\"."), count,
-			search_text);
+		ui_set_statusbar(FALSE, ngettext(
+			"Found %d match for \"%s\".", "Found %d matches for \"%s\".", count),
+			count, search_text);
+		msgwin_msg_add_fmt(COLOR_BLUE, -1, NULL, ngettext(
+			"Found %d match for \"%s\".", "Found %d matches for \"%s\".", count),
+			count, search_text);
 	}
 }
 
