@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.h 2287 2008-02-27 13:17:29Z eht16 $
+ * $Id: search.h 2724 2008-06-25 11:56:58Z ntrel $
  */
 
 
@@ -26,7 +26,7 @@
 #define GEANY_SEARCH_H 1
 
 /* the flags given in the search dialog for "find next", also used by the search bar */
-typedef struct
+typedef struct GeanySearchData
 {
 	gchar		*text;
 	gint		flags;
@@ -34,17 +34,22 @@ typedef struct
 	/* set to TRUE when text was set by a search bar callback to keep track of
 	 * search bar background colour */
 	gboolean	search_bar;
-} GeanySearchData;
+}
+GeanySearchData;
 
 extern GeanySearchData search_data;
 
 
-typedef struct
+typedef struct GeanySearchPrefs
 {
 	gchar		*fif_extra_options;
-} SearchPrefs;
+	gboolean	suppress_dialogs;
+	gboolean	use_current_word;
+	gboolean	use_current_file_dir;	/* find in files directory to use on showing dialog */
+}
+GeanySearchPrefs;
 
-extern SearchPrefs search_prefs;
+extern GeanySearchPrefs search_prefs;
 
 
 void search_init(void);
@@ -59,6 +64,6 @@ void search_show_find_in_files_dialog(const gchar *dir);
 
 void search_find_usage(const gchar *search_text, gint flags, gboolean in_session);
 
-void search_find_selection(gint idx, gboolean search_backwards);
+void search_find_selection(GeanyDocument *doc, gboolean search_backwards);
 
 #endif
