@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: symbols.c 2458 2008-04-08 15:21:54Z eht16 $
+ * $Id: symbols.c 3073 2008-10-12 17:52:22Z eht16 $
  */
 
 /*
@@ -28,6 +28,7 @@
  * matching filetype is first loaded.
  */
 
+#include "SciLexer.h"
 #include "geany.h"
 
 #include <ctype.h>
@@ -40,12 +41,15 @@
 #include "filetypes.h"
 #include "encodings.h"
 #include "document.h"
+#include "documentprivate.h"
 #include "support.h"
 #include "msgwindow.h"
 #include "treeviews.h"
 #include "main.h"
 #include "navqueue.h"
 #include "ui_utils.h"
+#include "editor.h"
+#include "sciwrappers.h"
 
 
 const guint TM_GLOBAL_TYPE_MASK =
@@ -61,6 +65,7 @@ typedef struct
 	const gchar	*tag_file;
 } TagFileInfo;
 
+/* Check before adding any more tags files, usually they should be downloaded separately. */
 enum	/* Geany tag files */
 {
 	GTF_C,
@@ -68,6 +73,7 @@ enum	/* Geany tag files */
 	GTF_PHP,
 	GTF_HTML_ENTITIES,
 	GTF_LATEX,
+	GTF_PYTHON,
 	GTF_MAX
 };
 
@@ -77,7 +83,8 @@ static TagFileInfo tag_file_info[GTF_MAX] =
 	{FALSE, "pascal.tags"},
 	{FALSE, "php.tags"},
 	{FALSE, "html_entities.tags"},
-	{FALSE, "latex.tags"}
+	{FALSE, "latex.tags"},
+	{FALSE, "python.tags"}
 };
 
 static gchar *user_tags_dir;
@@ -138,6 +145,7 @@ void symbols_global_tags_loaded(gint file_type_idx)
 		case GEANY_FILETYPES_PASCAL:tag_type = GTF_PASCAL; break;
 		case GEANY_FILETYPES_PHP:	tag_type = GTF_PHP; break;
 		case GEANY_FILETYPES_LATEX:	tag_type = GTF_LATEX; break;
+		case GEANY_FILETYPES_PYTHON:tag_type = GTF_PYTHON; break;
 		default:
 			return;
 	}
@@ -225,6 +233,7 @@ const gchar *symbols_get_context_separator(gint ft_id)
 	{
 		case GEANY_FILETYPES_C:	/* for C++ .h headers or C structs */
 		case GEANY_FILETYPES_CPP:
+		case GEANY_FILETYPES_GLSL:	/* for structs */
 		/*case GEANY_FILETYPES_RUBY:*/ /* not sure what to use atm*/
 		{
 			static gchar cc[] = "::";
@@ -264,7 +273,7 @@ GString *symbols_get_macro_list(void)
 			tm_tag_enum_t | tm_tag_variable_t | tm_tag_macro_t | tm_tag_macro_with_arg_t);
 		if (NULL != tags)
 		{
-			for (i = 0; ((i < tags->len) && (i < GEANY_MAX_AUTOCOMPLETE_WORDS)); ++i)
+			for (i = 0; ((i < tags->len) && (i < editor_prefs.autocompletion_max_entries)); ++i)
 			{
 				g_ptr_array_add(ftags, (gpointer) tags->pdata[i]);
 			}
@@ -395,12 +404,12 @@ static gint compare_symbol_lines(const GeanySymbol *a, const GeanySymbol *b)
 }
 
 
-static const GList *get_tag_list(gint idx, guint tag_types, gint sort_mode)
+static const GList *get_tag_list(GeanyDocument *doc, guint tag_types, gint sort_mode)
 {
 	static GList *tag_names = NULL;
 
-	if (DOC_IDX_VALID(idx) && doc_list[idx].tm_file &&
-		doc_list[idx].tm_file->tags_array)
+	if (doc != NULL && doc->tm_file &&
+		doc->tm_file->tags_array)
 	{
 		TMTag *tag;
 		guint i;
@@ -408,7 +417,7 @@ static const GList *get_tag_list(gint idx, guint tag_types, gint sort_mode)
 		gboolean doc_is_utf8 = FALSE;
 		gchar *utf8_name;
 		const gchar *cosep =
-			symbols_get_context_separator(FILETYPE_ID(doc_list[idx].file_type));
+			symbols_get_context_separator(FILETYPE_ID(doc->file_type));
 
 		if (tag_names)
 		{
@@ -424,20 +433,20 @@ static const GList *get_tag_list(gint idx, guint tag_types, gint sort_mode)
 
 		/* encodings_convert_to_utf8_from_charset() fails with charset "None", so skip conversion
 		 * for None at this point completely */
-		if (utils_str_equal(doc_list[idx].encoding, "UTF-8") ||
-			utils_str_equal(doc_list[idx].encoding, "None"))
+		if (utils_str_equal(doc->encoding, "UTF-8") ||
+			utils_str_equal(doc->encoding, "None"))
 			doc_is_utf8 = TRUE;
 
-		for (i = 0; i < (doc_list[idx].tm_file)->tags_array->len; ++i)
+		for (i = 0; i < (doc->tm_file)->tags_array->len; ++i)
 		{
-			tag = TM_TAG((doc_list[idx].tm_file)->tags_array->pdata[i]);
+			tag = TM_TAG((doc->tm_file)->tags_array->pdata[i]);
 			if (tag == NULL)
 				return NULL;
 
 			if (tag->type & tag_types)
 			{
 				if (! doc_is_utf8) utf8_name = encodings_convert_to_utf8_from_charset(tag->name,
-															(gsize)-1, doc_list[idx].encoding, TRUE);
+															(gsize)-1, doc->encoding, TRUE);
 				else utf8_name = tag->name;
 
 				if (utf8_name == NULL)
@@ -557,10 +566,10 @@ tag_list_add_groups(GtkTreeStore *tree_store, ...)
 }
 
 
-static void init_tag_list(gint idx)
+static void init_tag_list(GeanyDocument *doc)
 {
-	filetype_id ft_id = doc_list[idx].file_type->id;
-	GtkTreeStore *tag_store = doc_list[idx].tag_store;
+	filetype_id ft_id = doc->file_type->id;
+	GtkTreeStore *tag_store = doc->priv->tag_store;
 
 	init_tag_iters();
 
@@ -653,6 +662,16 @@ static void init_tag_list(gint idx)
 				NULL);
 			break;
 		}
+/*
+		case GEANY_FILETYPES_CSS:
+		{
+			tag_list_add_groups(tag_store,
+				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
+				&(tv_iters.tag_variable), _("ID Selectors"), "classviewer-var",
+				&(tv_iters.tag_struct), _("Type Selectors"), "classviewer-struct", NULL);
+			break;
+		}
+*/
 		case GEANY_FILETYPES_REST:
 		{
 			tag_list_add_groups(tag_store,
@@ -743,10 +762,11 @@ static void init_tag_list(gint idx)
 				NULL);
 			break;
 		}
+		case GEANY_FILETYPES_F77:
 		case GEANY_FILETYPES_FORTRAN:
 		{
 			tag_list_add_groups(tag_store,
-				&(tv_iters.tag_namespace), _("Module"), NULL,
+				&(tv_iters.tag_namespace), _("Module"), "classviewer-class",
 				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
 				&(tv_iters.tag_member), _("Subroutines"), "classviewer-method",
@@ -806,19 +826,19 @@ static void init_tag_list(gint idx)
 /* the following code surely can be improved, at the moment it collects some iters
  * for removal and after that the actual removal is done. I didn't find a way to find and remove
  * an empty row in one loop (next iter fails then) */
-static void hide_empty_rows(GtkTreeModel *model, GtkTreeStore *store)
+static void hide_empty_rows(GtkTreeStore *store)
 {
 	GtkTreeIter iter, *iters[MAX_SYMBOL_TYPES] = { NULL };
 	guint i = 0;
 
-	if (! gtk_tree_model_get_iter_first(model, &iter))
+	if (! gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
 		return; /* stop when first iter is invalid, i.e. no elements */
 
 	do /* first collect the iters we need to delete empty rows */
 	{
-		if (! gtk_tree_model_iter_has_child(model, &iter))
+		if (! gtk_tree_model_iter_has_child(GTK_TREE_MODEL(store), &iter))
 			iters[i++] = gtk_tree_iter_copy(&iter);
-	} while (gtk_tree_model_iter_next(model, &iter));
+	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter));
 
 	/* now actually delete the collected iters */
 	for (i = 0; i < MAX_SYMBOL_TYPES; i++)
@@ -831,35 +851,35 @@ static void hide_empty_rows(GtkTreeModel *model, GtkTreeStore *store)
 }
 
 
-gboolean symbols_recreate_tag_list(gint idx, gint sort_mode)
+gboolean symbols_recreate_tag_list(GeanyDocument *doc, gint sort_mode)
 {
 	GList *tmp;
 	const GList *tags;
 	GtkTreeIter iter;
-	GtkTreeModel *model;
 	static gint prev_sort_mode = SYMBOLS_SORT_BY_NAME;
-	filetype_id ft_id = FILETYPE_ID(doc_list[idx].file_type);
+	filetype_id ft_id;
 
-	g_return_val_if_fail(DOC_IDX_VALID(idx), FALSE);
+	g_return_val_if_fail(doc != NULL, FALSE);
+
+	ft_id = FILETYPE_ID(doc->file_type);
 
 	if (sort_mode == SYMBOLS_SORT_USE_PREVIOUS)
 		sort_mode = prev_sort_mode;
 	else
 		prev_sort_mode = sort_mode;
 
-	tags = get_tag_list(idx, tm_tag_max_t, sort_mode);
-	if (doc_list[idx].tm_file == NULL || tags == NULL)
+	tags = get_tag_list(doc, tm_tag_max_t, sort_mode);
+	if (doc->tm_file == NULL || tags == NULL)
 		return FALSE;
 
-	gtk_tree_store_clear(doc_list[idx].tag_store);
-	/* unref the store to speed up the filling(from TreeView Tutorial) */
-	model = gtk_tree_view_get_model(GTK_TREE_VIEW(doc_list[idx].tag_tree));
 	/* Make sure the model stays with us after the tree view unrefs it */
-	g_object_ref(model);
+	g_object_ref(GTK_TREE_MODEL(doc->priv->tag_store));
 	/* Detach model from view */
-	gtk_tree_view_set_model(GTK_TREE_VIEW(doc_list[idx].tag_tree), NULL);
+	gtk_tree_view_set_model(GTK_TREE_VIEW(doc->priv->tag_tree), NULL);
+	/* Clear all contents */
+	gtk_tree_store_clear(doc->priv->tag_store);
 
-	init_tag_list(idx);
+	init_tag_list(doc);
 	for (tmp = (GList*)tags; tmp; tmp = g_list_next(tmp))
 	{
 		gchar buf[100];
@@ -940,10 +960,10 @@ gboolean symbols_recreate_tag_list(gint idx, gint sort_mode)
 
 		if (parent)
 		{
-			gtk_tree_model_get(GTK_TREE_MODEL(doc_list[idx].tag_store), parent,
+			gtk_tree_model_get(GTK_TREE_MODEL(doc->priv->tag_store), parent,
 		 	                   SYMBOLS_COLUMN_ICON, &icon, -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &iter, parent);
-			gtk_tree_store_set(doc_list[idx].tag_store, &iter,
+			gtk_tree_store_append(doc->priv->tag_store, &iter, parent);
+			gtk_tree_store_set(doc->priv->tag_store, &iter,
 		 	                  SYMBOLS_COLUMN_ICON, icon,
                               SYMBOLS_COLUMN_NAME, buf,
                               SYMBOLS_COLUMN_LINE, symbol->line, -1);
@@ -952,22 +972,24 @@ gboolean symbols_recreate_tag_list(gint idx, gint sort_mode)
 				g_object_unref(icon);
 		}
 	}
-	hide_empty_rows(model, doc_list[idx].tag_store);
+	hide_empty_rows(doc->priv->tag_store);
 	/* Re-attach model to view */
-	gtk_tree_view_set_model(GTK_TREE_VIEW(doc_list[idx].tag_tree), model);
-	g_object_unref(model);
-	gtk_tree_view_expand_all(GTK_TREE_VIEW(doc_list[idx].tag_tree));
+	gtk_tree_view_set_model(GTK_TREE_VIEW(doc->priv->tag_tree),
+		GTK_TREE_MODEL(doc->priv->tag_store));
+	g_object_unref(GTK_TREE_MODEL(doc->priv->tag_store));
+	gtk_tree_view_expand_all(GTK_TREE_VIEW(doc->priv->tag_tree));
+
 	return TRUE;
 }
 
 
 /* Detects a global tags filetype from the *.lang.* language extension.
  * Returns NULL if there was no matching TM language. */
-static filetype *detect_global_tags_filetype(const gchar *utf8_filename)
+static GeanyFiletype *detect_global_tags_filetype(const gchar *utf8_filename)
 {
 	gchar *tags_ext;
 	gchar *shortname = g_strdup(utf8_filename);
-	filetype *ft = NULL;
+	GeanyFiletype *ft = NULL;
 
 	tags_ext = strstr(shortname, ".tags");
 	if (tags_ext)
@@ -1001,7 +1023,7 @@ int symbols_generate_global_tags(int argc, char **argv, gboolean want_preprocess
 		char *command;
 		const char *tags_file = argv[1];
 		char *utf8_fname;
-		filetype *ft;
+		GeanyFiletype *ft;
 
 		utf8_fname = utils_get_utf8_from_locale(tags_file);
 		ft = detect_global_tags_filetype(utf8_fname);
@@ -1012,6 +1034,10 @@ int symbols_generate_global_tags(int argc, char **argv, gboolean want_preprocess
 			g_printerr(_("Unknown filetype extension for \"%s\".\n"), tags_file);
 			return 1;
 		}
+		/* load ignore list for C/C++ parser */
+		if (ft->id == GEANY_FILETYPES_C || ft->id == GEANY_FILETYPES_CPP)
+			load_c_ignore_tags();
+
 		if (want_preprocess && (ft->id == GEANY_FILETYPES_C || ft->id == GEANY_FILETYPES_CPP))
 			command = g_strdup_printf("%s %s", pre_process, NVL(getenv("CFLAGS"), ""));
 		else
@@ -1023,6 +1049,7 @@ int symbols_generate_global_tags(int argc, char **argv, gboolean want_preprocess
 												 (const char **) (argv + 2),
 												 argc - 2, tags_file, ft->lang);
 		g_free(command);
+		symbols_finalize(); /* free c_tags_ignore data */
 		if (! status)
 		{
 			g_printerr(_("Failed to create tags file, perhaps because no tags "
@@ -1047,7 +1074,7 @@ void symbols_show_load_tags_dialog(void)
 	GtkWidget *dialog;
 	GtkFileFilter *filter;
 
-	dialog = gtk_file_chooser_dialog_new(_("Load Tags"), GTK_WINDOW(app->window),
+	dialog = gtk_file_chooser_dialog_new(_("Load Tags"), GTK_WINDOW(main_widgets.window),
 		GTK_FILE_CHOOSER_ACTION_OPEN,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
@@ -1067,7 +1094,7 @@ void symbols_show_load_tags_dialog(void)
 		{
 			gchar *fname = item->data;
 			gchar *utf8_fname;
-			filetype *ft;
+			GeanyFiletype *ft;
 
 			utf8_fname = utils_get_utf8_from_locale(fname);
 			ft = detect_global_tags_filetype(utf8_fname);
@@ -1097,11 +1124,11 @@ static GHashTable *get_tagfile_hash(const GSList *file_list)
 		GList *fnames;
 		gchar *fname = node->data;
 		gchar *utf8_fname = utils_get_utf8_from_locale(fname);
-		filetype *ft = detect_global_tags_filetype(utf8_fname);
+		GeanyFiletype *ft = detect_global_tags_filetype(utf8_fname);
 
 		g_free(utf8_fname);
 
-		if (FILETYPE_ID(ft) < GEANY_FILETYPES_ALL)
+		if (FILETYPE_ID(ft) < GEANY_FILETYPES_NONE)
 		{
 			fnames = g_hash_table_lookup(hash, ft);	/* may be NULL */
 			fnames = g_list_append(fnames, fname);
@@ -1137,13 +1164,13 @@ static GHashTable *init_user_tags(void)
 
 static void load_user_tags(filetype_id ft_id)
 {
-	static guchar tags_loaded[GEANY_FILETYPES_ALL] = {0};
+	static guchar tags_loaded[GEANY_FILETYPES_NONE] = {0};
 	static GHashTable *lang_hash = NULL;
 	GList *fnames;
 	const GList *node;
-	const filetype *ft = filetypes[ft_id];
+	const GeanyFiletype *ft = filetypes[ft_id];
 
-	g_return_if_fail(ft_id < GEANY_FILETYPES_ALL);
+	g_return_if_fail(ft_id < GEANY_FILETYPES_NONE);
 
 	if (tags_loaded[ft_id])
 		return;
@@ -1160,8 +1187,11 @@ static void load_user_tags(filetype_id ft_id)
 		gchar *fname;
 
 		fname = g_strconcat(user_tags_dir, G_DIR_SEPARATOR_S, node->data, NULL);
-		tm_workspace_load_global_tags(fname, tm_lang);
-		geany_debug("Loaded %s (%s).", fname, ft->name);
+		if (tm_workspace_load_global_tags(fname, tm_lang))
+		{
+			geany_debug("Loaded %s (%s), total tags: %u.", fname, ft->name,
+				tm_get_workspace()->global_tags->len);
+		}
 		g_free(fname);
 	}
 	g_list_foreach(fnames, (GFunc) g_free, NULL);
@@ -1175,14 +1205,14 @@ gboolean symbols_goto_tag(const gchar *name, gboolean definition)
 	const gint forward_types = tm_tag_prototype_t | tm_tag_externvar_t;
 	gint type;
 	TMTag *tmtag = NULL;
-	gint old_idx = document_get_cur_idx();
+	GeanyDocument *old_doc = document_get_current();
 
 	/* goto tag definition: all except prototypes / forward declarations / externs */
 	type = (definition) ? tm_tag_max_t - forward_types : forward_types;
 
 	/* first look in the current document */
-	if (doc_list[old_idx].tm_file)
-		tmtag = find_work_object_tag(doc_list[old_idx].tm_file, name, type);
+	if (old_doc != NULL && old_doc->tm_file)
+		tmtag = find_work_object_tag(old_doc->tm_file, name, type);
 
 	/* if not found, look in the workspace */
 	if (tmtag == NULL)
@@ -1190,15 +1220,16 @@ gboolean symbols_goto_tag(const gchar *name, gboolean definition)
 
 	if (tmtag != NULL)
 	{
-		gint new_idx = document_find_by_filename(
-			tmtag->atts.entry.file->work_object.file_name, TRUE);
+		GeanyDocument *new_doc = document_find_by_real_path(
+			tmtag->atts.entry.file->work_object.file_name);
+
 		/* not found in opened document, should open */
-		if (new_idx == -1)
+		if (new_doc == NULL)
 		{
-			new_idx = document_open_file(tmtag->atts.entry.file->work_object.file_name, FALSE, NULL, NULL);
+			new_doc = document_open_file(tmtag->atts.entry.file->work_object.file_name, FALSE, NULL, NULL);
 		}
 
-		if (navqueue_goto_line(old_idx, new_idx, tmtag->atts.entry.line))
+		if (navqueue_goto_line(old_doc, new_doc, tmtag->atts.entry.line))
 			return TRUE;
 	}
 	/* if we are here, there was no match and we are beeping ;-) */
@@ -1211,3 +1242,226 @@ gboolean symbols_goto_tag(const gchar *name, gboolean definition)
 }
 
 
+/* This could perhaps be improved to check for #if, class etc. */
+static gint get_function_fold_number(GeanyDocument *doc)
+{
+	/* for Java the functions are always one fold level above the class scope */
+	if (FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_JAVA)
+		return SC_FOLDLEVELBASE + 1;
+	else
+		return SC_FOLDLEVELBASE;
+}
+
+
+/* Should be used only with symbols_get_current_function. */
+static gboolean current_function_changed(GeanyDocument *doc, gint cur_line, gint fold_level)
+{
+	static gint old_line = -2;
+	static GeanyDocument *old_doc = NULL;
+	static gint old_fold_num = -1;
+	const gint fold_num = fold_level & SC_FOLDLEVELNUMBERMASK;
+	gboolean ret;
+
+	/* check if the cached line and file index have changed since last time: */
+	if (doc == NULL || doc != old_doc)
+		ret = TRUE;
+	else
+	if (cur_line == old_line)
+		ret = FALSE;
+	else
+	{
+		/* if the line has only changed by 1 */
+		if (abs(cur_line - old_line) == 1)
+		{
+			const gint fn_fold =
+				get_function_fold_number(doc);
+			/* It's the same function if the fold number hasn't changed, or both the new
+			 * and old fold numbers are above the function fold number. */
+			gboolean same =
+				fold_num == old_fold_num ||
+				(old_fold_num > fn_fold && fold_num > fn_fold);
+
+			ret = ! same;
+		}
+		else ret = TRUE;
+	}
+
+	/* record current line and file index for next time */
+	old_line = cur_line;
+	old_doc = doc;
+	old_fold_num = fold_num;
+	return ret;
+}
+
+
+/* Parse the function name up to 2 lines before tag_line.
+ * C++ like syntax should be parsed by parse_cpp_function_at_line, otherwise the return
+ * type or argument names can be confused with the function name. */
+static gchar *parse_function_at_line(ScintillaObject *sci, gint tag_line)
+{
+	gint start, end, max_pos;
+	gchar *cur_tag;
+	gint fn_style;
+
+	switch (sci_get_lexer(sci))
+	{
+		case SCLEX_RUBY:	fn_style = SCE_RB_DEFNAME; break;
+		case SCLEX_PYTHON:	fn_style = SCE_P_DEFNAME; break;
+		default: fn_style = SCE_C_IDENTIFIER;	/* several lexers use SCE_C_IDENTIFIER */
+	}
+	start = sci_get_position_from_line(sci, tag_line - 2);
+	max_pos = sci_get_position_from_line(sci, tag_line + 1);
+	while (sci_get_style_at(sci, start) != fn_style
+		&& start < max_pos) start++;
+
+	end = start;
+	while (sci_get_style_at(sci, end) == fn_style
+		&& end < max_pos) end++;
+
+	if (start == end) return NULL;
+	cur_tag = g_malloc(end - start + 1);
+	sci_get_text_range(sci, start, end, cur_tag);
+	return cur_tag;
+}
+
+
+/* Parse the function name */
+static gchar *parse_cpp_function_at_line(ScintillaObject *sci, gint tag_line)
+{
+	gint start, end, first_pos, max_pos;
+	gint tmp;
+	gchar c;
+	gchar *cur_tag;
+
+	first_pos = end = sci_get_position_from_line(sci, tag_line);
+	max_pos = sci_get_position_from_line(sci, tag_line + 1);
+	tmp = 0;
+	/* goto the begin of function body */
+	while (end < max_pos &&
+		(tmp = sci_get_char_at(sci, end)) != '{' &&
+		tmp != 0) end++;
+	if (tmp == 0) end --;
+
+	/* go back to the end of function identifier */
+	while (end > 0 && end > first_pos - 500 &&
+		(tmp = sci_get_char_at(sci, end)) != '(' &&
+		tmp != 0) end--;
+	end--;
+	if (end < 0) end = 0;
+
+	/* skip whitespaces between identifier and ( */
+	while (end > 0 && isspace(sci_get_char_at(sci, end))) end--;
+
+	start = end;
+	c = 0;
+	/* Use tmp to find SCE_C_IDENTIFIER or SCE_C_GLOBALCLASS chars */
+	while (start >= 0 && ((tmp = sci_get_style_at(sci, start)) == SCE_C_IDENTIFIER
+		 ||  tmp == SCE_C_GLOBALCLASS
+		 || (c = sci_get_char_at(sci, start)) == '~'
+		 ||  c == ':'))
+		start--;
+	if (start != 0 && start < end) start++;	/* correct for last non-matching char */
+
+	if (start == end) return NULL;
+	cur_tag = g_malloc(end - start + 2);
+	sci_get_text_range(sci, start, end + 1, cur_tag);
+	return cur_tag;
+}
+
+
+/* Sets *tagname to point at the current function or tag name.
+ * If doc is NULL, reset the cached current tag data to ensure it will be reparsed on the next
+ * call to this function.
+ * Returns: line number of the current tag, or -1 if unknown. */
+gint symbols_get_current_function(GeanyDocument *doc, const gchar **tagname)
+{
+	static gint tag_line = -1;
+	static gchar *cur_tag = NULL;
+	gint line;
+	gint fold_level;
+	TMWorkObject *tm_file;
+
+	if (doc == NULL)	/* reset current function */
+	{
+		current_function_changed(NULL, -1, -1);
+		g_free(cur_tag);
+		cur_tag = g_strdup(_("unknown"));
+		if (tagname != NULL)
+			*tagname = cur_tag;
+		tag_line = -1;
+		return tag_line;
+	}
+
+	line = sci_get_current_line(doc->editor->sci);
+	fold_level = sci_get_fold_level(doc->editor->sci, line);
+	/* check if the cached line and file index have changed since last time: */
+	if (! current_function_changed(doc, line, fold_level))
+	{
+		/* we can assume same current function as before */
+		*tagname = cur_tag;
+		return tag_line;
+	}
+	g_free(cur_tag); /* free the old tag, it will be replaced. */
+
+	/* if line is at base fold level, we're not in a function */
+	if ((fold_level & SC_FOLDLEVELNUMBERMASK) == SC_FOLDLEVELBASE)
+	{
+		cur_tag = g_strdup(_("unknown"));
+		*tagname = cur_tag;
+		tag_line = -1;
+		return tag_line;
+	}
+	tm_file = doc->tm_file;
+
+	/* if the document has no changes, get the previous function name from TM */
+	if(! doc->changed && tm_file != NULL && tm_file->tags_array != NULL)
+	{
+		const TMTag *tag = (const TMTag*) tm_get_current_function(tm_file->tags_array, line);
+
+		if (tag != NULL)
+		{
+			gchar *tmp;
+			tmp = tag->atts.entry.scope;
+			cur_tag = tmp ? g_strconcat(tmp, "::", tag->name, NULL) : g_strdup(tag->name);
+			*tagname = cur_tag;
+			tag_line = tag->atts.entry.line;
+			return tag_line;
+		}
+	}
+
+	/* parse the current function name here because TM line numbers may have changed,
+	 * and it would take too long to reparse the whole file. */
+	if (doc->file_type != NULL && doc->file_type->id != GEANY_FILETYPES_NONE)
+	{
+		const gint fn_fold = get_function_fold_number(doc);
+
+		tag_line = line;
+		do	/* find the top level fold point */
+		{
+			tag_line = sci_get_fold_parent(doc->editor->sci, tag_line);
+			fold_level = sci_get_fold_level(doc->editor->sci, tag_line);
+		} while (tag_line >= 0 &&
+			(fold_level & SC_FOLDLEVELNUMBERMASK) != fn_fold);
+
+		if (tag_line >= 0)
+		{
+			if (sci_get_lexer(doc->editor->sci) == SCLEX_CPP)
+				cur_tag = parse_cpp_function_at_line(doc->editor->sci, tag_line);
+			else
+				cur_tag = parse_function_at_line(doc->editor->sci, tag_line);
+
+			if (cur_tag != NULL)
+			{
+				*tagname = cur_tag;
+				return tag_line;
+			}
+		}
+	}
+
+	cur_tag = g_strdup(_("unknown"));
+	*tagname = cur_tag;
+	tag_line = -1;
+	return tag_line;
+}
+
+
