@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: symbols.h 2361 2008-03-18 16:09:11Z eht16 $
+ * $Id: symbols.h 2779 2008-07-15 14:29:41Z ntrel $
  */
 
 
@@ -41,7 +41,7 @@ GString *symbols_find_tags_as_string(GPtrArray *tags_array, guint tag_types, gin
 
 const gchar *symbols_get_context_separator(gint ft_id);
 
-const GList *symbols_get_tag_list(gint idx, guint tag_types);
+const GList *symbols_get_tag_list(GeanyDocument *doc, guint tag_types);
 
 GString *symbols_get_macro_list(void);
 
@@ -49,7 +49,7 @@ const gchar **symbols_get_html_entities(void);
 
 void symbols_finalize(void);
 
-gboolean symbols_recreate_tag_list(gint idx, gint sort_mode);
+gboolean symbols_recreate_tag_list(GeanyDocument *doc, gint sort_mode);
 
 gint symbols_generate_global_tags(gint argc, gchar **argv, gboolean want_preprocess);
 
@@ -57,4 +57,6 @@ void symbols_show_load_tags_dialog(void);
 
 gboolean symbols_goto_tag(const gchar *name, gboolean definition);
 
+gint symbols_get_current_function(GeanyDocument *doc, const gchar **tagname);
+
 #endif
