@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: templates.c 2287 2008-02-27 13:17:29Z eht16 $
+ * $Id: templates.c 3018 2008-09-28 16:32:49Z eht16 $
  */
 
 /*
@@ -32,7 +32,6 @@
 #include "geany.h"
 
 #include "templates.h"
-#include "prefs.h"
 #include "support.h"
 #include "utils.h"
 #include "document.h"
@@ -40,8 +39,10 @@
 #include "ui_utils.h"
 
 
-#define TEMPLATE_DATE_FORMAT "%Y-%m-%d"
-#define TEMPLATE_DATETIME_FORMAT "%d.%m.%Y %H:%M:%S %Z"
+GeanyTemplatePrefs template_prefs;
+
+static GtkWidget *new_with_template_menu = NULL;	/* File menu submenu */
+
 
 /* default templates, only for initial tempate file creation on first start of Geany */
 static const gchar templates_gpl_notice[] = "\
@@ -226,7 +227,7 @@ if __name__ == '__main__': main()\n\
 ";
 
 static gchar *templates[GEANY_MAX_TEMPLATES];
-static gchar *ft_templates[GEANY_MAX_FILE_TYPES] = {NULL};
+static gchar *ft_templates[GEANY_MAX_BUILT_IN_FILETYPES] = {NULL};
 
 
 /* some simple macros to reduce code size and make the code readable */
@@ -242,9 +243,6 @@ static gchar *ft_templates[GEANY_MAX_FILE_TYPES] = {NULL};
 	g_file_get_contents(fname, contents_ptr, NULL, NULL);
 
 
-/* prototype, because this function should never be used outside of templates.c */
-static gchar *templates_replace_all(gchar *source, const gchar *year, const gchar *date);
-
 
 static void init_general_templates(const gchar *year, const gchar *date)
 {
@@ -290,7 +288,7 @@ static void init_ft_templates(const gchar *year, const gchar *date)
 {
 	filetype_id ft_id;
 
-	for (ft_id = 0; ft_id < GEANY_MAX_FILE_TYPES; ft_id++)
+	for (ft_id = 0; ft_id < GEANY_MAX_BUILT_IN_FILETYPES; ft_id++)
 	{
 		gchar *ext = filetypes_get_conf_extension(ft_id);
 		gchar *shortname = g_strconcat("filetype.", ext, NULL);
@@ -298,7 +296,7 @@ static void init_ft_templates(const gchar *year, const gchar *date)
 
 		switch (ft_id)
 		{
-			case GEANY_FILETYPES_ALL:	TEMPLATES_CREATE_FILE(fname, templates_filetype_none); break;
+			case GEANY_FILETYPES_NONE:	TEMPLATES_CREATE_FILE(fname, templates_filetype_none); break;
 			case GEANY_FILETYPES_C:		TEMPLATES_CREATE_FILE(fname, templates_filetype_c); break;
 			case GEANY_FILETYPES_CPP:	TEMPLATES_CREATE_FILE(fname, templates_filetype_cpp); break;
 			case GEANY_FILETYPES_D:		TEMPLATES_CREATE_FILE(fname, templates_filetype_d); break;
@@ -324,7 +322,7 @@ static void
 on_new_with_template                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	filetype *ft = user_data;
+	GeanyFiletype *ft = user_data;
 	gchar *template = templates_get_template_new_file(ft);
 
 	document_new_file(NULL, ft, template);
@@ -335,44 +333,165 @@ on_new_with_template                   (GtkMenuItem     *menuitem,
 /* template items for the new file menu */
 static void create_new_menu_items(void)
 {
-	GtkWidget *template_menu = lookup_widget(app->window, "menu_new_with_template1_menu");
 	filetype_id ft_id;
 
-	for (ft_id = 0; ft_id < GEANY_MAX_FILE_TYPES; ft_id++)
+	for (ft_id = 0; ft_id < GEANY_MAX_BUILT_IN_FILETYPES; ft_id++)
 	{
 		GtkWidget *tmp_menu, *tmp_button;
-		filetype *ft = filetypes[ft_id];
+		GeanyFiletype *ft = filetypes[ft_id];
 		const gchar *label = ft->title;
 
 		if (ft_templates[ft_id] == NULL)
 			continue;
-		if (ft_id == GEANY_FILETYPES_ALL)
+		if (ft_id == GEANY_FILETYPES_NONE)
 			label = _("None");
 		tmp_menu = gtk_menu_item_new_with_label(label);
 		tmp_button = gtk_menu_item_new_with_label(label);
 		gtk_widget_show(tmp_menu);
 		gtk_widget_show(tmp_button);
-		gtk_container_add(GTK_CONTAINER(template_menu), tmp_menu);
+		gtk_container_add(GTK_CONTAINER(new_with_template_menu), tmp_menu);
 		gtk_container_add(GTK_CONTAINER(ui_widgets.new_file_menu), tmp_button);
-		g_signal_connect((gpointer) tmp_menu, "activate",
-			G_CALLBACK(on_new_with_template), (gpointer) ft);
-		g_signal_connect((gpointer) tmp_button, "activate",
-			G_CALLBACK(on_new_with_template), (gpointer) ft);
+		g_signal_connect(tmp_menu, "activate", G_CALLBACK(on_new_with_template), (gpointer) ft);
+		g_signal_connect(tmp_button, "activate", G_CALLBACK(on_new_with_template), (gpointer) ft);
+	}
+}
+
+
+static gchar *get_template_from_file(const gchar *locale_fname, GeanyFiletype *ft)
+{
+	GString template = {NULL, 0, 0};
+
+	g_file_get_contents(locale_fname, &template.str, &template.len, NULL);
+
+	if (template.len > 0)
+	{
+		gchar *file_header;
+
+		template.allocated_len = template.len + 1;
+
+		file_header = templates_get_template_fileheader(FILETYPE_ID(ft), NULL);
+		utils_string_replace_all(&template, "{fileheader}", file_header);
+		g_free(file_header);
+	}
+	return template.str;
+}
+
+
+static void
+on_new_with_file_template(GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_data)
+{
+	gchar *fname = ui_menu_item_get_text(menuitem);
+	GeanyFiletype *ft;
+	gchar *template;
+
+	ft = filetypes_detect_from_extension(fname);
+	setptr(fname, utils_get_locale_from_utf8(fname));
+	/* fname is just the basename from the menu item, so prepend the custom files path */
+	setptr(fname, g_build_path(G_DIR_SEPARATOR_S, app->configdir, GEANY_TEMPLATES_SUBDIR,
+		"files", fname, NULL));
+	template = get_template_from_file(fname, ft);
+	g_free(fname);
+
+	document_new_file(NULL, ft, template);
+	g_free(template);
+}
+
+
+static void add_file_item(gpointer data, gpointer user_data)
+{
+	GtkWidget *tmp_menu, *tmp_button;
+	gchar *label;
+
+	g_return_if_fail(data);
+
+	label = utils_get_utf8_from_locale(data);
+
+	tmp_menu = gtk_menu_item_new_with_label(label);
+	tmp_button = gtk_menu_item_new_with_label(label);
+
+	g_free(label);
+
+	gtk_widget_show(tmp_menu);
+	gtk_widget_show(tmp_button);
+	gtk_container_add(GTK_CONTAINER(new_with_template_menu), tmp_menu);
+	gtk_container_add(GTK_CONTAINER(ui_widgets.new_file_menu), tmp_button);
+	g_signal_connect(tmp_menu, "activate", G_CALLBACK(on_new_with_file_template), NULL);
+	g_signal_connect(tmp_button, "activate", G_CALLBACK(on_new_with_file_template), NULL);
+}
+
+
+static gint compare_filenames_by_filetype(gconstpointer a, gconstpointer b)
+{
+	GeanyFiletype *ft_a = filetypes_detect_from_extension(a);
+	GeanyFiletype *ft_b = filetypes_detect_from_extension(b);
+
+	/* sort by filetype name first */
+	if (ft_a != ft_b)
+	{
+		/* None filetypes should come first */
+		if (ft_a->id == GEANY_FILETYPES_NONE)
+			return -1;
+		if (ft_b->id == GEANY_FILETYPES_NONE)
+			return 1;
+
+		return g_strcasecmp(ft_a->name, ft_b->name);
+	}
+	return g_strcasecmp(a, b);
+}
+
+
+static gboolean add_custom_template_items(void)
+{
+	gchar *path = g_build_path(G_DIR_SEPARATOR_S, app->configdir, GEANY_TEMPLATES_SUBDIR,
+		"files", NULL);
+	GSList *list = utils_get_file_list(path, NULL, NULL);
+
+	if (list == NULL)
+	{
+		utils_mkdir(path, FALSE);
+		return FALSE;
+	}
+	list = g_slist_sort(list, compare_filenames_by_filetype);
+	g_slist_foreach(list, add_file_item, NULL);
+	g_slist_foreach(list, (GFunc) g_free, NULL);
+	g_slist_free(list);
+	g_free(path);
+	return TRUE;
+}
+
+
+static void create_file_template_menus(void)
+{
+	GtkWidget *sep1, *sep2;
+
+	new_with_template_menu = lookup_widget(main_widgets.window, "menu_new_with_template1_menu");
+	create_new_menu_items();
+
+	sep1 = gtk_separator_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(new_with_template_menu), sep1);
+	sep2 = gtk_separator_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(ui_widgets.new_file_menu), sep2);
+
+	if (add_custom_template_items())
+	{
+		gtk_widget_show(sep1);
+		gtk_widget_show(sep2);
 	}
 }
 
 
 void templates_init(void)
 {
-	gchar *year = utils_get_date_time("%Y", NULL);
-	gchar *date = utils_get_date_time(TEMPLATE_DATE_FORMAT, NULL);
+	gchar *year = utils_get_date_time(template_prefs.year_format, NULL);
+	gchar *date = utils_get_date_time(template_prefs.date_format, NULL);
 
 	init_general_templates(year, date);
 	init_ft_templates(year, date);
-	create_new_menu_items();
 
 	g_free(date);
 	g_free(year);
+
+	create_file_template_menus();
 }
 
 
@@ -391,8 +510,8 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 	gchar **lines;
 	guint i;
 
-	/** TODO the following switch could be replaced by some intelligent code which reads
-	 ** frame_start, frame_end and line_prefix from the filetype definition files */
+	/* TODO the following switch could be replaced by some intelligent code which reads
+	 * frame_start, frame_end and line_prefix from the filetype definition files */
 	switch (filetype_idx)
 	{
 		case GEANY_FILETYPES_HTML:
@@ -406,6 +525,7 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 		}
 
 		case GEANY_FILETYPES_PYTHON:
+		case GEANY_FILETYPES_R:
 		case GEANY_FILETYPES_RUBY:
 		case GEANY_FILETYPES_SH:
 		case GEANY_FILETYPES_MAKE:
@@ -414,6 +534,7 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 		case GEANY_FILETYPES_TCL:
 		case GEANY_FILETYPES_OMS:
 		case GEANY_FILETYPES_CONF:
+		case GEANY_FILETYPES_PO:
 		{
 			line_prefix = "#";
 			break;
@@ -440,6 +561,7 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 			break;
 		}
 
+		case GEANY_FILETYPES_F77:
 		case GEANY_FILETYPES_FORTRAN:
 		{
 			line_prefix = "c";
@@ -466,14 +588,6 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 			break;
 		}
 
-		case GEANY_FILETYPES_PHP:
-		{
-			frame_start = "/*\n";
-			frame_end = " */\n";
-			line_prefix = " *";
-			break;
-		}
-
 		case GEANY_FILETYPES_CAML:
 		{
 			frame_start = "(*\n";
@@ -482,7 +596,7 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 			break;
 		}
 
-		case GEANY_FILETYPES_ALL:
+		case GEANY_FILETYPES_NONE:
 		{
 			line_prefix = "";
 			break;
@@ -537,15 +651,15 @@ gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname)
 	gchar *template = g_strdup(templates[GEANY_TEMPLATE_FILEHEADER]);
 	gchar *shortname;
 	gchar *result;
-	gchar *date = utils_get_date_time(TEMPLATE_DATETIME_FORMAT, NULL);
+	gchar *date = utils_get_date_time(template_prefs.datetime_format, NULL);
 	filetype_id ft_id = filetype_idx;
-	filetype *ft = filetypes[ft_id];
+	GeanyFiletype *ft = filetypes[ft_id];
 
-	filetypes_load_config(ft_id);	/* load any user extension setting */
+	filetypes_load_config(ft_id, FALSE);	/* load any user extension setting */
 
 	if (fname == NULL)
 	{
-		if (ft_id == GEANY_FILETYPES_ALL)
+		if (ft_id == GEANY_FILETYPES_NONE)
 			shortname = g_strdup(GEANY_STRING_UNTITLED);
 		else
 			shortname = g_strconcat(GEANY_STRING_UNTITLED, ".", ft->extension, NULL);
@@ -567,7 +681,7 @@ gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname)
 }
 
 
-static gchar *get_file_template(filetype *ft)
+static gchar *get_file_template(GeanyFiletype *ft)
 {
 	filetype_id ft_id = FILETYPE_ID(ft);
 
@@ -575,12 +689,12 @@ static gchar *get_file_template(filetype *ft)
 }
 
 
-gchar *templates_get_template_new_file(struct filetype *ft)
+gchar *templates_get_template_new_file(GeanyFiletype *ft)
 {
 	gchar *ft_template = NULL;
 	gchar *file_header = NULL;
 
-	if (FILETYPE_ID(ft) == GEANY_FILETYPES_ALL)
+	if (FILETYPE_ID(ft) == GEANY_FILETYPES_NONE)
 		return get_file_template(ft);
 
 	file_header = templates_get_template_fileheader(ft->id, NULL);	/* file template only used for new files */
@@ -600,8 +714,8 @@ gchar *templates_get_template_generic(gint template)
 gchar *templates_get_template_function(gint filetype_idx, const gchar *func_name)
 {
 	gchar *template = g_strdup(templates[GEANY_TEMPLATE_FUNCTION]);
-	gchar *date = utils_get_date_time(TEMPLATE_DATE_FORMAT, NULL);
-	gchar *datetime = utils_get_date_time(TEMPLATE_DATETIME_FORMAT, NULL);
+	gchar *date = utils_get_date_time(template_prefs.date_format, NULL);
+	gchar *datetime = utils_get_date_time(template_prefs.datetime_format, NULL);
 	gchar *result;
 
 	template = utils_str_replace(template, "{date}", date);
@@ -619,7 +733,7 @@ gchar *templates_get_template_function(gint filetype_idx, const gchar *func_name
 
 gchar *templates_get_template_changelog(void)
 {
-	gchar *date = utils_get_date_time(TEMPLATE_DATETIME_FORMAT, NULL);
+	gchar *date = utils_get_date_time(template_prefs.datetime_format, NULL);
 	gchar *result = g_strdup(templates[GEANY_TEMPLATE_CHANGELOG]);
 	result = utils_str_replace(result, "{date}", date);
 
@@ -631,22 +745,39 @@ gchar *templates_get_template_changelog(void)
 void templates_free_templates(void)
 {
 	gint i;
+	GList *children, *item;
+
 	for (i = 0; i < GEANY_MAX_TEMPLATES; i++)
 	{
 		g_free(templates[i]);
 	}
+	for (i = 0; i < GEANY_MAX_BUILT_IN_FILETYPES; i++)
+	{
+		g_free(ft_templates[i]);
+	}
+	/* destroy "New with template" sub menu items (in case we want to reload the templates) */
+	children = gtk_container_get_children(GTK_CONTAINER(ui_widgets.new_file_menu));
+	for (item = children; item != NULL; item = g_list_next(item))
+	{
+		gtk_widget_destroy(GTK_WIDGET(item->data));
+	}
+	children = gtk_container_get_children(GTK_CONTAINER(new_with_template_menu));
+	for (item = children; item != NULL; item = g_list_next(item))
+	{
+		gtk_widget_destroy(GTK_WIDGET(item->data));
+	}
 }
 
 
-static gchar *templates_replace_all(gchar *text, const gchar *year, const gchar *date)
+gchar *templates_replace_all(gchar *text, const gchar *year, const gchar *date)
 {
 	text = utils_str_replace(text, "{year}", year);
 	text = utils_str_replace(text, "{date}", date);
-	text = utils_str_replace(text, "{version}", prefs.template_version);
-	text = utils_str_replace(text, "{initial}", prefs.template_initial);
-	text = utils_str_replace(text, "{developer}", prefs.template_developer);
-	text = utils_str_replace(text, "{mail}", prefs.template_mail);
-	text = utils_str_replace(text, "{company}", prefs.template_company);
+	text = utils_str_replace(text, "{version}", template_prefs.version);
+	text = utils_str_replace(text, "{initial}", template_prefs.initials);
+	text = utils_str_replace(text, "{developer}", template_prefs.developer);
+	text = utils_str_replace(text, "{mail}", template_prefs.mail);
+	text = utils_str_replace(text, "{company}", template_prefs.company);
 	text = utils_str_replace(text, "{untitled}", GEANY_STRING_UNTITLED);
 	text = utils_str_replace(text, "{geanyversion}", "Geany " VERSION);
 
