@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: tools.c 2348 2008-03-14 18:55:30Z eht16 $
+ * $Id: tools.c 3047 2008-10-06 18:37:10Z eht16 $
  */
 
 /*
@@ -42,6 +42,7 @@
 #include "tools.h"
 #include "support.h"
 #include "document.h"
+#include "editor.h"
 #include "sciwrappers.h"
 #include "utils.h"
 #include "ui_utils.h"
@@ -163,7 +164,7 @@ static gboolean cc_iofunc_err(GIOChannel *ioc, GIOCondition cond, gpointer data)
 
 static gboolean cc_replace_sel_cb(gpointer user_data)
 {
-	gint idx = GPOINTER_TO_INT(user_data);
+	GeanyDocument *doc = user_data;
 
 	if (! cc_reading_finished)
 	{	/* keep this function in the main loop until cc_iofunc_err() has finished */
@@ -172,7 +173,7 @@ static gboolean cc_replace_sel_cb(gpointer user_data)
 
 	if (! cc_error_occurred && cc_buffer != NULL)
 	{	/* Command completed successfully */
-		sci_replace_sel(doc_list[idx].sci, cc_buffer->str);
+		sci_replace_sel(doc->editor->sci, cc_buffer->str);
 		g_string_free(cc_buffer, TRUE);
 		cc_buffer = NULL;
 	}
@@ -226,7 +227,7 @@ static void cc_exit_cb(GPid child_pid, gint status, gpointer user_data)
 /* Executes command (which should include all necessary command line args) and passes the current
  * selection through the standard input of command. The whole output of command replaces the
  * current selection. */
-void tools_execute_custom_command(gint idx, const gchar *command)
+void tools_execute_custom_command(GeanyDocument *doc, const gchar *command)
 {
 	GError *error = NULL;
 	GPid pid;
@@ -235,9 +236,9 @@ void tools_execute_custom_command(gint idx, const gchar *command)
 	gint stdout_fd;
 	gint stderr_fd;
 
-	g_return_if_fail(DOC_IDX_VALID(idx) && command != NULL);
+	g_return_if_fail(doc != NULL && command != NULL);
 
-	if (! sci_can_copy(doc_list[idx].sci))
+	if (! sci_has_selection(doc->editor->sci))
 		return;
 
 	argv = g_strsplit(command, " ", -1);
@@ -252,7 +253,7 @@ void tools_execute_custom_command(gint idx, const gchar *command)
 		gint len, remaining, wrote;
 
 		if (pid > 0)
-			g_child_watch_add(pid, (GChildWatchFunc) cc_exit_cb, GINT_TO_POINTER(idx));
+			g_child_watch_add(pid, (GChildWatchFunc) cc_exit_cb, doc);
 
 		/* use GIOChannel to monitor stdout */
 		utils_set_up_io_channel(stdout_fd, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
@@ -262,9 +263,9 @@ void tools_execute_custom_command(gint idx, const gchar *command)
 				FALSE, cc_iofunc_err, (gpointer)command);
 
 		/* get selection */
-		len = sci_get_selected_text_length(doc_list[idx].sci);
+		len = sci_get_selected_text_length(doc->editor->sci);
 		sel = g_malloc0(len + 1);
-		sci_get_selected_text(doc_list[idx].sci, sel);
+		sci_get_selected_text(doc->editor->sci, sel);
 
 		/* write data to the command */
 		remaining = len - 1;
@@ -274,7 +275,7 @@ void tools_execute_custom_command(gint idx, const gchar *command)
 			if (wrote < 0)
 			{
 				g_warning("%s: %s: %s\n", __func__, "Failed sending data to command",
-										strerror(errno));
+										g_strerror(errno));
 				break;
 			}
 			remaining -= wrote;
@@ -299,7 +300,7 @@ static void cc_show_dialog_custom_commands(void)
 	guint i;
 	struct cc_dialog cc;
 
-	dialog = gtk_dialog_new_with_buttons(_("Set Custom Commands"), GTK_WINDOW(app->window),
+	dialog = gtk_dialog_new_with_buttons(_("Set Custom Commands"), GTK_WINDOW(main_widgets.window),
 						GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 						GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
@@ -331,8 +332,7 @@ static void cc_show_dialog_custom_commands(void)
 	}
 
 	button = gtk_button_new_from_stock("gtk-add");
-	g_signal_connect((gpointer) button, "clicked",
-			G_CALLBACK(cc_on_custom_commands_dlg_add_clicked), &cc);
+	g_signal_connect(button, "clicked", G_CALLBACK(cc_on_custom_commands_dlg_add_clicked), &cc);
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
 
 	gtk_widget_show_all(vbox);
@@ -395,14 +395,15 @@ static void cc_show_dialog_custom_commands(void)
 /* enable or disable all custom command menu items when the sub menu is opened */
 static void cc_on_custom_command_menu_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	gint i, len;
 	gboolean enable;
 	GList *children;
 
-	if (! DOC_IDX_VALID(idx)) return;
+	if (doc == NULL)
+		return;
 
-	enable = sci_can_copy(doc_list[idx].sci) && (ui_prefs.custom_commands != NULL);
+	enable = sci_has_selection(doc->editor->sci) && (ui_prefs.custom_commands != NULL);
 
 	children = gtk_container_get_children(GTK_CONTAINER(user_data));
 	len = g_list_length(children);
@@ -421,10 +422,11 @@ static void cc_on_custom_command_menu_activate(GtkMenuItem *menuitem, gpointer u
 
 static void cc_on_custom_command_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	gint command_idx;
 
-	if (! DOC_IDX_VALID(idx)) return;
+	if (doc == NULL)
+		return;
 
 	command_idx = GPOINTER_TO_INT(user_data);
 
@@ -437,7 +439,7 @@ static void cc_on_custom_command_activate(GtkMenuItem *menuitem, gpointer user_d
 
 	/* send it through the command and when the command returned the output the current selection
 	 * will be replaced */
-	tools_execute_custom_command(idx, ui_prefs.custom_commands[command_idx]);
+	tools_execute_custom_command(doc, ui_prefs.custom_commands[command_idx]);
 }
 
 
@@ -445,7 +447,7 @@ static void cc_insert_custom_command_items(GtkMenu *me, GtkMenu *mp, gchar *labe
 {
 	GtkWidget *item;
 	gint key_idx = -1;
-	KeyBinding *kb = NULL;
+	GeanyKeyBinding *kb = NULL;
 
 	switch (idx)
 	{
@@ -463,7 +465,7 @@ static void cc_insert_custom_command_items(GtkMenu *me, GtkMenu *mp, gchar *labe
 			kb->key, kb->mods, GTK_ACCEL_VISIBLE);
 	gtk_container_add(GTK_CONTAINER(me), item);
 	gtk_widget_show(item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(cc_on_custom_command_activate),
+	g_signal_connect(item, "activate", G_CALLBACK(cc_on_custom_command_activate),
 		GINT_TO_POINTER(idx));
 
 	item = gtk_menu_item_new_with_label(label);
@@ -472,15 +474,15 @@ static void cc_insert_custom_command_items(GtkMenu *me, GtkMenu *mp, gchar *labe
 			kb->key, kb->mods, GTK_ACCEL_VISIBLE);
 	gtk_container_add(GTK_CONTAINER(mp), item);
 	gtk_widget_show(item);
-	g_signal_connect((gpointer) item, "activate", G_CALLBACK(cc_on_custom_command_activate),
+	g_signal_connect(item, "activate", G_CALLBACK(cc_on_custom_command_activate),
 		GINT_TO_POINTER(idx));
 }
 
 
 void tools_create_insert_custom_command_menu_items(void)
 {
-	GtkMenu *menu_edit = GTK_MENU(lookup_widget(app->window, "send_selection_to2_menu"));
-	GtkMenu *menu_popup = GTK_MENU(lookup_widget(app->popup_menu, "send_selection_to1_menu"));
+	GtkMenu *menu_edit = GTK_MENU(lookup_widget(main_widgets.window, "send_selection_to2_menu"));
+	GtkMenu *menu_popup = GTK_MENU(lookup_widget(main_widgets.editor_menu, "send_selection_to1_menu"));
 	GtkWidget *item;
 	GList *me_children;
 	GList *mp_children;
@@ -536,9 +538,9 @@ void tools_create_insert_custom_command_menu_items(void)
 
 	if (! signal_set)
 	{
-		g_signal_connect((gpointer) lookup_widget(app->popup_menu, "send_selection_to1"),
+		g_signal_connect(lookup_widget(main_widgets.editor_menu, "send_selection_to1"),
 					"activate", G_CALLBACK(cc_on_custom_command_menu_activate), menu_popup);
-		g_signal_connect((gpointer) lookup_widget(app->window, "send_selection_to2"),
+		g_signal_connect(lookup_widget(main_widgets.window, "send_selection_to2"),
 					"activate", G_CALLBACK(cc_on_custom_command_menu_activate), menu_edit);
 		signal_set = TRUE;
 	}
@@ -599,29 +601,30 @@ static void word_count(gchar *text, guint *chars, guint *lines, guint *words)
 void tools_word_count(void)
 {
 	GtkWidget *dialog, *label, *vbox, *table;
-	gint idx;
+	GeanyDocument *doc;
 	guint chars = 0, lines = 0, words = 0;
 	gchar *text, *range;
 
-	idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
+	doc = document_get_current();
+	if (doc == NULL)
+		return;
 
-	dialog = gtk_dialog_new_with_buttons(_("Word Count"), GTK_WINDOW(app->window),
+	dialog = gtk_dialog_new_with_buttons(_("Word Count"), GTK_WINDOW(main_widgets.window),
 										 GTK_DIALOG_DESTROY_WITH_PARENT,
 										 GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
 	gtk_widget_set_name(dialog, "GeanyDialog");
 
-	if (sci_can_copy(doc_list[idx].sci))
+	if (sci_has_selection(doc->editor->sci))
 	{
-		text = g_malloc0(sci_get_selected_text_length(doc_list[idx].sci) + 1);
-		sci_get_selected_text(doc_list[idx].sci, text);
+		text = g_malloc0(sci_get_selected_text_length(doc->editor->sci) + 1);
+		sci_get_selected_text(doc->editor->sci, text);
 		range = _("selection");
 	}
 	else
 	{
-		text = g_malloc(sci_get_length(doc_list[idx].sci) + 1);
-		sci_get_text(doc_list[idx].sci, sci_get_length(doc_list[idx].sci) + 1 , text);
+		text = g_malloc(sci_get_length(doc->editor->sci) + 1);
+		sci_get_text(doc->editor->sci, sci_get_length(doc->editor->sci) + 1 , text);
 		range = _("whole document");
 	}
 	word_count(text, &chars, &lines, &words);
@@ -711,17 +714,18 @@ on_color_ok_button_clicked             (GtkButton       *button,
                                         gpointer         user_data)
 {
 	GdkColor color;
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 	gchar *hex;
 
 	gtk_widget_hide(ui_widgets.open_colorsel);
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
+	if (doc == NULL)
+		return;
 
 	gtk_color_selection_get_current_color(
 			GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(ui_widgets.open_colorsel)->colorsel), &color);
 
 	hex = utils_get_hex_from_color(&color);
-	document_insert_colour(idx, hex);
+	editor_insert_color(doc->editor, hex);
 	g_free(hex);
 }
 #endif
@@ -738,7 +742,7 @@ void tools_color_chooser(gchar *color)
 	{
 		ui_widgets.open_colorsel = gtk_color_selection_dialog_new(_("Color Chooser"));
 		gtk_widget_set_name(ui_widgets.open_colorsel, "GeanyDialog");
-		gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.open_colorsel), GTK_WINDOW(app->window));
+		gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.open_colorsel), GTK_WINDOW(main_widgets.window));
 		gtk_color_selection_set_has_palette(
 			GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(ui_widgets.open_colorsel)->colorsel), TRUE);
 
@@ -746,7 +750,7 @@ void tools_color_chooser(gchar *color)
 						G_CALLBACK(on_color_cancel_button_clicked), NULL);
 		g_signal_connect(GTK_COLOR_SELECTION_DIALOG(ui_widgets.open_colorsel)->ok_button, "clicked",
 						G_CALLBACK(on_color_ok_button_clicked), NULL);
-		g_signal_connect(ui_widgets.open_colorsel, "delete_event",
+		g_signal_connect(ui_widgets.open_colorsel, "delete-event",
 						G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 	}
 	/* if color is non-NULL set it in the dialog as preselected color */
