@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tools.h 2267 2008-02-20 11:24:23Z eht16 $
+ * $Id: tools.h 2705 2008-06-18 17:03:08Z ntrel $
  */
 
 
@@ -27,7 +27,7 @@
 
 void tools_create_insert_custom_command_menu_items(void);
 
-void tools_execute_custom_command(gint idx, const gchar *command);
+void tools_execute_custom_command(GeanyDocument *doc, const gchar *command);
 
 
 void tools_word_count(void);
