@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: treeviews.h 2267 2008-02-20 11:24:23Z eht16 $
+ * $Id: treeviews.h 2705 2008-06-18 17:03:08Z ntrel $
  */
 
 
@@ -47,16 +47,16 @@ enum
 
 void treeviews_init(void);
 
-void treeviews_update_tag_list(gint idx, gboolean update);
+void treeviews_update_tag_list(GeanyDocument *doc, gboolean update);
 
-void treeviews_openfiles_add(gint idx);
+void treeviews_openfiles_add(GeanyDocument *doc);
 
-void treeviews_openfiles_update(gint idx);
+void treeviews_openfiles_update(GeanyDocument *doc);
 
 void treeviews_openfiles_update_all(void);
 
-void treeviews_select_openfiles_item(gint idx);
+void treeviews_select_openfiles_item(GeanyDocument *doc);
 
-void treeviews_remove_document(gint idx);
+void treeviews_remove_document(GeanyDocument *doc);
 
 #endif
