@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.h 2468 2008-04-10 18:18:34Z eht16 $
+ * $Id: utils.h 2803 2008-07-23 11:46:02Z ntrel $
  */
 
 /**
@@ -54,23 +54,11 @@ gboolean utils_isbrace(gchar c, gboolean include_angles);
 
 gboolean utils_is_opening_brace(gchar c, gboolean include_angles);
 
-gboolean utils_goto_file_pos(const gchar *file, gboolean is_tm_filename, gint pos);
-
-gboolean utils_goto_line(gint idx, gint line);
-
-gboolean utils_goto_pos(gint idx, gint pos);
-
 gint utils_write_file(const gchar *filename, const gchar *text);
 
 gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag);
 
-gboolean utils_check_disk_status(gint idx, gboolean force);
-
-gint utils_get_current_function(gint idx, const gchar **tagname);
-
-gint utils_get_eol_char_len(gint idx);
-
-const gchar *utils_get_eol_char(gint idx);
+const gchar *utils_get_eol_name(gint eol_mode);
 
 gboolean utils_atob(const gchar *str);
 
@@ -104,10 +92,6 @@ gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gch
 
 gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gchar *key, const gchar *default_value);
 
-void utils_switch_document(gint direction);
-
-void utils_replace_filename(gint idx);
-
 gchar *utils_get_hex_from_color(GdkColor *color);
 
 const gchar *utils_get_default_dir_utf8(void);
