@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: vte.c 2430 2008-03-31 13:09:56Z ntrel $
+ * $Id: vte.c 2997 2008-09-25 17:04:43Z eht16 $
  */
 
 /*
@@ -35,12 +35,12 @@
 #include <errno.h>
 
 #include "vte.h"
-#include "msgwindow.h"
 #include "support.h"
 #include "prefs.h"
 #include "ui_utils.h"
 #include "utils.h"
 #include "document.h"
+#include "msgwindow.h"
 #include "callbacks.h"
 #include "geanywraplabel.h"
 
@@ -49,7 +49,7 @@ VteInfo vte_info;
 VteConfig *vc;
 
 extern gchar **environ;
-static pid_t pid;
+static pid_t pid = 0;
 static gboolean clean = TRUE;
 static GModule *module = NULL;
 static struct VteFunctions *vf;
@@ -61,11 +61,58 @@ static gint vte_prefs_tab_num = -1;
 static const gchar VTE_WORDCHARS[] = "-A-Za-z0-9,./?%&#:_";
 
 
+/* taken from original vte.h to make my life easier ;-) */
+
+typedef struct _VteTerminalPrivate VteTerminalPrivate;
+
+typedef struct _VteTerminal VteTerminal;
+struct _VteTerminal
+{
+	GtkWidget widget;
+	GtkAdjustment *adjustment;
+	glong char_width, char_height;
+	glong char_ascent, char_descent;
+	glong row_count, column_count;
+	gchar *window_title;
+	gchar *icon_title;
+	VteTerminalPrivate *pvt;
+};
+
 #define VTE_TERMINAL(obj) (GTK_CHECK_CAST((obj), VTE_TYPE_TERMINAL, VteTerminal))
 #define VTE_TYPE_TERMINAL (vf->vte_terminal_get_type())
 
+
+/* store function pointers in a struct to avoid a strange segfault if they are stored directly
+ * if accessed directly, gdb says the segfault arrives at old_tab_width(prefs.c), don't ask me */
+struct VteFunctions
+{
+	GtkWidget* (*vte_terminal_new) (void);
+	pid_t (*vte_terminal_fork_command) (VteTerminal *terminal, const char *command, char **argv,
+										char **envv, const char *directory, gboolean lastlog,
+										gboolean utmp, gboolean wtmp);
+	void (*vte_terminal_set_size) (VteTerminal *terminal, glong columns, glong rows);
+	void (*vte_terminal_set_word_chars) (VteTerminal *terminal, const char *spec);
+	void (*vte_terminal_set_mouse_autohide) (VteTerminal *terminal, gboolean setting);
+	void (*vte_terminal_reset) (VteTerminal *terminal, gboolean full, gboolean clear_history);
+	GtkType (*vte_terminal_get_type) (void);
+	void (*vte_terminal_set_scroll_on_output) (VteTerminal *terminal, gboolean scroll);
+	void (*vte_terminal_set_scroll_on_keystroke) (VteTerminal *terminal, gboolean scroll);
+	void (*vte_terminal_set_font_from_string) (VteTerminal *terminal, const char *name);
+	void (*vte_terminal_set_scrollback_lines) (VteTerminal *terminal, glong lines);
+	gboolean (*vte_terminal_get_has_selection) (VteTerminal *terminal);
+	void (*vte_terminal_copy_clipboard) (VteTerminal *terminal);
+	void (*vte_terminal_paste_clipboard) (VteTerminal *terminal);
+	void (*vte_terminal_set_emulation) (VteTerminal *terminal, const gchar *emulation);
+	void (*vte_terminal_set_color_foreground) (VteTerminal *terminal, const GdkColor *foreground);
+	void (*vte_terminal_set_color_background) (VteTerminal *terminal, const GdkColor *background);
+	void (*vte_terminal_feed_child) (VteTerminal *terminal, const char *data, glong length);
+	void (*vte_terminal_im_append_menuitems) (VteTerminal *terminal, GtkMenuShell *menushell);
+};
+
+
 static void create_vte(void);
 static void vte_start(GtkWidget *widget);
+static void vte_restart(GtkWidget *widget);
 static gboolean vte_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 static gboolean vte_keyrelease(GtkWidget *widget, GdkEventKey *event, gpointer data);
 static gboolean vte_keypress(GtkWidget *widget, GdkEventKey *event, gpointer data);
@@ -211,8 +258,7 @@ static void create_vte(void)
 {
 	GtkWidget *vte, *scrollbar, *hbox, *frame;
 
-	vte = vf->vte_terminal_new();
-	vc->vte = vte;
+	vc->vte = vte = vf->vte_terminal_new();
 	scrollbar = gtk_vscrollbar_new(GTK_ADJUSTMENT(VTE_TERMINAL(vte)->adjustment));
 	GTK_WIDGET_UNSET_FLAGS(scrollbar, GTK_CAN_FOCUS);
 
@@ -234,13 +280,13 @@ static void create_vte(void)
     gtk_drag_dest_set(vte, GTK_DEST_DEFAULT_ALL,
 		dnd_targets, G_N_ELEMENTS(dnd_targets), GDK_ACTION_COPY);
 
-	g_signal_connect(G_OBJECT(vte), "child-exited", G_CALLBACK(vte_start), NULL);
-	g_signal_connect(G_OBJECT(vte), "button-press-event", G_CALLBACK(vte_button_pressed), NULL);
-	g_signal_connect(G_OBJECT(vte), "event", G_CALLBACK(vte_keypress), NULL);
-	g_signal_connect(G_OBJECT(vte), "key-release-event", G_CALLBACK(vte_keyrelease), NULL);
-	g_signal_connect(G_OBJECT(vte), "commit", G_CALLBACK(vte_commit), NULL);
-	g_signal_connect(G_OBJECT(vte), "motion-notify-event", G_CALLBACK(on_motion_event), NULL);
-	g_signal_connect(G_OBJECT(vte), "drag-data-received", G_CALLBACK(vte_drag_data_received), NULL);
+	g_signal_connect(vte, "child-exited", G_CALLBACK(vte_start), NULL);
+	g_signal_connect(vte, "button-press-event", G_CALLBACK(vte_button_pressed), NULL);
+	g_signal_connect(vte, "event", G_CALLBACK(vte_keypress), NULL);
+	g_signal_connect(vte, "key-release-event", G_CALLBACK(vte_keyrelease), NULL);
+	g_signal_connect(vte, "commit", G_CALLBACK(vte_commit), NULL);
+	g_signal_connect(vte, "motion-notify-event", G_CALLBACK(on_motion_event), NULL);
+	g_signal_connect(vte, "drag-data-received", G_CALLBACK(vte_drag_data_received), NULL);
 
 	vte_start(vte);
 
@@ -248,7 +294,7 @@ static void create_vte(void)
 	gtk_notebook_insert_page(GTK_NOTEBOOK(msgwindow.notebook), frame, gtk_label_new(_("Terminal")), MSG_VTE);
 
 	/* the vte widget has to be realised before color changes take effect */
-	g_signal_connect(G_OBJECT(vte), "realize", G_CALLBACK(vte_apply_user_settings), NULL);
+	g_signal_connect_after(vte, "realize", G_CALLBACK(vte_apply_user_settings), NULL);
 }
 
 
@@ -279,7 +325,8 @@ static gboolean vte_keyrelease(GtkWidget *widget, GdkEventKey *event, gpointer d
 			 event->keyval == GDK_KP_Enter ||
 			 ((event->keyval == GDK_c) && (event->state & GDK_CONTROL_MASK)))
 	{
-		clean = TRUE; /* assume any text on the prompt has been executed when pressing Enter/Return */
+		/* assume any text on the prompt has been executed when pressing Enter/Return */
+		clean = TRUE;
 	}
 	return FALSE;
 }
@@ -300,16 +347,7 @@ static gboolean vte_keypress(GtkWidget *widget, GdkEventKey *event, gpointer dat
 		event->state & GDK_CONTROL_MASK &&
 		! (event->state & GDK_SHIFT_MASK) && ! (event->state & GDK_MOD1_MASK))
 	{
-		vte_get_working_directory(); /* try to keep the working directory when restarting the VTE */
-
-		if (pid > 0)
-		{
-			kill(pid, SIGINT);
-			pid = 0;
-		}
-		vf->vte_terminal_reset(VTE_TERMINAL(widget), TRUE, TRUE);
-		vte_start(widget);
-
+		vte_restart(widget);
 		return TRUE;
 	}
 	return FALSE;
@@ -324,7 +362,6 @@ void vte_commit(VteTerminal *vte, gchar *arg1, guint arg2, gpointer user_data)
 
 static void vte_start(GtkWidget *widget)
 {
-	VteTerminal *vte = VTE_TERMINAL(widget);
 	gchar **env;
 	gchar **argv;
 
@@ -334,7 +371,7 @@ static void vte_start(GtkWidget *widget)
 	if (argv != NULL)
 	{
 		env = vte_get_child_environment();
-		pid = vf->vte_terminal_fork_command(VTE_TERMINAL(vte), argv[0], argv, env,
+		pid = vf->vte_terminal_fork_command(VTE_TERMINAL(widget), argv[0], argv, env,
 												vte_info.dir, TRUE, TRUE, TRUE);
 		g_strfreev(env);
 		g_strfreev(argv);
@@ -348,14 +385,14 @@ static void vte_start(GtkWidget *widget)
 
 static void vte_restart(GtkWidget *widget)
 {
-		vte_get_working_directory(); /* try to keep the working directory when restarting the VTE */
-	 	if (pid > 0)
-		{
-			kill(pid, SIGINT);
-			pid = 0;
-		}
-		vf->vte_terminal_reset(VTE_TERMINAL(widget), TRUE, TRUE);
-		vte_start(widget);
+	vte_get_working_directory(); /* try to keep the working directory when restarting the VTE */
+	if (pid > 0)
+	{
+		kill(pid, SIGINT);
+		pid = 0;
+	}
+	vf->vte_terminal_reset(VTE_TERMINAL(widget), TRUE, TRUE);
+	clean = TRUE;
 }
 
 
@@ -404,8 +441,9 @@ static void vte_register_symbols(GModule *mod)
 
 void vte_apply_user_settings(void)
 {
-	if (! ui_prefs.msgwindow_visible) return;
-	/*if (! GTK_WIDGET_REALIZED(vc->vte)) gtk_widget_realize(vc->vte);*/
+	if (! ui_prefs.msgwindow_visible)
+		return;
+
 	vf->vte_terminal_set_scrollback_lines(VTE_TERMINAL(vc->vte), vc->scrollback_lines);
 	vf->vte_terminal_set_scroll_on_keystroke(VTE_TERMINAL(vc->vte), vc->scroll_on_key);
 	vf->vte_terminal_set_scroll_on_output(VTE_TERMINAL(vc->vte), vc->scroll_on_out);
@@ -435,9 +473,9 @@ static void vte_popup_menu_clicked(GtkMenuItem *menuitem, gpointer user_data)
 		}
 		case POPUP_CHANGEPATH:
 		{
-			gint idx = document_get_cur_idx();
-			if (DOC_IDX_VALID(idx))
-				vte_cwd(doc_list[idx].file_name, TRUE);
+			GeanyDocument *doc = document_get_current();
+			if (doc != NULL)
+				vte_cwd(doc->file_name, TRUE);
 			break;
 		}
 		case POPUP_RESTARTTERMINAL:
@@ -469,12 +507,12 @@ static GtkWidget *vte_create_popup_menu(void)
 	item = gtk_image_menu_item_new_from_stock("gtk-copy", NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_COPY));
+	g_signal_connect(item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_COPY));
 
 	item = gtk_image_menu_item_new_from_stock("gtk-paste", NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_PASTE));
+	g_signal_connect(item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_PASTE));
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -483,12 +521,12 @@ static GtkWidget *vte_create_popup_menu(void)
 	item = gtk_image_menu_item_new_with_mnemonic(_("_Set Path From Document"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_CHANGEPATH));
+	g_signal_connect(item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_CHANGEPATH));
 
 	item = gtk_image_menu_item_new_with_mnemonic(_("_Restart Terminal"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_RESTARTTERMINAL));
+	g_signal_connect(item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_RESTARTTERMINAL));
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -509,7 +547,7 @@ static GtkWidget *vte_create_popup_menu(void)
 	item = gtk_image_menu_item_new_from_stock("gtk-preferences", NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_PREFERENCES));
+	g_signal_connect(item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_PREFERENCES));
 
 	msgwin_menu_add_common_items(GTK_MENU(menu));
 
@@ -649,6 +687,7 @@ void vte_append_preferences_tab(void)
 		notebook = lookup_widget(ui_widgets.prefs_dialog, "notebook2");
 
 		frame = ui_frame_new_with_alignment(_("Terminal plugin"), &alignment);
+		gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
 		vbox = gtk_vbox_new(FALSE, 12);
 		gtk_container_add(GTK_CONTAINER(alignment), vbox);
 
@@ -710,7 +749,7 @@ void vte_append_preferences_tab(void)
 					(GtkAttachOptions) (0), 0, 0);
 		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
-		spin_scrollback_adj = gtk_adjustment_new(500, 0, 5000, 1, 10, 10);
+		spin_scrollback_adj = gtk_adjustment_new(500, 0, 5000, 1, 10, 0);
 		spin_scrollback = gtk_spin_button_new(GTK_ADJUSTMENT(spin_scrollback_adj), 1, 0);
 		gtk_table_attach(GTK_TABLE(table), spin_scrollback, 1, 2, 3, 4,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
@@ -757,42 +796,35 @@ void vte_append_preferences_tab(void)
 		box = gtk_vbox_new(FALSE, 3);
 		check_scroll_key = gtk_check_button_new_with_mnemonic(_("Scroll on keystroke"));
 		gtk_tooltips_set_tip(tooltips, check_scroll_key, _("Whether to scroll to the bottom if a key was pressed."), NULL);
-		gtk_button_set_focus_on_click(GTK_BUTTON(check_scroll_key), FALSE);
 		gtk_container_add(GTK_CONTAINER(box), check_scroll_key);
 
 		check_scroll_out = gtk_check_button_new_with_mnemonic(_("Scroll on output"));
 		gtk_tooltips_set_tip(tooltips, check_scroll_out, _("Whether to scroll to the bottom when output is generated."), NULL);
-		gtk_button_set_focus_on_click(GTK_BUTTON(check_scroll_out), FALSE);
 		gtk_container_add(GTK_CONTAINER(box), check_scroll_out);
 
 		check_enable_bash_keys = gtk_check_button_new_with_mnemonic(_("Override Geany keybindings"));
 		gtk_tooltips_set_tip(tooltips, check_enable_bash_keys,
 			_("Allows the VTE to receive keyboard shortcuts (apart from focus commands)."), NULL);
-		gtk_button_set_focus_on_click(GTK_BUTTON(check_enable_bash_keys), FALSE);
 		gtk_container_add(GTK_CONTAINER(box), check_enable_bash_keys);
 
 		check_ignore_menu_key = gtk_check_button_new_with_mnemonic(_("Disable menu shortcut key (F10 by default)"));
 		gtk_tooltips_set_tip(tooltips, check_ignore_menu_key, _("This option disables the keybinding to popup the menu bar (default is F10). Disabling it can be useful if you use, for example, Midnight Commander within the VTE."), NULL);
-		gtk_button_set_focus_on_click(GTK_BUTTON(check_ignore_menu_key), FALSE);
 		gtk_container_add(GTK_CONTAINER(box), check_ignore_menu_key);
 
 		check_follow_path = gtk_check_button_new_with_mnemonic(_("Follow the path of the current file"));
 		gtk_tooltips_set_tip(tooltips, check_follow_path, _("Whether to execute \"cd $path\" when you switch between opened files."), NULL);
-		gtk_button_set_focus_on_click(GTK_BUTTON(check_follow_path), FALSE);
 		gtk_container_add(GTK_CONTAINER(box), check_follow_path);
 
 		/* create check_skip_script checkbox before the check_skip_script checkbox to be able to
 		 * use the object for the toggled handler of check_skip_script checkbox */
 		check_skip_script = gtk_check_button_new_with_mnemonic(_("Don't use run script"));
 		gtk_tooltips_set_tip(tooltips, check_skip_script, _("Don't use the simple run script which is usually used to display the exit status of the executed program."), NULL);
-		gtk_button_set_focus_on_click(GTK_BUTTON(check_skip_script), FALSE);
 		gtk_widget_set_sensitive(check_skip_script, vc->run_in_vte);
 
 		check_run_in_vte = gtk_check_button_new_with_mnemonic(_("Execute programs in VTE"));
 		gtk_tooltips_set_tip(tooltips, check_run_in_vte, _("Run programs in VTE instead of opening a terminal emulation window. Please note, programs executed in VTE cannot be stopped."), NULL);
-		gtk_button_set_focus_on_click(GTK_BUTTON(check_run_in_vte), FALSE);
 		gtk_container_add(GTK_CONTAINER(box), check_run_in_vte);
-		g_signal_connect((gpointer) check_run_in_vte, "toggled",
+		g_signal_connect(check_run_in_vte, "toggled",
 			G_CALLBACK(check_run_in_vte_toggled), check_skip_script);
 
 		/* now add the check_skip_script checkbox after the check_run_in_vte checkbox */
@@ -829,13 +861,13 @@ void vte_append_preferences_tab(void)
 
 		gtk_widget_show_all(frame);
 
-		g_signal_connect((gpointer) font_term, "font-set", G_CALLBACK(on_prefs_font_choosed),
-														   GINT_TO_POINTER(4));
-		g_signal_connect((gpointer) color_fore, "color-set", G_CALLBACK(on_prefs_color_choosed),
-															 GINT_TO_POINTER(2));
-		g_signal_connect((gpointer) color_back, "color-set", G_CALLBACK(on_prefs_color_choosed),
-															 GINT_TO_POINTER(3));
-		g_signal_connect((gpointer) button_shell, "clicked",
+		g_signal_connect(font_term, "font-set", G_CALLBACK(on_prefs_font_choosed),
+															GINT_TO_POINTER(4));
+		g_signal_connect(color_fore, "color-set", G_CALLBACK(on_prefs_color_choosed),
+															GINT_TO_POINTER(2));
+		g_signal_connect(color_back, "color-set", G_CALLBACK(on_prefs_color_choosed),
+															GINT_TO_POINTER(3));
+		g_signal_connect(button_shell, "clicked",
 				G_CALLBACK(on_prefs_tools_button_clicked), entry_shell);
 	}
 }
