@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: win32.c 2456 2008-04-08 15:00:06Z eht16 $
+ *  $Id: win32.c 3097 2008-10-15 14:14:31Z eht16 $
  */
 
 /*
@@ -53,6 +53,8 @@
 #include "sciwrappers.h"
 #include "dialogs.h"
 #include "filetypes.h"
+#include "project.h"
+#include "editor.h"
 
 #define BUFSIZE 4096
 
@@ -87,14 +89,14 @@ static gchar *get_file_filters()
 	GString *all_patterns = g_string_sized_new(100);
 	gchar *tmp;
 
-	for (i = 0; i < GEANY_MAX_FILE_TYPES; i++)
+	for (i = 0; i < filetypes_array->len; i++)
 	{
 		tmp = g_strjoinv(";", filetypes[i]->pattern);
 		g_string_append_printf(str, "%s\t%s\t", filetypes[i]->title, tmp);
 		g_free(tmp);
 	}
 	/* create meta file filter "All Source" */
-	for (i = 0; i < GEANY_MAX_FILE_TYPES; i++)
+	for (i = 0; i < filetypes_array->len; i++)
 	{
 		for (j = 0; filetypes[i]->pattern[j] != NULL; j++)
 		{
@@ -127,14 +129,14 @@ static gchar *get_filters(gboolean project_files)
 	if (project_files)
 	{
 		string = g_strconcat(_("Geany project files"), "\t", "*." GEANY_PROJECT_EXT, "\t",
-			filetypes[GEANY_FILETYPES_ALL]->title, "\t",
-			filetypes[GEANY_FILETYPES_ALL]->pattern[0], "\t", NULL);
+			filetypes[GEANY_FILETYPES_NONE]->title, "\t",
+			filetypes[GEANY_FILETYPES_NONE]->pattern[0], "\t", NULL);
 	}
 	else
 	{
 		string = g_strconcat(_("Executables"), "\t", "*.exe;*.bat;*.cmd", "\t",
-			filetypes[GEANY_FILETYPES_ALL]->title, "\t",
-			filetypes[GEANY_FILETYPES_ALL]->pattern[0], "\t", NULL);
+			filetypes[GEANY_FILETYPES_NONE]->title, "\t",
+			filetypes[GEANY_FILETYPES_NONE]->pattern[0], "\t", NULL);
 	}
 
 	/* replace all "\t"s by \0 */
@@ -209,7 +211,7 @@ gchar *win32_show_project_folder_dialog(GtkWidget *parent, const gchar *title,
 	gchar *result = NULL;
 
 	if (parent == NULL)
-		parent = app->window;
+		parent = main_widgets.window;
 
 	memset(&bi, 0, sizeof bi);
 	bi.hwndOwner = GDK_WINDOW_HWND(parent->window);
@@ -262,7 +264,7 @@ gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
 	fname[0] = '\0';
 
 	if (parent == NULL)
-		parent = app->window;
+		parent = main_widgets.window;
 
 	/* initialise file dialog info struct */
 	memset(&of, 0, sizeof of);
@@ -327,11 +329,11 @@ gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(app->window->window);
+	of.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
 	of.lpstrFilter = filters;
 
 	of.lpstrCustomFilter = NULL;
-	of.nFilterIndex = GEANY_FILETYPES_ALL + 1;
+	of.nFilterIndex = GEANY_FILETYPES_NONE + 1;
 	of.lpstrFile = fname;
 	of.lpstrInitialDir = initial_dir;
 	of.nMaxFile = 2048;
@@ -379,8 +381,6 @@ gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 		}
 		else
 		{	/* open multiple files */
-			document_delay_colourise();
-
 			for (; ;)
 			{
 				if (! fname[x])
@@ -397,15 +397,16 @@ gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 				}
 				x++;
 			}
-			document_colourise_new();
 		}
 	}
 	else
 	{
-		gint idx = document_get_cur_idx();
+		GeanyDocument *doc = document_get_current();
 		/* convert the resulting filename into UTF-8 */
-		doc_list[idx].file_name = g_locale_to_utf8(fname, -1, NULL, NULL, NULL);
-		document_save_file_as(idx);
+		gchar *utf8 = g_locale_to_utf8(fname, -1, NULL, NULL, NULL);
+
+		document_save_file_as(doc, utf8);
+		g_free(utf8);
 	}
 	g_free(fname);
 	return (retval != 0);
@@ -420,7 +421,7 @@ void win32_show_font_dialog(void)
 
 	memset(&cf, 0, sizeof cf);
 	cf.lStructSize = sizeof cf;
-	cf.hwndOwner = GDK_WINDOW_HWND(app->window->window);
+	cf.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
 	cf.lpLogFont = &lf;
 	cf.Flags = CF_APPLY | CF_NOSCRIPTSEL | CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
 
@@ -451,12 +452,12 @@ void win32_show_color_dialog(const gchar *colour)
 	static COLORREF acr_cust_clr[16];
 	static DWORD rgb_current;
 	gchar *hex = g_malloc0(12);
-	gint idx = document_get_cur_idx();
+	GeanyDocument *doc = document_get_current();
 
 	/* Initialize CHOOSECOLOR */
 	memset(&cc, 0, sizeof cc);
 	cc.lStructSize = sizeof(cc);
-	cc.hwndOwner = GDK_WINDOW_HWND(app->window->window);
+	cc.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
 	cc.lpCustColors = (LPDWORD) acr_cust_clr;
 	cc.rgbResult = (colour != NULL) ? utils_strtod(colour, NULL, colour[0] == '#') : 0;
 	cc.Flags = CC_FULLOPEN | CC_RGBINIT;
@@ -469,7 +470,7 @@ void win32_show_color_dialog(const gchar *colour)
 	      (guint) (utils_scale_round(GetGValue(rgb_current), 255)),
 	      (guint) (utils_scale_round(GetBValue(rgb_current), 255)));
 
-		document_insert_colour(idx, hex);
+		editor_insert_color(doc->editor, hex);
 	}
 	g_free(hex);
 }
@@ -601,8 +602,8 @@ gboolean win32_message_dialog(GtkWidget *parent, GtkMessageType type, const gcha
 
 	if (parent != NULL)
 		parent_hwnd = GDK_WINDOW_HWND(parent->window);
-	else if (app->window != NULL)
-		parent_hwnd = GDK_WINDOW_HWND(app->window->window);
+	else if (main_widgets.window != NULL)
+		parent_hwnd = GDK_WINDOW_HWND(main_widgets.window->window);
 
 	/* display the message box */
 	rc = MessageBoxW(parent_hwnd, w_msg, w_title, t);
@@ -638,8 +639,8 @@ gint win32_message_dialog_unsaved(const gchar *msg)
 	MultiByteToWideChar(CP_UTF8, 0, msg, -1, w_msg, G_N_ELEMENTS(w_msg));
 	MultiByteToWideChar(CP_UTF8, 0, _("Question"), -1, w_title, G_N_ELEMENTS(w_title));
 
-	if (app->window != NULL)
-		parent_hwnd = GDK_WINDOW_HWND(app->window->window);
+	if (main_widgets.window != NULL)
+		parent_hwnd = GDK_WINDOW_HWND(main_widgets.window->window);
 
 	ret = MessageBoxW(parent_hwnd, w_msg, w_title, MB_YESNOCANCEL | MB_ICONQUESTION);
 	switch(ret)
@@ -702,27 +703,15 @@ static void debug_setup_console()
 }
 
 
-static void debug_log_handler(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message,
-					   gpointer user_data)
-{
-	if (log_domain != NULL)
-		fprintf(stderr, "%s: %s\n", log_domain, message);
-	else
-		fprintf(stderr, "%s\n", message);
-}
-
-
 void win32_init_debug_code()
 {
-#ifndef GEANY_DEBUG
 	if (app->debug_mode)
-#endif
-	{	/* create a console window to get log messages on Windows */
+	{
+		/* create a console window to get log messages on Windows */
+		/** TODO remove me? */
 		debug_setup_console();
-		/* change the log handlers to output log messages in ther created console window */
-		g_log_set_handler("GLib",
-			G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, debug_log_handler, NULL);
-		g_log_set_default_handler(debug_log_handler, NULL);
+		/* Enable GLib process spawn debug mode when Geany was started with the debug flag */
+		g_setenv("G_SPAWN_WIN32_DEBUG", "1", FALSE);
 	}
 }
 
@@ -1170,7 +1159,7 @@ static gboolean resolve_link(HWND hWnd, wchar_t *link, gchar **lpszPath)
 		 * Get an interface pointer to it. */
 		pslW = (IShellLinkW*) pslWV;
 		hres = pslW->lpVtbl->QueryInterface(pslW, &IID_IPersistFile, &ppfV);
-	}     
+	}
 
 	if (SUCCEEDED(hres))
 	{
@@ -1212,9 +1201,9 @@ gchar *win32_get_shortcut_target(const gchar *file_name)
 	gchar *path = NULL;
 	wchar_t *wfilename = g_utf8_to_utf16(file_name, -1, NULL, NULL, NULL);
 
-	resolve_link(GDK_WINDOW_HWND(app->window->window), wfilename, &path);
+	resolve_link(GDK_WINDOW_HWND(main_widgets.window->window), wfilename, &path);
 	g_free(wfilename);
-	
+
 	if (path == NULL)
 		return g_strdup(file_name);
 	else
