@@ -79,9 +79,10 @@ TMWorkObject *tm_workspace_find_object(TMWorkObject *work_object, const char *fi
 /*! Removes a member object from the workspace if it exists.
  \param work_object Pointer to the work object to be removed.
  \param free Whether the work object is to be freed as well.
+ \param update Whether to update workspace objects.
  \return TRUE on success, FALSE on failure (e.g. the work object does not exist).
 */
-gboolean tm_workspace_remove_object(TMWorkObject *work_object, gboolean free);
+gboolean tm_workspace_remove_object(TMWorkObject *work_object, gboolean free, gboolean update);
 
 /*! Loads the global tag list from the specified file. The global tag list should
  have been first created using tm_workspace_create_global_tags().
@@ -90,7 +91,7 @@ gboolean tm_workspace_remove_object(TMWorkObject *work_object, gboolean free);
  \sa tm_workspace_create_global_tags()
 */
 gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode);
-//gboolean tm_workspace_load_global_tags(const char *tags_file);
+/*gboolean tm_workspace_load_global_tags(const char *tags_file);*/
 
 /*! Creates a list of global tags. Ideally, this should be created once during
  installations so that all users can use the same file. Thsi is because a full
@@ -103,8 +104,8 @@ gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode);
  \param lang The language to use for the tags file.
  \return TRUE on success, FALSE on failure.
 */
-gboolean tm_workspace_create_global_tags(const char *pre_process, const char **includes
-  , int includes_count, const char *tags_file, int lang);
+gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre_process,
+	const char **includes, int includes_count, const char *tags_file, int lang);
 
 /*! Recreates the tag array of the workspace by collecting the tags of
  all member work objects. You shouldn't have to call this directly since
