@@ -44,7 +44,9 @@
     FreeBasicParser,\
     HaxeParser,\
     RestParser, \
-    HtmlParser
+    HtmlParser, \
+    F77Parser, \
+    GLSLParser
 
 /*
 langType of each parser
@@ -78,6 +80,8 @@ langType of each parser
 27  HaxeParser
 28  RestParser
 29  HtmlParser
+30  F77Parser
+31  GLSLParser
 */
 #endif	/* _PARSERS_H */
 
